# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: contrib/envoy/extensions/filters/network/generic_proxy/router/v3/router.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nMcontrib/envoy/extensions/filters/network/generic_proxy/router/v3/router.proto\x12\x38\x65nvoy.extensions.filters.network.generic_proxy.router.v3\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"*\n\x06Router\x12 \n\x18\x62ind_upstream_connection\x18\x01 \x01(\x08\x42\xd9\x01\nFio.envoyproxy.envoy.extensions.filters.network.generic_proxy.router.v3B\x0bRouterProtoP\x01Zpgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/generic_proxy/router/v3;routerv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.generic_proxy.router.v3.router_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\nFio.envoyproxy.envoy.extensions.filters.network.generic_proxy.router.v3B\013RouterProtoP\001Zpgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/generic_proxy/router/v3;routerv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_ROUTER']._serialized_start=203
  _globals['_ROUTER']._serialized_end=245
# @@protoc_insertion_point(module_scope)
