/* This file was generated by upb_generator from the input file:
 *
 *     xds/type/v3/range.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "xds/type/v3/range.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const UPB_PRIVATE(upb_GeneratedExtensionListEntry)* UPB_PRIVATE(upb_generated_extension_list);
typedef struct {
  upb_MiniTableField fields[2];
} xds__type__v3__Int64Range_msg_init_Fields;

static const xds__type__v3__Int64Range_msg_init_Fields xds_type_v3_Int64Range__fields = {{
  {1, 8, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, 16, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable xds__type__v3__Int64Range_msg_init = {
  &xds_type_v3_Int64Range__fields.fields[0],
  24, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.v3.Int64Range",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_DecodeFast_Varint64_Scalar_Tag1Byte},
    {0x001000003f000010, &upb_DecodeFast_Varint64_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
} xds__type__v3__Int32Range_msg_init_Fields;

static const xds__type__v3__Int32Range_msg_init_Fields xds_type_v3_Int32Range__fields = {{
  {1, 8, 0, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {2, 12, 0, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable xds__type__v3__Int32Range_msg_init = {
  &xds_type_v3_Int32Range__fields.fields[0],
  16, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.v3.Int32Range",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_DecodeFast_Varint32_Scalar_Tag1Byte},
    {0x000c00003f000010, &upb_DecodeFast_Varint32_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
} xds__type__v3__DoubleRange_msg_init_Fields;

static const xds__type__v3__DoubleRange_msg_init_Fields xds_type_v3_DoubleRange__fields = {{
  {1, 8, 0, kUpb_NoSub, 1, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, 16, 0, kUpb_NoSub, 1, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable xds__type__v3__DoubleRange_msg_init = {
  &xds_type_v3_DoubleRange__fields.fields[0],
  24, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.v3.DoubleRange",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000009, &upb_DecodeFast_Fixed64_Scalar_Tag1Byte},
    {0x001000003f000011, &upb_DecodeFast_Fixed64_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

static const upb_MiniTable *messages_layout[3] = {
  &xds__type__v3__Int64Range_msg_init,
  &xds__type__v3__Int32Range_msg_init,
  &xds__type__v3__DoubleRange_msg_init,
};

const upb_MiniTableFile xds_type_v3_range_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  3,
  0,
  0,
};

#include "upb/port/undef.inc"

