# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cel/expr/conformance/conformance_service.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'cel/expr/conformance/conformance_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cel.expr import checked_pb2 as cel_dot_expr_dot_checked__pb2
from cel.expr import eval_pb2 as cel_dot_expr_dot_eval__pb2
from cel.expr import syntax_pb2 as cel_dot_expr_dot_syntax__pb2
from google.rpc import status_pb2 as google_dot_rpc_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.cel/expr/conformance/conformance_service.proto\x12\x14\x63\x65l.expr.conformance\x1a\x16\x63\x65l/expr/checked.proto\x1a\x13\x63\x65l/expr/eval.proto\x1a\x15\x63\x65l/expr/syntax.proto\x1a\x17google/rpc/status.proto\"k\n\x0cParseRequest\x12\x12\n\ncel_source\x18\x01 \x01(\t\x12\x16\n\x0esyntax_version\x18\x02 \x01(\t\x12\x17\n\x0fsource_location\x18\x03 \x01(\t\x12\x16\n\x0e\x64isable_macros\x18\x04 \x01(\x08\"^\n\rParseResponse\x12)\n\x0bparsed_expr\x18\x01 \x01(\x0b\x32\x14.cel.expr.ParsedExpr\x12\"\n\x06issues\x18\x02 \x03(\x0b\x32\x12.google.rpc.Status\"\x82\x01\n\x0c\x43heckRequest\x12)\n\x0bparsed_expr\x18\x01 \x01(\x0b\x32\x14.cel.expr.ParsedExpr\x12 \n\x08type_env\x18\x02 \x03(\x0b\x32\x0e.cel.expr.Decl\x12\x11\n\tcontainer\x18\x03 \x01(\t\x12\x12\n\nno_std_env\x18\x04 \x01(\x08\"`\n\rCheckResponse\x12+\n\x0c\x63hecked_expr\x18\x01 \x01(\x0b\x32\x15.cel.expr.CheckedExpr\x12\"\n\x06issues\x18\x02 \x03(\x0b\x32\x12.google.rpc.Status\"\x92\x02\n\x0b\x45valRequest\x12+\n\x0bparsed_expr\x18\x01 \x01(\x0b\x32\x14.cel.expr.ParsedExprH\x00\x12-\n\x0c\x63hecked_expr\x18\x02 \x01(\x0b\x32\x15.cel.expr.CheckedExprH\x00\x12\x41\n\x08\x62indings\x18\x03 \x03(\x0b\x32/.cel.expr.conformance.EvalRequest.BindingsEntry\x12\x11\n\tcontainer\x18\x04 \x01(\t\x1a\x44\n\rBindingsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.cel.expr.ExprValue:\x02\x38\x01\x42\x0b\n\texpr_kind\"W\n\x0c\x45valResponse\x12#\n\x06result\x18\x01 \x01(\x0b\x32\x13.cel.expr.ExprValue\x12\"\n\x06issues\x18\x02 \x03(\x0b\x32\x12.google.rpc.Status\"P\n\x0eSourcePosition\x12\x10\n\x08location\x18\x01 \x01(\t\x12\x0e\n\x06offset\x18\x02 \x01(\x05\x12\x0c\n\x04line\x18\x03 \x01(\x05\x12\x0e\n\x06\x63olumn\x18\x04 \x01(\x05\"\xe0\x01\n\x0cIssueDetails\x12=\n\x08severity\x18\x01 \x01(\x0e\x32+.cel.expr.conformance.IssueDetails.Severity\x12\x36\n\x08position\x18\x02 \x01(\x0b\x32$.cel.expr.conformance.SourcePosition\x12\n\n\x02id\x18\x03 \x01(\x03\"M\n\x08Severity\x12\x18\n\x14SEVERITY_UNSPECIFIED\x10\x00\x12\x0f\n\x0b\x44\x45PRECATION\x10\x01\x12\x0b\n\x07WARNING\x10\x02\x12\t\n\x05\x45RROR\x10\x03\x32\x8d\x02\n\x12\x43onformanceService\x12R\n\x05Parse\x12\".cel.expr.conformance.ParseRequest\x1a#.cel.expr.conformance.ParseResponse\"\x00\x12R\n\x05\x43heck\x12\".cel.expr.conformance.CheckRequest\x1a#.cel.expr.conformance.CheckResponse\"\x00\x12O\n\x04\x45val\x12!.cel.expr.conformance.EvalRequest\x1a\".cel.expr.conformance.EvalResponse\"\x00\x42R\n\x18\x64\x65v.cel.expr.conformanceB\x17\x43onformanceServiceProtoP\x01Z\x18\x63\x65l.dev/expr/conformance\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cel.expr.conformance.conformance_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030dev.cel.expr.conformanceB\027ConformanceServiceProtoP\001Z\030cel.dev/expr/conformance\370\001\001'
  _globals['_EVALREQUEST_BINDINGSENTRY']._loaded_options = None
  _globals['_EVALREQUEST_BINDINGSENTRY']._serialized_options = b'8\001'
  _globals['_PARSEREQUEST']._serialized_start=165
  _globals['_PARSEREQUEST']._serialized_end=272
  _globals['_PARSERESPONSE']._serialized_start=274
  _globals['_PARSERESPONSE']._serialized_end=368
  _globals['_CHECKREQUEST']._serialized_start=371
  _globals['_CHECKREQUEST']._serialized_end=501
  _globals['_CHECKRESPONSE']._serialized_start=503
  _globals['_CHECKRESPONSE']._serialized_end=599
  _globals['_EVALREQUEST']._serialized_start=602
  _globals['_EVALREQUEST']._serialized_end=876
  _globals['_EVALREQUEST_BINDINGSENTRY']._serialized_start=795
  _globals['_EVALREQUEST_BINDINGSENTRY']._serialized_end=863
  _globals['_EVALRESPONSE']._serialized_start=878
  _globals['_EVALRESPONSE']._serialized_end=965
  _globals['_SOURCEPOSITION']._serialized_start=967
  _globals['_SOURCEPOSITION']._serialized_end=1047
  _globals['_ISSUEDETAILS']._serialized_start=1050
  _globals['_ISSUEDETAILS']._serialized_end=1274
  _globals['_ISSUEDETAILS_SEVERITY']._serialized_start=1197
  _globals['_ISSUEDETAILS_SEVERITY']._serialized_end=1274
  _globals['_CONFORMANCESERVICE']._serialized_start=1277
  _globals['_CONFORMANCESERVICE']._serialized_end=1546
# @@protoc_insertion_point(module_scope)
