/* This file was generated by upb_generator from the input file:
 *
 *     google/api/expr/v1alpha1/checked.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "google/api/expr/v1alpha1/checked.upb_minitable.h"
#include "google/api/expr/v1alpha1/syntax.upb_minitable.h"
#include "google/protobuf/empty.upb_minitable.h"
#include "google/protobuf/struct.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const UPB_PRIVATE(upb_GeneratedExtensionListEntry)* UPB_PRIVATE(upb_generated_extension_list);
typedef struct {
  upb_MiniTableField fields[5];
  upb_MiniTableSubInternal subs[4];
} google__api__expr__v1alpha1__CheckedExpr_msg_init_Fields;

static const google__api__expr__v1alpha1__CheckedExpr_msg_init_Fields google_api_expr_v1alpha1_CheckedExpr__fields = {{
  {2, UPB_SIZE(12, 32), 0, UPB_SIZE(15, 16), 11, (int)kUpb_FieldMode_Map | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(16, 40), 0, UPB_SIZE(13, 15), 11, (int)kUpb_FieldMode_Map | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(20, 48), 64, UPB_SIZE(11, 14), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(24, 56), 65, UPB_SIZE(9, 13), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(28, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__CheckedExpr__ReferenceMapEntry_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__CheckedExpr__TypeMapEntry_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Expr_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__SourceInfo_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__CheckedExpr_msg_init = {
  &google_api_expr_v1alpha1_CheckedExpr__fields.fields[0],
  UPB_SIZE(40, 64), 5, kUpb_ExtMode_NonExtendable, 0, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.CheckedExpr",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000032, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} google__api__expr__v1alpha1__CheckedExpr__ReferenceMapEntry_msg_init_Fields;

static const google__api__expr__v1alpha1__CheckedExpr__ReferenceMapEntry_msg_init_Fields google_api_expr_v1alpha1_CheckedExpr_ReferenceMapEntry__fields = {{
  {1, 16, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, 32, 64, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Reference_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__CheckedExpr__ReferenceMapEntry_msg_init = {
  &google_api_expr_v1alpha1_CheckedExpr_ReferenceMapEntry__fields.fields[0],
  48, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.CheckedExpr.ReferenceMapEntry",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000008, &upb_DecodeFast_Varint64_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} google__api__expr__v1alpha1__CheckedExpr__TypeMapEntry_msg_init_Fields;

static const google__api__expr__v1alpha1__CheckedExpr__TypeMapEntry_msg_init_Fields google_api_expr_v1alpha1_CheckedExpr_TypeMapEntry__fields = {{
  {1, 16, 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, 32, 64, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__CheckedExpr__TypeMapEntry_msg_init = {
  &google_api_expr_v1alpha1_CheckedExpr_TypeMapEntry__fields.fields[0],
  48, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.CheckedExpr.TypeMapEntry",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000008, &upb_DecodeFast_Varint64_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[13];
  upb_MiniTableSubInternal subs[7];
} google__api__expr__v1alpha1__Type_msg_init_Fields;

static const google__api__expr__v1alpha1__Type_msg_init_Fields google_api_expr_v1alpha1_Type__fields = {{
  {1, UPB_SIZE(12, 16), -9, UPB_SIZE(39, 40), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), -9, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 16), -9, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(12, 16), -9, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(12, 16), -9, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(12, 16), -9, UPB_SIZE(25, 27), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {7, UPB_SIZE(12, 16), -9, UPB_SIZE(23, 26), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {8, UPB_SIZE(12, 16), -9, UPB_SIZE(21, 25), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {9, UPB_SIZE(12, 16), -9, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {10, UPB_SIZE(12, 16), -9, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {11, UPB_SIZE(12, 16), -9, UPB_SIZE(13, 18), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {12, UPB_SIZE(12, 16), -9, UPB_SIZE(11, 17), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {14, UPB_SIZE(12, 16), -9, UPB_SIZE(9, 16), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Empty_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type__ListType_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type__MapType_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type__FunctionType_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type_msg_init},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Empty_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type__AbstractType_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__Type_msg_init = {
  &google_api_expr_v1alpha1_Type__fields.fields[0],
  UPB_SIZE(24, 32), 13, kUpb_ExtMode_NonExtendable, 12, UPB_FASTTABLE_MASK(120), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.Type",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0010000802000010, &upb_DecodeFast_Varint32_Oneof_Tag1Byte},
    {0x0010000803000018, &upb_DecodeFast_Varint32_Oneof_Tag1Byte},
    {0x0010000804000020, &upb_DecodeFast_Varint32_Oneof_Tag1Byte},
    {0x0010000805000028, &upb_DecodeFast_Varint32_Oneof_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000080900004a, &upb_DecodeFast_String_Oneof_Tag1Byte},
    {0x001000080a000052, &upb_DecodeFast_String_Oneof_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[1];
  upb_MiniTableSubInternal subs[1];
} google__api__expr__v1alpha1__Type__ListType_msg_init_Fields;

static const google__api__expr__v1alpha1__Type__ListType_msg_init_Fields google_api_expr_v1alpha1_Type_ListType__fields = {{
  {1, UPB_SIZE(12, 16), 64, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__Type__ListType_msg_init = {
  &google_api_expr_v1alpha1_Type_ListType__fields.fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.Type.ListType",
#endif
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[2];
} google__api__expr__v1alpha1__Type__MapType_msg_init_Fields;

static const google__api__expr__v1alpha1__Type__MapType_msg_init_Fields google_api_expr_v1alpha1_Type_MapType__fields = {{
  {1, UPB_SIZE(12, 16), 64, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 65, UPB_SIZE(4, 5), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__Type__MapType_msg_init = {
  &google_api_expr_v1alpha1_Type_MapType__fields.fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.Type.MapType",
#endif
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[2];
} google__api__expr__v1alpha1__Type__FunctionType_msg_init_Fields;

static const google__api__expr__v1alpha1__Type__FunctionType_msg_init_Fields google_api_expr_v1alpha1_Type_FunctionType__fields = {{
  {1, UPB_SIZE(12, 16), 64, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 0, UPB_SIZE(4, 5), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__Type__FunctionType_msg_init = {
  &google_api_expr_v1alpha1_Type_FunctionType__fields.fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.Type.FunctionType",
#endif
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} google__api__expr__v1alpha1__Type__AbstractType_msg_init_Fields;

static const google__api__expr__v1alpha1__Type__AbstractType_msg_init_Fields google_api_expr_v1alpha1_Type_AbstractType__fields = {{
  {1, UPB_SIZE(12, 8), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(8, 24), 0, 3, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__Type__AbstractType_msg_init = {
  &google_api_expr_v1alpha1_Type_AbstractType__fields.fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.Type.AbstractType",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[2];
} google__api__expr__v1alpha1__Decl_msg_init_Fields;

static const google__api__expr__v1alpha1__Decl_msg_init_Fields google_api_expr_v1alpha1_Decl__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), -9, UPB_SIZE(6, 7), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 32), -9, UPB_SIZE(4, 6), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Decl__IdentDecl_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Decl__FunctionDecl_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__Decl_msg_init = {
  &google_api_expr_v1alpha1_Decl__fields.fields[0],
  UPB_SIZE(24, 40), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.Decl",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[2];
} google__api__expr__v1alpha1__Decl__IdentDecl_msg_init_Fields;

static const google__api__expr__v1alpha1__Decl__IdentDecl_msg_init_Fields google_api_expr_v1alpha1_Decl_IdentDecl__fields = {{
  {1, UPB_SIZE(12, 32), 64, UPB_SIZE(9, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 40), 65, UPB_SIZE(7, 9), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Constant_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__Decl__IdentDecl_msg_init = {
  &google_api_expr_v1alpha1_Decl_IdentDecl__fields.fields[0],
  UPB_SIZE(32, 48), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.Decl.IdentDecl",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00001a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[1];
  upb_MiniTableSubInternal subs[1];
} google__api__expr__v1alpha1__Decl__FunctionDecl_msg_init_Fields;

static const google__api__expr__v1alpha1__Decl__FunctionDecl_msg_init_Fields google_api_expr_v1alpha1_Decl_FunctionDecl__fields = {{
  {1, 8, 0, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Decl__FunctionDecl__Overload_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__Decl__FunctionDecl_msg_init = {
  &google_api_expr_v1alpha1_Decl_FunctionDecl__fields.fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.Decl.FunctionDecl",
#endif
};

typedef struct {
  upb_MiniTableField fields[6];
  upb_MiniTableSubInternal subs[2];
} google__api__expr__v1alpha1__Decl__FunctionDecl__Overload_msg_init_Fields;

static const google__api__expr__v1alpha1__Decl__FunctionDecl__Overload_msg_init_Fields google_api_expr_v1alpha1_Decl_FunctionDecl_Overload__fields = {{
  {1, UPB_SIZE(24, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 48), 0, 15, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(16, 56), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(20, 64), 64, UPB_SIZE(10, 11), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, 9, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
  {6, 32, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type_msg_init},
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Type_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__Decl__FunctionDecl__Overload_msg_init = {
  &google_api_expr_v1alpha1_Decl_FunctionDecl_Overload__fields.fields[0],
  UPB_SIZE(40, 72), 6, kUpb_ExtMode_NonExtendable, 6, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.Decl.FunctionDecl.Overload",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x003800003f00001a, &upb_DecodeFast_String_Repeated_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000900003f000028, &upb_DecodeFast_Bool_Scalar_Tag1Byte},
    {0x002000003f000032, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[1];
} google__api__expr__v1alpha1__Reference_msg_init_Fields;

static const google__api__expr__v1alpha1__Reference_msg_init_Fields google_api_expr_v1alpha1_Reference__fields = {{
  {1, UPB_SIZE(20, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 32), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(16, 40), 64, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__api__expr__v1alpha1__Constant_msg_init},
}};

const upb_MiniTable google__api__expr__v1alpha1__Reference_msg_init = {
  &google_api_expr_v1alpha1_Reference__fields.fields[0],
  UPB_SIZE(32, 48), 3, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "google.api.expr.v1alpha1.Reference",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x002000003f00001a, &upb_DecodeFast_String_Repeated_Tag1Byte},
  })
};

static const upb_MiniTable *messages_layout[13] = {
  &google__api__expr__v1alpha1__CheckedExpr_msg_init,
  &google__api__expr__v1alpha1__CheckedExpr__ReferenceMapEntry_msg_init,
  &google__api__expr__v1alpha1__CheckedExpr__TypeMapEntry_msg_init,
  &google__api__expr__v1alpha1__Type_msg_init,
  &google__api__expr__v1alpha1__Type__ListType_msg_init,
  &google__api__expr__v1alpha1__Type__MapType_msg_init,
  &google__api__expr__v1alpha1__Type__FunctionType_msg_init,
  &google__api__expr__v1alpha1__Type__AbstractType_msg_init,
  &google__api__expr__v1alpha1__Decl_msg_init,
  &google__api__expr__v1alpha1__Decl__IdentDecl_msg_init,
  &google__api__expr__v1alpha1__Decl__FunctionDecl_msg_init,
  &google__api__expr__v1alpha1__Decl__FunctionDecl__Overload_msg_init,
  &google__api__expr__v1alpha1__Reference_msg_init,
};

const upb_MiniTableFile google_api_expr_v1alpha1_checked_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  13,
  0,
  0,
};

#include "upb/port/undef.inc"

