/*
 *
 * Copyright 2019 gRPC authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#import <XCTest/XCTest.h>

#import <GRPCClient/GRPCTransport.h>
#import "src/objective-c/GRPCClient/private/GRPCTransport+Private.h"

@interface TransportRegistryTests : XCTestCase

@end

@implementation TransportRegistryTests

- (void)testDefaultImplementationsExist {
  id<GRPCTransportFactory> secureTransportFactory = [[GRPCTransportRegistry sharedInstance]
      getTransportFactoryWithID:GRPCDefaultTransportImplList.core_secure];
  id<GRPCTransportFactory> insecureTransportFactory = [[GRPCTransportRegistry sharedInstance]
      getTransportFactoryWithID:GRPCDefaultTransportImplList.core_insecure];
  XCTAssertNotNil(secureTransportFactory);
  XCTAssertNotNil(insecureTransportFactory);
  XCTAssertNotEqual(secureTransportFactory, insecureTransportFactory);
}

@end
