# frozen_string_literal: true
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/proto/grpc/testing/report_qps_scenario_service.proto

require 'google/protobuf'

require 'src/proto/grpc/testing/control_pb'


descriptor_data = "\n8src/proto/grpc/testing/report_qps_scenario_service.proto\x12\x0cgrpc.testing\x1a$src/proto/grpc/testing/control.proto2^\n\x18ReportQpsScenarioService\x12\x42\n\x0eReportScenario\x12\x1c.grpc.testing.ScenarioResult\x1a\x12.grpc.testing.Voidb\x06proto3"

pool = ::Google::Protobuf::DescriptorPool.generated_pool
pool.add_serialized_file(descriptor_data)

module Grpc
  module Testing
  end
end
