// Copyright 2023 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package hpack_sync_fuzzer;

import "test/core/test_util/fuzz_config_vars.proto";

message Empty {}

message StringKeyValue {
  string key = 1;
  string value = 2;
}

message IndexedKeyValue {
  uint32 index = 1;
  string value = 2;
}

message Header {
  oneof ty {
    uint32 indexed = 1;
    StringKeyValue literal_inc_idx = 2;
    StringKeyValue literal_not_idx = 3;
    IndexedKeyValue literal_not_idx_from_idx = 4;
  }
}

message Msg {
  bool use_true_binary_metadata = 1;
  repeated Header headers = 2;
  grpc.testing.FuzzConfigVars config_vars = 3;
  repeated uint64 random_numbers = 4;
  // Ensure that a header "a: b" appended to headers with hpack incremental
  // indexing is correctly added to the hpack table.
  bool check_ab_preservation = 5;
}
