# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/http/ext_proc/processing_request_modifiers/mapped_attribute_builder/v3/mapped_attribute_builder.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/http/ext_proc/processing_request_modifiers/mapped_attribute_builder/v3/mapped_attribute_builder.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nvenvoy/extensions/http/ext_proc/processing_request_modifiers/mapped_attribute_builder/v3/mapped_attribute_builder.proto\x12Wenvoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"\x81\x04\n\x16MappedAttributeBuilder\x12\xb0\x01\n\x19mapped_request_attributes\x18\x01 \x03(\x0b\x32\x8c\x01.envoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3.MappedAttributeBuilder.MappedRequestAttributesEntry\x12\xb2\x01\n\x1amapped_response_attributes\x18\x02 \x03(\x0b\x32\x8d\x01.envoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3.MappedAttributeBuilder.MappedResponseAttributesEntry\x1a>\n\x1cMappedRequestAttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a?\n\x1dMappedResponseAttributesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x42\xb2\x02\neio.envoyproxy.envoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3B\x1bMappedAttributeBuilderProtoP\x01Z\x99\x01github.com/envoyproxy/go-control-plane/envoy/extensions/http/ext_proc/processing_request_modifiers/mapped_attribute_builder/v3;mapped_attribute_builderv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3.mapped_attribute_builder_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\neio.envoyproxy.envoy.extensions.http.ext_proc.processing_request_modifiers.mapped_attribute_builder.v3B\033MappedAttributeBuilderProtoP\001Z\231\001github.com/envoyproxy/go-control-plane/envoy/extensions/http/ext_proc/processing_request_modifiers/mapped_attribute_builder/v3;mapped_attribute_builderv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDREQUESTATTRIBUTESENTRY']._loaded_options = None
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDREQUESTATTRIBUTESENTRY']._serialized_options = b'8\001'
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDRESPONSEATTRIBUTESENTRY']._loaded_options = None
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDRESPONSEATTRIBUTESENTRY']._serialized_options = b'8\001'
  _globals['_MAPPEDATTRIBUTEBUILDER']._serialized_start=276
  _globals['_MAPPEDATTRIBUTEBUILDER']._serialized_end=789
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDREQUESTATTRIBUTESENTRY']._serialized_start=662
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDREQUESTATTRIBUTESENTRY']._serialized_end=724
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDRESPONSEATTRIBUTESENTRY']._serialized_start=726
  _globals['_MAPPEDATTRIBUTEBUILDER_MAPPEDRESPONSEATTRIBUTESENTRY']._serialized_end=789
# @@protoc_insertion_point(module_scope)
