<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/proto/grpc/testing/proxy-service.proto

namespace Grpc\Testing;

use Google\Protobuf\Internal\GPBType;
use Google\Protobuf\Internal\RepeatedField;
use Google\Protobuf\Internal\GPBUtil;

/**
 * Generated from protobuf message <code>grpc.testing.ProxyStat</code>
 */
class ProxyStat extends \Google\Protobuf\Internal\Message
{
    /**
     * Generated from protobuf field <code>double latency = 1;</code>
     */
    protected $latency = 0.0;

    /**
     * Constructor.
     *
     * @param array $data {
     *     Optional. Data for populating the Message object.
     *
     *     @type float $latency
     * }
     */
    public function __construct($data = NULL) {
        \GPBMetadata\Src\Proto\Grpc\Testing\ProxyService::initOnce();
        parent::__construct($data);
    }

    /**
     * Generated from protobuf field <code>double latency = 1;</code>
     * @return float
     */
    public function getLatency()
    {
        return $this->latency;
    }

    /**
     * Generated from protobuf field <code>double latency = 1;</code>
     * @param float $var
     * @return $this
     */
    public function setLatency($var)
    {
        GPBUtil::checkDouble($var);
        $this->latency = $var;

        return $this;
    }

}

