<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: src/proto/grpc/testing/messages.proto

namespace Grpc\Testing\LoadBalancerStatsResponse;

use UnexpectedValueException;

/**
 * Protobuf type <code>grpc.testing.LoadBalancerStatsResponse.MetadataType</code>
 */
class MetadataType
{
    /**
     * Generated from protobuf enum <code>UNKNOWN = 0;</code>
     */
    const UNKNOWN = 0;
    /**
     * Generated from protobuf enum <code>INITIAL = 1;</code>
     */
    const INITIAL = 1;
    /**
     * Generated from protobuf enum <code>TRAILING = 2;</code>
     */
    const TRAILING = 2;

    private static $valueToName = [
        self::UNKNOWN => 'UNKNOWN',
        self::INITIAL => 'INITIAL',
        self::TRAILING => 'TRAILING',
    ];

    public static function name($value)
    {
        if (!isset(self::$valueToName[$value])) {
            throw new UnexpectedValueException(sprintf(
                    'Enum %s has no name defined for value %s', __CLASS__, $value));
        }
        return self::$valueToName[$value];
    }


    public static function value($name)
    {
        $const = __CLASS__ . '::' . strtoupper($name);
        if (!defined($const)) {
            throw new UnexpectedValueException(sprintf(
                    'Enum %s has no value defined for name %s', __CLASS__, $name));
        }
        return constant($const);
    }
}

