// Copyright 2017 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

import "tests/testing/proto/requests.proto";

package tests_of_grpc_testing;

message Down {
  int32 first_down_field = 1;
}

message Strange {
  int32 first_strange_field = 1;
}

message Bottom {
  int32 first_bottom_field = 1;
}

service FirstService {
  rpc UnUn(Up) returns (Down);
  rpc UnStre(Charm) returns (stream Strange);
  rpc StreUn(stream Charm) returns (Strange);
  rpc StreStre(stream Top) returns (stream Bottom);
}

service SecondService {
  rpc UnStre(Strange) returns (stream Charm);
}
