# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/network/generic_proxy/action/v3/action.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/network/generic_proxy/action/v3/action.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.config.route.v3 import route_components_pb2 as envoy_dot_config_dot_route_dot_v3_dot_route__components__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nEenvoy/extensions/filters/network/generic_proxy/action/v3/action.proto\x12\x38\x65nvoy.extensions.filters.network.generic_proxy.action.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a,envoy/config/route/v3/route_components.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xe9\x03\n\x0bRouteAction\x12\x0c\n\x04name\x18\x05 \x01(\t\x12\x11\n\x07\x63luster\x18\x01 \x01(\tH\x00\x12\x43\n\x11weighted_clusters\x18\x02 \x01(\x0b\x32&.envoy.config.route.v3.WeightedClusterH\x00\x12\x30\n\x08metadata\x18\x03 \x01(\x0b\x32\x1e.envoy.config.core.v3.Metadata\x12u\n\x11per_filter_config\x18\x04 \x03(\x0b\x32Z.envoy.extensions.filters.network.generic_proxy.action.v3.RouteAction.PerFilterConfigEntry\x12*\n\x07timeout\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x37\n\x0cretry_policy\x18\x07 \x01(\x0b\x32!.envoy.config.core.v3.RetryPolicy\x1aL\n\x14PerFilterConfigEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any:\x02\x38\x01\x42\x18\n\x11\x63luster_specifier\x12\x03\xf8\x42\x01\x42\xd1\x01\nFio.envoyproxy.envoy.extensions.filters.network.generic_proxy.action.v3B\x0b\x41\x63tionProtoP\x01Zhgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/generic_proxy/action/v3;actionv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.network.generic_proxy.action.v3.action_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nFio.envoyproxy.envoy.extensions.filters.network.generic_proxy.action.v3B\013ActionProtoP\001Zhgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/generic_proxy/action/v3;actionv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_ROUTEACTION_PERFILTERCONFIGENTRY']._loaded_options = None
  _globals['_ROUTEACTION_PERFILTERCONFIGENTRY']._serialized_options = b'8\001'
  _globals['_ROUTEACTION'].oneofs_by_name['cluster_specifier']._loaded_options = None
  _globals['_ROUTEACTION'].oneofs_by_name['cluster_specifier']._serialized_options = b'\370B\001'
  _globals['_ROUTEACTION']._serialized_start=359
  _globals['_ROUTEACTION']._serialized_end=848
  _globals['_ROUTEACTION_PERFILTERCONFIGENTRY']._serialized_start=746
  _globals['_ROUTEACTION_PERFILTERCONFIGENTRY']._serialized_end=822
# @@protoc_insertion_point(module_scope)
