# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: test/v1/simple.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'test/v1/simple.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api.expr.v1alpha1 import checked_pb2 as google_dot_api_dot_expr_dot_v1alpha1_dot_checked__pb2
from google.api.expr.v1alpha1 import eval_pb2 as google_dot_api_dot_expr_dot_v1alpha1_dot_eval__pb2
from google.api.expr.v1alpha1 import value_pb2 as google_dot_api_dot_expr_dot_v1alpha1_dot_value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14test/v1/simple.proto\x12\x17google.api.expr.test.v1\x1a&google/api/expr/v1alpha1/checked.proto\x1a#google/api/expr/v1alpha1/eval.proto\x1a$google/api/expr/v1alpha1/value.proto\"p\n\x0eSimpleTestFile\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12;\n\x07section\x18\x03 \x03(\x0b\x32*.google.api.expr.test.v1.SimpleTestSection\"i\n\x11SimpleTestSection\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x31\n\x04test\x18\x03 \x03(\x0b\x32#.google.api.expr.test.v1.SimpleTest\"\xe4\x05\n\nSimpleTest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0c\n\x04\x65xpr\x18\x03 \x01(\t\x12\x16\n\x0e\x64isable_macros\x18\x04 \x01(\x08\x12\x15\n\rdisable_check\x18\x05 \x01(\x08\x12\x12\n\ncheck_only\x18\x0f \x01(\x08\x12\x30\n\x08type_env\x18\x06 \x03(\x0b\x32\x1e.google.api.expr.v1alpha1.Decl\x12\x11\n\tcontainer\x18\r \x01(\t\x12\x43\n\x08\x62indings\x18\x07 \x03(\x0b\x32\x31.google.api.expr.test.v1.SimpleTest.BindingsEntry\x12\x30\n\x05value\x18\x08 \x01(\x0b\x32\x1f.google.api.expr.v1alpha1.ValueH\x00\x12<\n\x0ctyped_result\x18\x10 \x01(\x0b\x32$.google.api.expr.test.v1.TypedResultH\x00\x12\x38\n\neval_error\x18\t \x01(\x0b\x32\".google.api.expr.v1alpha1.ErrorSetH\x00\x12\x43\n\x0f\x61ny_eval_errors\x18\n \x01(\x0b\x32(.google.api.expr.test.v1.ErrorSetMatcherH\x00\x12\x37\n\x07unknown\x18\x0b \x01(\x0b\x32$.google.api.expr.v1alpha1.UnknownSetH\x00\x12\x42\n\x0c\x61ny_unknowns\x18\x0c \x01(\x0b\x32*.google.api.expr.test.v1.UnknownSetMatcherH\x00\x1aT\n\rBindingsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x32\n\x05value\x18\x02 \x01(\x0b\x32#.google.api.expr.v1alpha1.ExprValue:\x02\x38\x01\x42\x10\n\x0eresult_matcherJ\x04\x08\x0e\x10\x0f\"t\n\x0bTypedResult\x12/\n\x06result\x18\x01 \x01(\x0b\x32\x1f.google.api.expr.v1alpha1.Value\x12\x34\n\x0c\x64\x65\x64uced_type\x18\x02 \x01(\x0b\x32\x1e.google.api.expr.v1alpha1.Type\"E\n\x0f\x45rrorSetMatcher\x12\x32\n\x06\x65rrors\x18\x01 \x03(\x0b\x32\".google.api.expr.v1alpha1.ErrorSet\"K\n\x11UnknownSetMatcher\x12\x36\n\x08unknowns\x18\x01 \x03(\x0b\x32$.google.api.expr.v1alpha1.UnknownSetBG\n\x1b\x63om.google.api.expr.test.v1B\x0bSimpleProtoZ\x1b\x63\x65l.dev/expr/test/v1/testpbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'test.v1.simple_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\033com.google.api.expr.test.v1B\013SimpleProtoZ\033cel.dev/expr/test/v1/testpb'
  _globals['_SIMPLETEST_BINDINGSENTRY']._loaded_options = None
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_options = b'8\001'
  _globals['_SIMPLETESTFILE']._serialized_start=164
  _globals['_SIMPLETESTFILE']._serialized_end=276
  _globals['_SIMPLETESTSECTION']._serialized_start=278
  _globals['_SIMPLETESTSECTION']._serialized_end=383
  _globals['_SIMPLETEST']._serialized_start=386
  _globals['_SIMPLETEST']._serialized_end=1126
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_start=1018
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_end=1102
  _globals['_TYPEDRESULT']._serialized_start=1128
  _globals['_TYPEDRESULT']._serialized_end=1244
  _globals['_ERRORSETMATCHER']._serialized_start=1246
  _globals['_ERRORSETMATCHER']._serialized_end=1315
  _globals['_UNKNOWNSETMATCHER']._serialized_start=1317
  _globals['_UNKNOWNSETMATCHER']._serialized_end=1392
# @@protoc_insertion_point(module_scope)
