# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/trace/v3/lightstep.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/trace/v3/lightstep.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%envoy/config/trace/v3/lightstep.proto\x12\x15\x65nvoy.config.trace.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xed\x02\n\x0fLightstepConfig\x12\"\n\x11\x63ollector_cluster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12&\n\x11\x61\x63\x63\x65ss_token_file\x18\x02 \x01(\tB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12\x36\n\x0c\x61\x63\x63\x65ss_token\x18\x04 \x01(\x0b\x32 .envoy.config.core.v3.DataSource\x12`\n\x11propagation_modes\x18\x03 \x03(\x0e\x32\x36.envoy.config.trace.v3.LightstepConfig.PropagationModeB\r\xfa\x42\n\x92\x01\x07\"\x05\x82\x01\x02\x10\x01\"F\n\x0fPropagationMode\x12\t\n\x05\x45NVOY\x10\x00\x12\r\n\tLIGHTSTEP\x10\x01\x12\x06\n\x02\x42\x33\x10\x02\x12\x11\n\rTRACE_CONTEXT\x10\x03:,\x9a\xc5\x88\x1e\'\n%envoy.config.trace.v2.LightstepConfigB\xb7\x01\n#io.envoyproxy.envoy.config.trace.v3B\x0eLightstepProtoP\x01ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v3;tracev3\xf2\x98\xfe\x8f\x05,\x12*envoy.extensions.tracers.lightstep.v4alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.trace.v3.lightstep_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n#io.envoyproxy.envoy.config.trace.v3B\016LightstepProtoP\001ZDgithub.com/envoyproxy/go-control-plane/envoy/config/trace/v3;tracev3\362\230\376\217\005,\022*envoy.extensions.tracers.lightstep.v4alpha\272\200\310\321\006\002\020\002'
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['collector_cluster']._loaded_options = None
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['collector_cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['access_token_file']._loaded_options = None
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['access_token_file']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['propagation_modes']._loaded_options = None
  _globals['_LIGHTSTEPCONFIG'].fields_by_name['propagation_modes']._serialized_options = b'\372B\n\222\001\007\"\005\202\001\002\020\001'
  _globals['_LIGHTSTEPCONFIG']._loaded_options = None
  _globals['_LIGHTSTEPCONFIG']._serialized_options = b'\232\305\210\036\'\n%envoy.config.trace.v2.LightstepConfig'
  _globals['_LIGHTSTEPCONFIG']._serialized_start=258
  _globals['_LIGHTSTEPCONFIG']._serialized_end=623
  _globals['_LIGHTSTEPCONFIG_PROPAGATIONMODE']._serialized_start=507
  _globals['_LIGHTSTEPCONFIG_PROPAGATIONMODE']._serialized_end=577
# @@protoc_insertion_point(module_scope)
