# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/vcl/v3alpha/vcl_socket_interface.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/vcl/v3alpha/vcl_socket_interface.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?contrib/envoy/extensions/vcl/v3alpha/vcl_socket_interface.proto\x12\x1c\x65nvoy.extensions.vcl.v3alpha\x1a\x1dudpa/annotations/status.proto\"\x14\n\x12VclSocketInterfaceB\x9e\x01\n*io.envoyproxy.envoy.extensions.vcl.v3alphaB\x17VclSocketInterfaceProtoP\x01ZKgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/vcl/v3alpha\xba\x80\xc8\xd1\x06\x04\x08\x01\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.vcl.v3alpha.vcl_socket_interface_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n*io.envoyproxy.envoy.extensions.vcl.v3alphaB\027VclSocketInterfaceProtoP\001ZKgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/vcl/v3alpha\272\200\310\321\006\004\010\001\020\002'
  _globals['_VCLSOCKETINTERFACE']._serialized_start=128
  _globals['_VCLSOCKETINTERFACE']._serialized_end=148
# @@protoc_insertion_point(module_scope)
