# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/credential_injector/v3/credential_injector.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/credential_injector/v3/credential_injector.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import extension_pb2 as envoy_dot_config_dot_core_dot_v3_dot_extension__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nNenvoy/extensions/filters/http/credential_injector/v3/credential_injector.proto\x12\x34\x65nvoy.extensions.filters.http.credential_injector.v3\x1a$envoy/config/core/v3/extension.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x9b\x01\n\x12\x43redentialInjector\x12\x11\n\toverwrite\x18\x01 \x01(\x08\x12(\n allow_request_without_credential\x18\x02 \x01(\x08\x12H\n\ncredential\x18\x03 \x01(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfigB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x42\xda\x01\nBio.envoyproxy.envoy.extensions.filters.http.credential_injector.v3B\x17\x43redentialInjectorProtoP\x01Zqgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/credential_injector/v3;credential_injectorv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.credential_injector.v3.credential_injector_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nBio.envoyproxy.envoy.extensions.filters.http.credential_injector.v3B\027CredentialInjectorProtoP\001Zqgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/credential_injector/v3;credential_injectorv3\272\200\310\321\006\002\020\002'
  _globals['_CREDENTIALINJECTOR'].fields_by_name['credential']._loaded_options = None
  _globals['_CREDENTIALINJECTOR'].fields_by_name['credential']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_CREDENTIALINJECTOR']._serialized_start=231
  _globals['_CREDENTIALINJECTOR']._serialized_end=386
# @@protoc_insertion_point(module_scope)
