# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cel/expr/conformance/test/simple.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'cel/expr/conformance/test/simple.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cel.expr import checked_pb2 as cel_dot_expr_dot_checked__pb2
from cel.expr import eval_pb2 as cel_dot_expr_dot_eval__pb2
from cel.expr import value_pb2 as cel_dot_expr_dot_value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&cel/expr/conformance/test/simple.proto\x12\x19\x63\x65l.expr.conformance.test\x1a\x16\x63\x65l/expr/checked.proto\x1a\x13\x63\x65l/expr/eval.proto\x1a\x14\x63\x65l/expr/value.proto\"r\n\x0eSimpleTestFile\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12=\n\x07section\x18\x03 \x03(\x0b\x32,.cel.expr.conformance.test.SimpleTestSection\"k\n\x11SimpleTestSection\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x33\n\x04test\x18\x03 \x03(\x0b\x32%.cel.expr.conformance.test.SimpleTest\"\xa6\x05\n\nSimpleTest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x0c\n\x04\x65xpr\x18\x03 \x01(\t\x12\x16\n\x0e\x64isable_macros\x18\x04 \x01(\x08\x12\x15\n\rdisable_check\x18\x05 \x01(\x08\x12\x12\n\ncheck_only\x18\x0f \x01(\x08\x12 \n\x08type_env\x18\x06 \x03(\x0b\x32\x0e.cel.expr.Decl\x12\x11\n\tcontainer\x18\r \x01(\t\x12\x0e\n\x06locale\x18\x0e \x01(\t\x12\x45\n\x08\x62indings\x18\x07 \x03(\x0b\x32\x33.cel.expr.conformance.test.SimpleTest.BindingsEntry\x12 \n\x05value\x18\x08 \x01(\x0b\x32\x0f.cel.expr.ValueH\x00\x12>\n\x0ctyped_result\x18\x10 \x01(\x0b\x32&.cel.expr.conformance.test.TypedResultH\x00\x12(\n\neval_error\x18\t \x01(\x0b\x32\x12.cel.expr.ErrorSetH\x00\x12\x45\n\x0f\x61ny_eval_errors\x18\n \x01(\x0b\x32*.cel.expr.conformance.test.ErrorSetMatcherH\x00\x12\'\n\x07unknown\x18\x0b \x01(\x0b\x32\x14.cel.expr.UnknownSetH\x00\x12\x44\n\x0c\x61ny_unknowns\x18\x0c \x01(\x0b\x32,.cel.expr.conformance.test.UnknownSetMatcherH\x00\x1a\x44\n\rBindingsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.cel.expr.ExprValue:\x02\x38\x01\x42\x10\n\x0eresult_matcher\"T\n\x0bTypedResult\x12\x1f\n\x06result\x18\x01 \x01(\x0b\x32\x0f.cel.expr.Value\x12$\n\x0c\x64\x65\x64uced_type\x18\x02 \x01(\x0b\x32\x0e.cel.expr.Type\"5\n\x0f\x45rrorSetMatcher\x12\"\n\x06\x65rrors\x18\x01 \x03(\x0b\x32\x12.cel.expr.ErrorSet\";\n\x11UnknownSetMatcher\x12&\n\x08unknowns\x18\x01 \x03(\x0b\x32\x14.cel.expr.UnknownSetBP\n\x1d\x64\x65v.cel.expr.conformance.testB\x0bSimpleProtoP\x01Z\x1d\x63\x65l.dev/expr/conformance/test\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cel.expr.conformance.test.simple_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\035dev.cel.expr.conformance.testB\013SimpleProtoP\001Z\035cel.dev/expr/conformance/test\370\001\001'
  _globals['_SIMPLETEST_BINDINGSENTRY']._loaded_options = None
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_options = b'8\001'
  _globals['_SIMPLETESTFILE']._serialized_start=136
  _globals['_SIMPLETESTFILE']._serialized_end=250
  _globals['_SIMPLETESTSECTION']._serialized_start=252
  _globals['_SIMPLETESTSECTION']._serialized_end=359
  _globals['_SIMPLETEST']._serialized_start=362
  _globals['_SIMPLETEST']._serialized_end=1040
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_start=954
  _globals['_SIMPLETEST_BINDINGSENTRY']._serialized_end=1022
  _globals['_TYPEDRESULT']._serialized_start=1042
  _globals['_TYPEDRESULT']._serialized_end=1126
  _globals['_ERRORSETMATCHER']._serialized_start=1128
  _globals['_ERRORSETMATCHER']._serialized_end=1181
  _globals['_UNKNOWNSETMATCHER']._serialized_start=1183
  _globals['_UNKNOWNSETMATCHER']._serialized_end=1242
# @@protoc_insertion_point(module_scope)
