<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: src/proto/grpc/testing/messages.proto

namespace Grpc\Testing;

use Google\Protobuf\Internal\GPBType;
use Google\Protobuf\Internal\GPBUtil;
use Google\Protobuf\RepeatedField;

/**
 * Generated from protobuf message <code>grpc.testing.HookRequest</code>
 */
class HookRequest extends \Google\Protobuf\Internal\Message
{
    /**
     * Generated from protobuf field <code>.grpc.testing.HookRequest.HookRequestCommand command = 1;</code>
     */
    protected $command = 0;
    /**
     * Generated from protobuf field <code>int32 grpc_code_to_return = 2;</code>
     */
    protected $grpc_code_to_return = 0;
    /**
     * Generated from protobuf field <code>string grpc_status_description = 3;</code>
     */
    protected $grpc_status_description = '';
    /**
     * Server port to listen to
     *
     * Generated from protobuf field <code>int32 server_port = 4;</code>
     */
    protected $server_port = 0;

    /**
     * Constructor.
     *
     * @param array $data {
     *     Optional. Data for populating the Message object.
     *
     *     @type int $command
     *     @type int $grpc_code_to_return
     *     @type string $grpc_status_description
     *     @type int $server_port
     *           Server port to listen to
     * }
     */
    public function __construct($data = NULL) {
        \GPBMetadata\Src\Proto\Grpc\Testing\Messages::initOnce();
        parent::__construct($data);
    }

    /**
     * Generated from protobuf field <code>.grpc.testing.HookRequest.HookRequestCommand command = 1;</code>
     * @return int one of the values in {@see \Grpc\Testing\HookRequest\HookRequestCommand}
     */
    public function getCommand()
    {
        return $this->command;
    }

    /**
     * Generated from protobuf field <code>.grpc.testing.HookRequest.HookRequestCommand command = 1;</code>
     * @param int $var one of the values in {@see \Grpc\Testing\HookRequest\HookRequestCommand}
     * @return $this
     */
    public function setCommand(int $var)
    {
        GPBUtil::checkEnum($var, \Grpc\Testing\HookRequest\HookRequestCommand::class);
        $this->command = $var;

        return $this;
    }

    /**
     * Generated from protobuf field <code>int32 grpc_code_to_return = 2;</code>
     * @return int
     */
    public function getGrpcCodeToReturn()
    {
        return $this->grpc_code_to_return;
    }

    /**
     * Generated from protobuf field <code>int32 grpc_code_to_return = 2;</code>
     * @param int $var
     * @return $this
     */
    public function setGrpcCodeToReturn(int $var)
    {
        GPBUtil::checkInt32($var);
        $this->grpc_code_to_return = $var;

        return $this;
    }

    /**
     * Generated from protobuf field <code>string grpc_status_description = 3;</code>
     * @return string
     */
    public function getGrpcStatusDescription()
    {
        return $this->grpc_status_description;
    }

    /**
     * Generated from protobuf field <code>string grpc_status_description = 3;</code>
     * @param string $var
     * @return $this
     */
    public function setGrpcStatusDescription(string $var)
    {
        GPBUtil::checkString($var, true);
        $this->grpc_status_description = $var;

        return $this;
    }

    /**
     * Server port to listen to
     *
     * Generated from protobuf field <code>int32 server_port = 4;</code>
     * @return int
     */
    public function getServerPort()
    {
        return $this->server_port;
    }

    /**
     * Server port to listen to
     *
     * Generated from protobuf field <code>int32 server_port = 4;</code>
     * @param int $var
     * @return $this
     */
    public function setServerPort(int $var)
    {
        GPBUtil::checkInt32($var);
        $this->server_port = $var;

        return $this;
    }

}

