<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/proto/grpc/testing/messages.proto

namespace Grpc\Testing;

if (false) {
    /**
     * This class is deprecated. Use Grpc\Testing\LoadBalancerAccumulatedStatsResponse\MethodStats instead.
     * @deprecated
     */
    class LoadBalancerAccumulatedStatsResponse_MethodStats {}
}
class_exists(LoadBalancerAccumulatedStatsResponse\MethodStats::class);
@trigger_error('Grpc\Testing\LoadBalancerAccumulatedStatsResponse_MethodStats is deprecated and will be removed in the next major release. Use Grpc\Testing\LoadBalancerAccumulatedStatsResponse\MethodStats instead', E_USER_DEPRECATED);

