# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/mcp_router/v3/mcp_router.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/mcp_router/v3/mcp_router.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<envoy/extensions/filters/http/mcp_router/v3/mcp_router.proto\x12+envoy.extensions.filters.http.mcp_router.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xd3\x02\n\tMcpRouter\x12R\n\x07servers\x18\x01 \x03(\x0b\x32\x41.envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpBackend\x1ar\n\nMcpBackend\x12\x0c\n\x04name\x18\x01 \x01(\t\x12V\n\x0bmcp_cluster\x18\x02 \x01(\x0b\x32\x41.envoy.extensions.filters.http.mcp_router.v3.McpRouter.McpCluster\x1a~\n\nMcpCluster\x12\x18\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x0c\n\x04path\x18\x02 \x01(\t\x12*\n\x07timeout\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x1c\n\x14host_rewrite_literal\x18\x04 \x01(\tB\xbe\x01\n9io.envoyproxy.envoy.extensions.filters.http.mcp_router.v3B\x0eMcpRouterProtoP\x01Z_github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/mcp_router/v3;mcp_routerv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.mcp_router.v3.mcp_router_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n9io.envoyproxy.envoy.extensions.filters.http.mcp_router.v3B\016McpRouterProtoP\001Z_github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/mcp_router/v3;mcp_routerv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_MCPROUTER_MCPCLUSTER'].fields_by_name['cluster']._loaded_options = None
  _globals['_MCPROUTER_MCPCLUSTER'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_MCPROUTER']._serialized_start=231
  _globals['_MCPROUTER']._serialized_end=570
  _globals['_MCPROUTER_MCPBACKEND']._serialized_start=328
  _globals['_MCPROUTER_MCPBACKEND']._serialized_end=442
  _globals['_MCPROUTER_MCPCLUSTER']._serialized_start=444
  _globals['_MCPROUTER_MCPCLUSTER']._serialized_end=570
# @@protoc_insertion_point(module_scope)
