/* This file was generated by upb_generator from the input file:
 *
 *     envoy/service/discovery/v3/discovery.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "envoy/service/discovery/v3/discovery.upb_minitable.h"
#include "envoy/config/core/v3/base.upb_minitable.h"
#include "google/protobuf/any.upb_minitable.h"
#include "google/protobuf/duration.upb_minitable.h"
#include "google/rpc/status.upb_minitable.h"
#include "udpa/annotations/status.upb_minitable.h"
#include "udpa/annotations/versioning.upb_minitable.h"
#include "validate/validate.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const UPB_PRIVATE(upb_GeneratedExtensionListEntry)* UPB_PRIVATE(upb_generated_extension_list);
typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__service__discovery__v3__ResourceLocator_msg_init_Fields;

static const envoy__service__discovery__v3__ResourceLocator_msg_init_Fields envoy_service_discovery_v3_ResourceLocator__fields = {{
  {1, UPB_SIZE(12, 8), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(8, 24), 0, 3, 11, (int)kUpb_FieldMode_Map | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__ResourceLocator__DynamicParametersEntry_msg_init},
}};

const upb_MiniTable envoy__service__discovery__v3__ResourceLocator_msg_init = {
  &envoy_service_discovery_v3_ResourceLocator__fields.fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.ResourceLocator",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
} envoy__service__discovery__v3__ResourceLocator__DynamicParametersEntry_msg_init_Fields;

static const envoy__service__discovery__v3__ResourceLocator__DynamicParametersEntry_msg_init_Fields envoy_service_discovery_v3_ResourceLocator_DynamicParametersEntry__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, 32, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__service__discovery__v3__ResourceLocator__DynamicParametersEntry_msg_init = {
  &envoy_service_discovery_v3_ResourceLocator_DynamicParametersEntry__fields.fields[0],
  48, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.ResourceLocator.DynamicParametersEntry",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x002000003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__service__discovery__v3__ResourceName_msg_init_Fields;

static const envoy__service__discovery__v3__ResourceName_msg_init_Fields envoy_service_discovery_v3_ResourceName__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), 64, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__DynamicParameterConstraints_msg_init},
}};

const upb_MiniTable envoy__service__discovery__v3__ResourceName_msg_init = {
  &envoy_service_discovery_v3_ResourceName__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.ResourceName",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[2];
} envoy__service__discovery__v3__ResourceError_msg_init_Fields;

static const envoy__service__discovery__v3__ResourceError_msg_init_Fields envoy_service_discovery_v3_ResourceError__fields = {{
  {1, UPB_SIZE(12, 16), 64, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 65, UPB_SIZE(4, 5), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__ResourceName_msg_init},
  {.UPB_PRIVATE(submsg) = &google__rpc__Status_msg_init},
}};

const upb_MiniTable envoy__service__discovery__v3__ResourceError_msg_init = {
  &envoy_service_discovery_v3_ResourceError__fields.fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.ResourceError",
#endif
};

typedef struct {
  upb_MiniTableField fields[7];
  upb_MiniTableSubInternal subs[3];
} envoy__service__discovery__v3__DiscoveryRequest_msg_init_Fields;

static const envoy__service__discovery__v3__DiscoveryRequest_msg_init_Fields envoy_service_discovery_v3_DiscoveryRequest__fields = {{
  {1, UPB_SIZE(28, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 64), 64, UPB_SIZE(18, 19), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(16, 72), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(36, 32), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(44, 48), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(20, 80), 65, UPB_SIZE(7, 9), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {7, UPB_SIZE(24, 88), 0, UPB_SIZE(5, 8), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__Node_msg_init},
  {.UPB_PRIVATE(submsg) = &google__rpc__Status_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__ResourceLocator_msg_init},
}};

const upb_MiniTable envoy__service__discovery__v3__DiscoveryRequest_msg_init = {
  &envoy_service_discovery_v3_DiscoveryRequest__fields.fields[0],
  UPB_SIZE(56, 96), 7, kUpb_ExtMode_NonExtendable, 7, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.DiscoveryRequest",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x004800003f00001a, &upb_DecodeFast_String_Repeated_Tag1Byte},
    {0x002000003f000022, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x003000003f00002a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[7];
  upb_MiniTableSubInternal subs[3];
} envoy__service__discovery__v3__DiscoveryResponse_msg_init_Fields;

static const envoy__service__discovery__v3__DiscoveryResponse_msg_init_Fields envoy_service_discovery_v3_DiscoveryResponse__fields = {{
  {1, UPB_SIZE(24, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 64), 0, UPB_SIZE(18, 19), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, 9, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
  {4, 32, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(40, 48), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(16, 72), 64, UPB_SIZE(7, 9), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {7, UPB_SIZE(20, 80), 0, UPB_SIZE(5, 8), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__ControlPlane_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__ResourceError_msg_init},
}};

const upb_MiniTable envoy__service__discovery__v3__DiscoveryResponse_msg_init = {
  &envoy_service_discovery_v3_DiscoveryResponse__fields.fields[0],
  UPB_SIZE(48, 88), 7, kUpb_ExtMode_NonExtendable, 7, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.DiscoveryResponse",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000900003f000018, &upb_DecodeFast_Bool_Scalar_Tag1Byte},
    {0x002000003f000022, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x003000003f00002a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[9];
  upb_MiniTableSubInternal subs[5];
} envoy__service__discovery__v3__DeltaDiscoveryRequest_msg_init_Fields;

static const envoy__service__discovery__v3__DeltaDiscoveryRequest_msg_init_Fields envoy_service_discovery_v3_DeltaDiscoveryRequest__fields = {{
  {1, UPB_SIZE(12, 48), 64, UPB_SIZE(27, 28), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(40, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(16, 56), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(20, 64), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(24, 72), 0, UPB_SIZE(16, 18), 11, (int)kUpb_FieldMode_Map | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(48, 32), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {7, UPB_SIZE(28, 80), 65, UPB_SIZE(11, 14), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {8, UPB_SIZE(32, 88), 0, UPB_SIZE(9, 13), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {9, UPB_SIZE(36, 96), 0, UPB_SIZE(7, 12), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__Node_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__DeltaDiscoveryRequest__InitialResourceVersionsEntry_msg_init},
  {.UPB_PRIVATE(submsg) = &google__rpc__Status_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__ResourceLocator_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__ResourceLocator_msg_init},
}};

const upb_MiniTable envoy__service__discovery__v3__DeltaDiscoveryRequest_msg_init = {
  &envoy_service_discovery_v3_DeltaDiscoveryRequest__fields.fields[0],
  UPB_SIZE(56, 104), 9, kUpb_ExtMode_NonExtendable, 9, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.DeltaDiscoveryRequest",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x003800003f00001a, &upb_DecodeFast_String_Repeated_Tag1Byte},
    {0x004000003f000022, &upb_DecodeFast_String_Repeated_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x002000003f000032, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
} envoy__service__discovery__v3__DeltaDiscoveryRequest__InitialResourceVersionsEntry_msg_init_Fields;

static const envoy__service__discovery__v3__DeltaDiscoveryRequest__InitialResourceVersionsEntry_msg_init_Fields envoy_service_discovery_v3_DeltaDiscoveryRequest_InitialResourceVersionsEntry__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, 32, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__service__discovery__v3__DeltaDiscoveryRequest__InitialResourceVersionsEntry_msg_init = {
  &envoy_service_discovery_v3_DeltaDiscoveryRequest_InitialResourceVersionsEntry__fields.fields[0],
  48, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.DeltaDiscoveryRequest.InitialResourceVersionsEntry",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x002000003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[8];
  upb_MiniTableSubInternal subs[4];
} envoy__service__discovery__v3__DeltaDiscoveryResponse_msg_init_Fields;

static const envoy__service__discovery__v3__DeltaDiscoveryResponse_msg_init_Fields envoy_service_discovery_v3_DeltaDiscoveryResponse__fields = {{
  {1, UPB_SIZE(32, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 64), 0, 21, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(40, 32), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {5, 48, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(16, 72), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {7, UPB_SIZE(20, 80), 64, UPB_SIZE(10, 11), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {8, UPB_SIZE(24, 88), 0, UPB_SIZE(8, 10), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {9, UPB_SIZE(28, 96), 0, UPB_SIZE(6, 9), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__Resource_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__ControlPlane_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__ResourceName_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__ResourceError_msg_init},
}};

const upb_MiniTable envoy__service__discovery__v3__DeltaDiscoveryResponse_msg_init = {
  &envoy_service_discovery_v3_DeltaDiscoveryResponse__fields.fields[0],
  UPB_SIZE(56, 104), 8, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.DeltaDiscoveryResponse",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x002000003f000022, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x003000003f00002a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x004800003f000032, &upb_DecodeFast_String_Repeated_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[4];
  upb_MiniTableSubInternal subs[4];
} envoy__service__discovery__v3__DynamicParameterConstraints_msg_init_Fields;

static const envoy__service__discovery__v3__DynamicParameterConstraints_msg_init_Fields envoy_service_discovery_v3_DynamicParameterConstraints__fields = {{
  {1, UPB_SIZE(12, 16), -9, 12, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), -9, UPB_SIZE(10, 11), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 16), -9, UPB_SIZE(8, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(12, 16), -9, UPB_SIZE(6, 9), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__DynamicParameterConstraints__ConstraintList_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__DynamicParameterConstraints__ConstraintList_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__DynamicParameterConstraints_msg_init},
}};

const upb_MiniTable envoy__service__discovery__v3__DynamicParameterConstraints_msg_init = {
  &envoy_service_discovery_v3_DynamicParameterConstraints__fields.fields[0],
  UPB_SIZE(16, 24), 4, kUpb_ExtMode_NonExtendable, 4, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.DynamicParameterConstraints",
#endif
};

typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[1];
} envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint_msg_init_Fields;

static const envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint_msg_init_Fields envoy_service_discovery_v3_DynamicParameterConstraints_SingleConstraint__fields = {{
  {1, UPB_SIZE(12, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(20, 32), -9, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 32), -9, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint__Exists_msg_init},
}};

const upb_MiniTable envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint_msg_init = {
  &envoy_service_discovery_v3_DynamicParameterConstraints_SingleConstraint__fields.fields[0],
  UPB_SIZE(32, 48), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.DynamicParameterConstraints.SingleConstraint",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0020000802000012, &upb_DecodeFast_String_Oneof_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

const upb_MiniTable envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint__Exists_msg_init = {
  NULL,
  8, 0, kUpb_ExtMode_NonExtendable, 0, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.DynamicParameterConstraints.SingleConstraint.Exists",
#endif
};

typedef struct {
  upb_MiniTableField fields[1];
  upb_MiniTableSubInternal subs[1];
} envoy__service__discovery__v3__DynamicParameterConstraints__ConstraintList_msg_init_Fields;

static const envoy__service__discovery__v3__DynamicParameterConstraints__ConstraintList_msg_init_Fields envoy_service_discovery_v3_DynamicParameterConstraints_ConstraintList__fields = {{
  {1, 8, 0, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__DynamicParameterConstraints_msg_init},
}};

const upb_MiniTable envoy__service__discovery__v3__DynamicParameterConstraints__ConstraintList_msg_init = {
  &envoy_service_discovery_v3_DynamicParameterConstraints_ConstraintList__fields.fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.DynamicParameterConstraints.ConstraintList",
#endif
};

typedef struct {
  upb_MiniTableField fields[8];
  upb_MiniTableSubInternal subs[5];
} envoy__service__discovery__v3__Resource_msg_init_Fields;

static const envoy__service__discovery__v3__Resource_msg_init_Fields envoy_service_discovery_v3_Resource__fields = {{
  {1, UPB_SIZE(36, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 48), 64, 21, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(44, 32), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(16, 56), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(20, 64), 65, UPB_SIZE(13, 14), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {7, UPB_SIZE(24, 72), 66, UPB_SIZE(11, 13), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {8, UPB_SIZE(28, 80), 67, UPB_SIZE(9, 12), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {9, UPB_SIZE(32, 88), 68, UPB_SIZE(7, 11), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Duration_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__Resource__CacheControl_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__service__discovery__v3__ResourceName_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__Metadata_msg_init},
}};

const upb_MiniTable envoy__service__discovery__v3__Resource_msg_init = {
  &envoy_service_discovery_v3_Resource__fields.fields[0],
  UPB_SIZE(56, 96), 8, kUpb_ExtMode_NonExtendable, 4, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.Resource",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x002000003f00001a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x003800003f000022, &upb_DecodeFast_String_Repeated_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[1];
} envoy__service__discovery__v3__Resource__CacheControl_msg_init_Fields;

static const envoy__service__discovery__v3__Resource__CacheControl_msg_init_Fields envoy_service_discovery_v3_Resource_CacheControl__fields = {{
  {1, 8, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__service__discovery__v3__Resource__CacheControl_msg_init = {
  &envoy_service_discovery_v3_Resource_CacheControl__fields.fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.service.discovery.v3.Resource.CacheControl",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_DecodeFast_Bool_Scalar_Tag1Byte},
  })
};

static const upb_MiniTable *messages_layout[15] = {
  &envoy__service__discovery__v3__ResourceLocator_msg_init,
  &envoy__service__discovery__v3__ResourceLocator__DynamicParametersEntry_msg_init,
  &envoy__service__discovery__v3__ResourceName_msg_init,
  &envoy__service__discovery__v3__ResourceError_msg_init,
  &envoy__service__discovery__v3__DiscoveryRequest_msg_init,
  &envoy__service__discovery__v3__DiscoveryResponse_msg_init,
  &envoy__service__discovery__v3__DeltaDiscoveryRequest_msg_init,
  &envoy__service__discovery__v3__DeltaDiscoveryRequest__InitialResourceVersionsEntry_msg_init,
  &envoy__service__discovery__v3__DeltaDiscoveryResponse_msg_init,
  &envoy__service__discovery__v3__DynamicParameterConstraints_msg_init,
  &envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint_msg_init,
  &envoy__service__discovery__v3__DynamicParameterConstraints__SingleConstraint__Exists_msg_init,
  &envoy__service__discovery__v3__DynamicParameterConstraints__ConstraintList_msg_init,
  &envoy__service__discovery__v3__Resource_msg_init,
  &envoy__service__discovery__v3__Resource__CacheControl_msg_init,
};

const upb_MiniTableFile envoy_service_discovery_v3_discovery_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  15,
  0,
  0,
};

#include "upb/port/undef.inc"

