// Copyright 2025 The gRPC Authors
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

service Greeter {
  // Standard unary method
  rpc SayHello (TestAsyncRequest) returns (TestAsyncReply);
  
  // Method that already ends with Async
  rpc SayHelloAsync (TestAsyncRequest) returns (TestAsyncReply);
  
  // Server streaming
  rpc LotsOfReplies (TestAsyncRequest) returns (stream TestAsyncReply);
  
  // Client streaming
  rpc LotsOfGreetings (stream TestAsyncRequest) returns (TestAsyncReply);
  
  // Bidirectional streaming
  rpc BidiHello (stream TestAsyncRequest) returns (stream TestAsyncReply);
  
  // Deprecated method
  rpc OldHello (TestAsyncRequest) returns (TestAsyncReply) {
    option deprecated = true;
  }
}

message TestAsyncRequest {
  string name = 1;
}

message TestAsyncReply {
  string message = 1;
}
