# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/listener/proxy_protocol/v3/proxy_protocol.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/listener/proxy_protocol/v3/proxy_protocol.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import proxy_protocol_pb2 as envoy_dot_config_dot_core_dot_v3_dot_proxy__protocol__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nHenvoy/extensions/filters/listener/proxy_protocol/v3/proxy_protocol.proto\x12\x33\x65nvoy.extensions.filters.listener.proxy_protocol.v3\x1a)envoy/config/core/v3/proxy_protocol.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xfc\x05\n\rProxyProtocol\x12V\n\x05rules\x18\x01 \x03(\x0b\x32G.envoy.extensions.filters.listener.proxy_protocol.v3.ProxyProtocol.Rule\x12-\n%allow_requests_without_proxy_protocol\x18\x02 \x01(\x08\x12M\n\x11pass_through_tlvs\x18\x03 \x01(\x0b\x32\x32.envoy.config.core.v3.ProxyProtocolPassThroughTLVs\x12N\n\x13\x64isallowed_versions\x18\x04 \x03(\x0e\x32\x31.envoy.config.core.v3.ProxyProtocolConfig.Version\x12\x13\n\x0bstat_prefix\x18\x05 \x01(\t\x12\x64\n\x0ctlv_location\x18\x06 \x01(\x0e\x32N.envoy.extensions.filters.listener.proxy_protocol.v3.ProxyProtocol.TlvLocation\x1a@\n\x0cKeyValuePair\x12\x1a\n\x12metadata_namespace\x18\x01 \x01(\t\x12\x14\n\x03key\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x1a\x8b\x01\n\x04Rule\x12\x1a\n\x08tlv_type\x18\x01 \x01(\rB\x08\xfa\x42\x05*\x03\x10\x80\x02\x12g\n\x0eon_tlv_present\x18\x02 \x01(\x0b\x32O.envoy.extensions.filters.listener.proxy_protocol.v3.ProxyProtocol.KeyValuePair\"5\n\x0bTlvLocation\x12\x14\n\x10\x44YNAMIC_METADATA\x10\x00\x12\x10\n\x0c\x46ILTER_STATE\x10\x01:C\x9a\xc5\x88\x1e>\n<envoy.config.filter.listener.proxy_protocol.v2.ProxyProtocolB\xce\x01\nAio.envoyproxy.envoy.extensions.filters.listener.proxy_protocol.v3B\x12ProxyProtocolProtoP\x01Zkgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/listener/proxy_protocol/v3;proxy_protocolv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.listener.proxy_protocol.v3.proxy_protocol_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nAio.envoyproxy.envoy.extensions.filters.listener.proxy_protocol.v3B\022ProxyProtocolProtoP\001Zkgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/listener/proxy_protocol/v3;proxy_protocolv3\272\200\310\321\006\002\020\002'
  _globals['_PROXYPROTOCOL_KEYVALUEPAIR'].fields_by_name['key']._loaded_options = None
  _globals['_PROXYPROTOCOL_KEYVALUEPAIR'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_PROXYPROTOCOL_RULE'].fields_by_name['tlv_type']._loaded_options = None
  _globals['_PROXYPROTOCOL_RULE'].fields_by_name['tlv_type']._serialized_options = b'\372B\005*\003\020\200\002'
  _globals['_PROXYPROTOCOL']._loaded_options = None
  _globals['_PROXYPROTOCOL']._serialized_options = b'\232\305\210\036>\n<envoy.config.filter.listener.proxy_protocol.v2.ProxyProtocol'
  _globals['_PROXYPROTOCOL']._serialized_start=264
  _globals['_PROXYPROTOCOL']._serialized_end=1028
  _globals['_PROXYPROTOCOL_KEYVALUEPAIR']._serialized_start=698
  _globals['_PROXYPROTOCOL_KEYVALUEPAIR']._serialized_end=762
  _globals['_PROXYPROTOCOL_RULE']._serialized_start=765
  _globals['_PROXYPROTOCOL_RULE']._serialized_end=904
  _globals['_PROXYPROTOCOL_TLVLOCATION']._serialized_start=906
  _globals['_PROXYPROTOCOL_TLVLOCATION']._serialized_end=959
# @@protoc_insertion_point(module_scope)
