// Copyright 2025 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "src/core/lib/resource_tracker/resource_tracker.h"

#include <atomic>

namespace grpc_core {

namespace {
std::atomic<ResourceTracker*> g_resource_tracker(nullptr);
}  // namespace

ResourceTracker* ResourceTracker::Get() {
  return g_resource_tracker.load(std::memory_order_acquire);
}

void ResourceTracker::Set(ResourceTracker* tracker) {
  g_resource_tracker.store(tracker, std::memory_order_release);
}

}  // namespace grpc_core
