# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/access_loggers/stats/v3/stats.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/access_loggers/stats/v3/stats.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n4envoy/extensions/access_loggers/stats/v3/stats.proto\x12(envoy.extensions.access_loggers.stats.v3\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xd9\x06\n\x06\x43onfig\x12\x1c\n\x0bstat_prefix\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12N\n\nhistograms\x18\x03 \x03(\x0b\x32:.envoy.extensions.access_loggers.stats.v3.Config.Histogram\x12J\n\x08\x63ounters\x18\x04 \x03(\x0b\x32\x38.envoy.extensions.access_loggers.stats.v3.Config.Counter\x1a;\n\x03Tag\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x1d\n\x0cvalue_format\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x1a\x61\n\x04Stat\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x42\n\x04tags\x18\x02 \x03(\x0b\x32\x34.envoy.extensions.access_loggers.stats.v3.Config.Tag\x1a\xad\x02\n\tHistogram\x12M\n\x04stat\x18\x01 \x01(\x0b\x32\x35.envoy.extensions.access_loggers.stats.v3.Config.StatB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12W\n\x04unit\x18\x02 \x01(\x0e\x32?.envoy.extensions.access_loggers.stats.v3.Config.Histogram.UnitB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12#\n\x0cvalue_format\x18\x03 \x01(\tB\r\xfa\x42\nr\x08\x10\x01:\x01%B\x01%\"S\n\x04Unit\x12\x0f\n\x0bUnspecified\x10\x00\x12\t\n\x05\x42ytes\x10\x01\x12\x10\n\x0cMicroseconds\x10\x02\x12\x10\n\x0cMilliseconds\x10\x03\x12\x0b\n\x07Percent\x10\x04\x1a\xba\x01\n\x07\x43ounter\x12M\n\x04stat\x18\x01 \x01(\x0b\x32\x35.envoy.extensions.access_loggers.stats.v3.Config.StatB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12$\n\x0cvalue_format\x18\x02 \x01(\tB\x0e\xfa\x42\x0br\t:\x01%B\x01%\xd0\x01\x01\x12:\n\x0bvalue_fixed\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.UInt64ValueB\x07\xfa\x42\x04\x32\x02 \x00:\x08\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x42\xa7\x01\n6io.envoyproxy.envoy.extensions.access_loggers.stats.v3B\nStatsProtoP\x01ZWgithub.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/stats/v3;statsv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.access_loggers.stats.v3.stats_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n6io.envoyproxy.envoy.extensions.access_loggers.stats.v3B\nStatsProtoP\001ZWgithub.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/stats/v3;statsv3\272\200\310\321\006\002\020\002'
  _globals['_CONFIG_TAG'].fields_by_name['name']._loaded_options = None
  _globals['_CONFIG_TAG'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CONFIG_TAG'].fields_by_name['value_format']._loaded_options = None
  _globals['_CONFIG_TAG'].fields_by_name['value_format']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CONFIG_STAT'].fields_by_name['name']._loaded_options = None
  _globals['_CONFIG_STAT'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['stat']._loaded_options = None
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['stat']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['unit']._loaded_options = None
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['unit']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['value_format']._loaded_options = None
  _globals['_CONFIG_HISTOGRAM'].fields_by_name['value_format']._serialized_options = b'\372B\nr\010\020\001:\001%B\001%'
  _globals['_CONFIG_COUNTER'].fields_by_name['stat']._loaded_options = None
  _globals['_CONFIG_COUNTER'].fields_by_name['stat']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_CONFIG_COUNTER'].fields_by_name['value_format']._loaded_options = None
  _globals['_CONFIG_COUNTER'].fields_by_name['value_format']._serialized_options = b'\372B\013r\t:\001%B\001%\320\001\001'
  _globals['_CONFIG_COUNTER'].fields_by_name['value_fixed']._loaded_options = None
  _globals['_CONFIG_COUNTER'].fields_by_name['value_fixed']._serialized_options = b'\372B\0042\002 \000'
  _globals['_CONFIG'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_CONFIG'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CONFIG']._loaded_options = None
  _globals['_CONFIG']._serialized_options = b'\322\306\244\341\006\002\010\001'
  _globals['_CONFIG']._serialized_start=220
  _globals['_CONFIG']._serialized_end=1077
  _globals['_CONFIG_TAG']._serialized_start=416
  _globals['_CONFIG_TAG']._serialized_end=475
  _globals['_CONFIG_STAT']._serialized_start=477
  _globals['_CONFIG_STAT']._serialized_end=574
  _globals['_CONFIG_HISTOGRAM']._serialized_start=577
  _globals['_CONFIG_HISTOGRAM']._serialized_end=878
  _globals['_CONFIG_HISTOGRAM_UNIT']._serialized_start=795
  _globals['_CONFIG_HISTOGRAM_UNIT']._serialized_end=878
  _globals['_CONFIG_COUNTER']._serialized_start=881
  _globals['_CONFIG_COUNTER']._serialized_end=1067
# @@protoc_insertion_point(module_scope)
