# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/matching/common_inputs/transport_socket/v3/transport_socket_inputs.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/matching/common_inputs/transport_socket/v3/transport_socket_inputs.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nYenvoy/extensions/matching/common_inputs/transport_socket/v3/transport_socket_inputs.proto\x12;envoy.extensions.matching.common_inputs.transport_socket.v3\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xd6\x01\n\x15\x45ndpointMetadataInput\x12\x0e\n\x06\x66ilter\x18\x01 \x01(\t\x12v\n\x04path\x18\x02 \x03(\x0b\x32^.envoy.extensions.matching.common_inputs.transport_socket.v3.EndpointMetadataInput.PathSegmentB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x1a\x35\n\x0bPathSegment\x12\x16\n\x03key\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00\x42\x0e\n\x07segment\x12\x03\xf8\x42\x01\"\xd6\x01\n\x15LocalityMetadataInput\x12\x0e\n\x06\x66ilter\x18\x01 \x01(\t\x12v\n\x04path\x18\x02 \x03(\x0b\x32^.envoy.extensions.matching.common_inputs.transport_socket.v3.LocalityMetadataInput.PathSegmentB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x1a\x35\n\x0bPathSegment\x12\x16\n\x03key\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00\x42\x0e\n\x07segment\x12\x03\xf8\x42\x01\"(\n\x10\x46ilterStateInput\x12\x14\n\x03key\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\"2\n\x19TransportSocketNameAction\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x42\xe8\x01\nIio.envoyproxy.envoy.extensions.matching.common_inputs.transport_socket.v3B\x1aTransportSocketInputsProtoP\x01Zugithub.com/envoyproxy/go-control-plane/envoy/extensions/matching/common_inputs/transport_socket/v3;transport_socketv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.matching.common_inputs.transport_socket.v3.transport_socket_inputs_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nIio.envoyproxy.envoy.extensions.matching.common_inputs.transport_socket.v3B\032TransportSocketInputsProtoP\001Zugithub.com/envoyproxy/go-control-plane/envoy/extensions/matching/common_inputs/transport_socket/v3;transport_socketv3\272\200\310\321\006\002\020\002'
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT'].oneofs_by_name['segment']._loaded_options = None
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT'].oneofs_by_name['segment']._serialized_options = b'\370B\001'
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT'].fields_by_name['key']._loaded_options = None
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_ENDPOINTMETADATAINPUT'].fields_by_name['path']._loaded_options = None
  _globals['_ENDPOINTMETADATAINPUT'].fields_by_name['path']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT'].oneofs_by_name['segment']._loaded_options = None
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT'].oneofs_by_name['segment']._serialized_options = b'\370B\001'
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT'].fields_by_name['key']._loaded_options = None
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_LOCALITYMETADATAINPUT'].fields_by_name['path']._loaded_options = None
  _globals['_LOCALITYMETADATAINPUT'].fields_by_name['path']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_FILTERSTATEINPUT'].fields_by_name['key']._loaded_options = None
  _globals['_FILTERSTATEINPUT'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_TRANSPORTSOCKETNAMEACTION'].fields_by_name['name']._loaded_options = None
  _globals['_TRANSPORTSOCKETNAMEACTION'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_ENDPOINTMETADATAINPUT']._serialized_start=211
  _globals['_ENDPOINTMETADATAINPUT']._serialized_end=425
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT']._serialized_start=372
  _globals['_ENDPOINTMETADATAINPUT_PATHSEGMENT']._serialized_end=425
  _globals['_LOCALITYMETADATAINPUT']._serialized_start=428
  _globals['_LOCALITYMETADATAINPUT']._serialized_end=642
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT']._serialized_start=372
  _globals['_LOCALITYMETADATAINPUT_PATHSEGMENT']._serialized_end=425
  _globals['_FILTERSTATEINPUT']._serialized_start=644
  _globals['_FILTERSTATEINPUT']._serialized_end=684
  _globals['_TRANSPORTSOCKETNAMEACTION']._serialized_start=686
  _globals['_TRANSPORTSOCKETNAMEACTION']._serialized_end=736
# @@protoc_insertion_point(module_scope)
