# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/network/kafka_mesh/v3alpha/kafka_mesh.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/network/kafka_mesh/v3alpha/kafka_mesh.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nLcontrib/envoy/extensions/filters/network/kafka_mesh/v3alpha/kafka_mesh.proto\x12\x33\x65nvoy.extensions.filters.network.kafka_mesh.v3alpha\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xb5\x03\n\tKafkaMesh\x12 \n\x0f\x61\x64vertised_host\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12 \n\x0f\x61\x64vertised_port\x18\x02 \x01(\x05\x42\x07\xfa\x42\x04\x1a\x02 \x00\x12\x66\n\x11upstream_clusters\x18\x03 \x03(\x0b\x32K.envoy.extensions.filters.network.kafka_mesh.v3alpha.KafkaClusterDefinition\x12]\n\x10\x66orwarding_rules\x18\x04 \x03(\x0b\x32\x43.envoy.extensions.filters.network.kafka_mesh.v3alpha.ForwardingRule\x12m\n\x13\x63onsumer_proxy_mode\x18\x05 \x01(\x0e\x32P.envoy.extensions.filters.network.kafka_mesh.v3alpha.KafkaMesh.ConsumerProxyMode\".\n\x11\x43onsumerProxyMode\x12\x19\n\x15StatefulConsumerProxy\x10\x00\"\xdf\x03\n\x16KafkaClusterDefinition\x12\x1d\n\x0c\x63luster_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\"\n\x11\x62ootstrap_servers\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12 \n\x0fpartition_count\x18\x03 \x01(\x05\x42\x07\xfa\x42\x04\x1a\x02 \x00\x12x\n\x0fproducer_config\x18\x04 \x03(\x0b\x32_.envoy.extensions.filters.network.kafka_mesh.v3alpha.KafkaClusterDefinition.ProducerConfigEntry\x12x\n\x0f\x63onsumer_config\x18\x05 \x03(\x0b\x32_.envoy.extensions.filters.network.kafka_mesh.v3alpha.KafkaClusterDefinition.ConsumerConfigEntry\x1a\x35\n\x13ProducerConfigEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x35\n\x13\x43onsumerConfigEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"K\n\x0e\x46orwardingRule\x12\x16\n\x0etarget_cluster\x18\x01 \x01(\t\x12\x16\n\x0ctopic_prefix\x18\x02 \x01(\tH\x00\x42\t\n\x07triggerB\xc9\x01\nAio.envoyproxy.envoy.extensions.filters.network.kafka_mesh.v3alphaB\x0eKafkaMeshProtoP\x01Zbgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/kafka_mesh/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.kafka_mesh.v3alpha.kafka_mesh_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nAio.envoyproxy.envoy.extensions.filters.network.kafka_mesh.v3alphaB\016KafkaMeshProtoP\001Zbgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/kafka_mesh/v3alpha\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_KAFKAMESH'].fields_by_name['advertised_host']._loaded_options = None
  _globals['_KAFKAMESH'].fields_by_name['advertised_host']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_KAFKAMESH'].fields_by_name['advertised_port']._loaded_options = None
  _globals['_KAFKAMESH'].fields_by_name['advertised_port']._serialized_options = b'\372B\004\032\002 \000'
  _globals['_KAFKACLUSTERDEFINITION_PRODUCERCONFIGENTRY']._loaded_options = None
  _globals['_KAFKACLUSTERDEFINITION_PRODUCERCONFIGENTRY']._serialized_options = b'8\001'
  _globals['_KAFKACLUSTERDEFINITION_CONSUMERCONFIGENTRY']._loaded_options = None
  _globals['_KAFKACLUSTERDEFINITION_CONSUMERCONFIGENTRY']._serialized_options = b'8\001'
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['cluster_name']._loaded_options = None
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['cluster_name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['bootstrap_servers']._loaded_options = None
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['bootstrap_servers']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['partition_count']._loaded_options = None
  _globals['_KAFKACLUSTERDEFINITION'].fields_by_name['partition_count']._serialized_options = b'\372B\004\032\002 \000'
  _globals['_KAFKAMESH']._serialized_start=223
  _globals['_KAFKAMESH']._serialized_end=660
  _globals['_KAFKAMESH_CONSUMERPROXYMODE']._serialized_start=614
  _globals['_KAFKAMESH_CONSUMERPROXYMODE']._serialized_end=660
  _globals['_KAFKACLUSTERDEFINITION']._serialized_start=663
  _globals['_KAFKACLUSTERDEFINITION']._serialized_end=1142
  _globals['_KAFKACLUSTERDEFINITION_PRODUCERCONFIGENTRY']._serialized_start=1034
  _globals['_KAFKACLUSTERDEFINITION_PRODUCERCONFIGENTRY']._serialized_end=1087
  _globals['_KAFKACLUSTERDEFINITION_CONSUMERCONFIGENTRY']._serialized_start=1089
  _globals['_KAFKACLUSTERDEFINITION_CONSUMERCONFIGENTRY']._serialized_end=1142
  _globals['_FORWARDINGRULE']._serialized_start=1144
  _globals['_FORWARDINGRULE']._serialized_end=1219
# @@protoc_insertion_point(module_scope)
