# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/access_loggers/filters/process_ratelimit/v3/process_ratelimit.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/access_loggers/filters/process_ratelimit/v3/process_ratelimit.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import config_source_pb2 as envoy_dot_config_dot_core_dot_v3_dot_config__source__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nTenvoy/extensions/access_loggers/filters/process_ratelimit/v3/process_ratelimit.proto\x12<envoy.extensions.access_loggers.filters.process_ratelimit.v3\x1a(envoy/config/core/v3/config_source.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x82\x01\n\x16ProcessRateLimitFilter\x12h\n\x0e\x64ynamic_config\x18\x01 \x01(\x0b\x32P.envoy.extensions.access_loggers.filters.process_ratelimit.v3.DynamicTokenBucket\"y\n\x12\x44ynamicTokenBucket\x12\x1e\n\rresource_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x43\n\rconfig_source\x18\x02 \x01(\x0b\x32\".envoy.config.core.v3.ConfigSourceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x42\xe6\x01\nJio.envoyproxy.envoy.extensions.access_loggers.filters.process_ratelimit.v3B\x15ProcessRatelimitProtoP\x01Zwgithub.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/filters/process_ratelimit/v3;process_ratelimitv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.access_loggers.filters.process_ratelimit.v3.process_ratelimit_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nJio.envoyproxy.envoy.extensions.access_loggers.filters.process_ratelimit.v3B\025ProcessRatelimitProtoP\001Zwgithub.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/filters/process_ratelimit/v3;process_ratelimitv3\272\200\310\321\006\002\020\002'
  _globals['_DYNAMICTOKENBUCKET'].fields_by_name['resource_name']._loaded_options = None
  _globals['_DYNAMICTOKENBUCKET'].fields_by_name['resource_name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_DYNAMICTOKENBUCKET'].fields_by_name['config_source']._loaded_options = None
  _globals['_DYNAMICTOKENBUCKET'].fields_by_name['config_source']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_PROCESSRATELIMITFILTER']._serialized_start=249
  _globals['_PROCESSRATELIMITFILTER']._serialized_end=379
  _globals['_DYNAMICTOKENBUCKET']._serialized_start=381
  _globals['_DYNAMICTOKENBUCKET']._serialized_end=502
# @@protoc_insertion_point(module_scope)
