// Copyright 2023 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Auto generated by tools/codegen/core/gen_experiments.py
//
// This file contains the autogenerated parts of the experiments API.
//
// It generates two symbols for each experiment.
//
// For the experiment named new_car_project, it generates:
//
// - a function IsNewCarProjectEnabled() that returns true if the experiment
//   should be enabled at runtime.
//
// - a macro GRPC_EXPERIMENT_IS_INCLUDED_NEW_CAR_PROJECT that is defined if the
//   experiment *could* be enabled at runtime.
//
// The function is used to determine whether to run the experiment or
// non-experiment code path.
//
// If the experiment brings significant bloat, the macro can be used to avoid
// including the experiment code path in the binary for binaries that are size
// sensitive.
//
// By default that includes our iOS and Android builds.
//
// Finally, a small array is included that contains the metadata for each
// experiment.
//
// A macro, GRPC_EXPERIMENTS_ARE_FINAL, controls whether we fix experiment
// configuration at build time (if it's defined) or allow it to be tuned at
// runtime (if it's disabled).
//
// If you are using the Bazel build system, that macro can be configured with
// --define=grpc_experiments_are_final=true

#ifndef GRPC_SRC_CORE_LIB_EXPERIMENTS_EXPERIMENTS_H
#define GRPC_SRC_CORE_LIB_EXPERIMENTS_EXPERIMENTS_H

#include <grpc/support/port_platform.h>

#include "src/core/lib/experiments/config.h"

namespace grpc_core {

#ifdef GRPC_EXPERIMENTS_ARE_FINAL

#if defined(GRPC_CFSTREAM)
inline bool IsBufferListDeletionPrepEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CALL_TRACER_IN_TRANSPORT
inline bool IsCallTracerInTransportEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CALL_TRACER_SEND_INITIAL_METADATA_IS_AN_ANNOTATION
inline bool IsCallTracerSendInitialMetadataIsAnAnnotationEnabled() {
  return true;
}
inline bool IsCallTracerSendTrailingMetadataIsAnAnnotationEnabled() {
  return false;
}
#define GRPC_EXPERIMENT_IS_INCLUDED_CALLV3_BATCH_VALIDATION
inline bool IsCallv3BatchValidationEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CHAOTIC_GOOD_FRAMING_LAYER
inline bool IsChaoticGoodFramingLayerEnabled() { return true; }
inline bool IsChaoticGoodSendSupportedFeaturesEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CUSTOM_FRAME_CHECK
inline bool IsCustomFrameCheckEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_ERROR_FLATTEN
inline bool IsErrorFlattenEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_CLIENT
inline bool IsEventEngineClientEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_DNS
inline bool IsEventEngineDnsEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_DNS_NON_CLIENT_CHANNEL
inline bool IsEventEngineDnsNonClientChannelEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_FORK
inline bool IsEventEngineForkEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_LISTENER
inline bool IsEventEngineListenerEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_CALLBACK_CQ
inline bool IsEventEngineCallbackCqEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_FOR_ALL_OTHER_ENDPOINTS
inline bool IsEventEngineForAllOtherEndpointsEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_POLLER_FOR_PYTHON
inline bool IsEventEnginePollerForPythonEnabled() { return true; }
inline bool IsFailRecvMetadataOnDeadlineExceededEnabled() { return false; }
inline bool IsFreeLargeAllocatorEnabled() { return false; }
inline bool IsFuseFiltersEnabled() { return false; }
inline bool IsH2MaxDeallocatingStreamsHeadroomEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_INPROC_CANCEL_STREAM
inline bool IsInprocCancelStreamEnabled() { return true; }
inline bool IsKeepAlivePingTimerBatchEnabled() { return false; }
inline bool IsLocalConnectorSecureEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_MAX_INFLIGHT_PINGS_STRICT_LIMIT
inline bool IsMaxInflightPingsStrictLimitEnabled() { return true; }
inline bool IsMemoryOptimization01Enabled() { return false; }
inline bool IsMemoryOptimization02Enabled() { return false; }
inline bool IsMessageSizeRefactoringEnabled() { return false; }
inline bool IsMetadataOutstandingTokenRefactorEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_METADATA_PUBLISH_TO_APP_TAG
inline bool IsMetadataPublishToAppTagEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_MONITORING_EXPERIMENT
inline bool IsMonitoringExperimentEnabled() { return true; }
inline bool IsMultipingEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_01
inline bool IsOptimization01Enabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_02
inline bool IsOptimization02Enabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_03
inline bool IsOptimization03Enabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_04
inline bool IsOptimization04Enabled() { return true; }
inline bool IsOtelExportTelemetryDomainsEnabled() { return false; }
inline bool IsPh2ClientEnabled() { return false; }
inline bool IsPh2ClientServerEnabled() { return false; }
inline bool IsPh2Perf01Enabled() { return false; }
inline bool IsPh2ServerEnabled() { return false; }
inline bool IsPickFirstIgnoreEmptyUpdatesEnabled() { return false; }
inline bool IsPipelinedReadSecureEndpointEnabled() { return false; }
inline bool IsPollsetAlternativeEnabled() { return false; }
inline bool IsPrioritizeFinishedRequestsEnabled() { return false; }
inline bool IsPromiseBasedInprocTransportEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_PROMISE_BATCH_CLEANUP_ON_CANCEL
inline bool IsPromiseBatchCleanupOnCancelEnabled() { return true; }
inline bool IsPromiseFilterSendCancelMetadataEnabled() { return false; }
inline bool IsRetryInCallv3Enabled() { return false; }
inline bool IsReturnPreexistingErrorsEnabled() { return false; }
inline bool IsScheduleCancellationOverWriteEnabled() { return false; }
inline bool IsSecureEndpointOffloadLargeReadsEnabled() { return false; }
inline bool IsSecureEndpointOffloadLargeWritesEnabled() { return false; }
inline bool IsSecureEndpointReadCoalescingEnabled() { return false; }
inline bool IsSkipClearPeerOnCancellationEnabled() { return false; }
inline bool IsSubchannelConnectionScalingEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_SUBCHANNEL_WRAPPER_CLEANUP_ON_ORPHAN
inline bool IsSubchannelWrapperCleanupOnOrphanEnabled() { return true; }
inline bool IsTcpFrameSizeTuningEnabled() { return false; }
inline bool IsTcpRcvLowatEnabled() { return false; }
inline bool IsTsiFrameProtectorWithoutLocksEnabled() { return false; }
inline bool IsUnconstrainedMaxQuotaBufferSizeEnabled() { return false; }
inline bool IsUseCallEventEngineInCompletionQueueEnabled() { return false; }
inline bool IsV2NonOwningWakerImplementationEnabled() { return false; }
inline bool IsVerboseChannelzConnectionLoggingEnabled() { return false; }
inline bool IsWildcardIpExpansionRestrictionEnabled() { return false; }
inline bool IsXdsServerFilterChainPerRouteEnabled() { return false; }

#elif defined(GPR_WINDOWS)
inline bool IsBufferListDeletionPrepEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CALL_TRACER_IN_TRANSPORT
inline bool IsCallTracerInTransportEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CALL_TRACER_SEND_INITIAL_METADATA_IS_AN_ANNOTATION
inline bool IsCallTracerSendInitialMetadataIsAnAnnotationEnabled() {
  return true;
}
inline bool IsCallTracerSendTrailingMetadataIsAnAnnotationEnabled() {
  return false;
}
#define GRPC_EXPERIMENT_IS_INCLUDED_CALLV3_BATCH_VALIDATION
inline bool IsCallv3BatchValidationEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CHAOTIC_GOOD_FRAMING_LAYER
inline bool IsChaoticGoodFramingLayerEnabled() { return true; }
inline bool IsChaoticGoodSendSupportedFeaturesEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CUSTOM_FRAME_CHECK
inline bool IsCustomFrameCheckEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_ERROR_FLATTEN
inline bool IsErrorFlattenEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_CLIENT
inline bool IsEventEngineClientEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_DNS
inline bool IsEventEngineDnsEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_DNS_NON_CLIENT_CHANNEL
inline bool IsEventEngineDnsNonClientChannelEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_FORK
inline bool IsEventEngineForkEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_LISTENER
inline bool IsEventEngineListenerEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_CALLBACK_CQ
inline bool IsEventEngineCallbackCqEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_FOR_ALL_OTHER_ENDPOINTS
inline bool IsEventEngineForAllOtherEndpointsEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_POLLER_FOR_PYTHON
inline bool IsEventEnginePollerForPythonEnabled() { return true; }
inline bool IsFailRecvMetadataOnDeadlineExceededEnabled() { return false; }
inline bool IsFreeLargeAllocatorEnabled() { return false; }
inline bool IsFuseFiltersEnabled() { return false; }
inline bool IsH2MaxDeallocatingStreamsHeadroomEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_INPROC_CANCEL_STREAM
inline bool IsInprocCancelStreamEnabled() { return true; }
inline bool IsKeepAlivePingTimerBatchEnabled() { return false; }
inline bool IsLocalConnectorSecureEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_MAX_INFLIGHT_PINGS_STRICT_LIMIT
inline bool IsMaxInflightPingsStrictLimitEnabled() { return true; }
inline bool IsMemoryOptimization01Enabled() { return false; }
inline bool IsMemoryOptimization02Enabled() { return false; }
inline bool IsMessageSizeRefactoringEnabled() { return false; }
inline bool IsMetadataOutstandingTokenRefactorEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_METADATA_PUBLISH_TO_APP_TAG
inline bool IsMetadataPublishToAppTagEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_MONITORING_EXPERIMENT
inline bool IsMonitoringExperimentEnabled() { return true; }
inline bool IsMultipingEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_01
inline bool IsOptimization01Enabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_02
inline bool IsOptimization02Enabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_03
inline bool IsOptimization03Enabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_04
inline bool IsOptimization04Enabled() { return true; }
inline bool IsOtelExportTelemetryDomainsEnabled() { return false; }
inline bool IsPh2ClientEnabled() { return false; }
inline bool IsPh2ClientServerEnabled() { return false; }
inline bool IsPh2Perf01Enabled() { return false; }
inline bool IsPh2ServerEnabled() { return false; }
inline bool IsPickFirstIgnoreEmptyUpdatesEnabled() { return false; }
inline bool IsPipelinedReadSecureEndpointEnabled() { return false; }
inline bool IsPollsetAlternativeEnabled() { return false; }
inline bool IsPrioritizeFinishedRequestsEnabled() { return false; }
inline bool IsPromiseBasedInprocTransportEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_PROMISE_BATCH_CLEANUP_ON_CANCEL
inline bool IsPromiseBatchCleanupOnCancelEnabled() { return true; }
inline bool IsPromiseFilterSendCancelMetadataEnabled() { return false; }
inline bool IsRetryInCallv3Enabled() { return false; }
inline bool IsReturnPreexistingErrorsEnabled() { return false; }
inline bool IsScheduleCancellationOverWriteEnabled() { return false; }
inline bool IsSecureEndpointOffloadLargeReadsEnabled() { return false; }
inline bool IsSecureEndpointOffloadLargeWritesEnabled() { return false; }
inline bool IsSecureEndpointReadCoalescingEnabled() { return false; }
inline bool IsSkipClearPeerOnCancellationEnabled() { return false; }
inline bool IsSubchannelConnectionScalingEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_SUBCHANNEL_WRAPPER_CLEANUP_ON_ORPHAN
inline bool IsSubchannelWrapperCleanupOnOrphanEnabled() { return true; }
inline bool IsTcpFrameSizeTuningEnabled() { return false; }
inline bool IsTcpRcvLowatEnabled() { return false; }
inline bool IsTsiFrameProtectorWithoutLocksEnabled() { return false; }
inline bool IsUnconstrainedMaxQuotaBufferSizeEnabled() { return false; }
inline bool IsUseCallEventEngineInCompletionQueueEnabled() { return false; }
inline bool IsV2NonOwningWakerImplementationEnabled() { return false; }
inline bool IsVerboseChannelzConnectionLoggingEnabled() { return false; }
inline bool IsWildcardIpExpansionRestrictionEnabled() { return false; }
inline bool IsXdsServerFilterChainPerRouteEnabled() { return false; }

#else
inline bool IsBufferListDeletionPrepEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CALL_TRACER_IN_TRANSPORT
inline bool IsCallTracerInTransportEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CALL_TRACER_SEND_INITIAL_METADATA_IS_AN_ANNOTATION
inline bool IsCallTracerSendInitialMetadataIsAnAnnotationEnabled() {
  return true;
}
inline bool IsCallTracerSendTrailingMetadataIsAnAnnotationEnabled() {
  return false;
}
#define GRPC_EXPERIMENT_IS_INCLUDED_CALLV3_BATCH_VALIDATION
inline bool IsCallv3BatchValidationEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CHAOTIC_GOOD_FRAMING_LAYER
inline bool IsChaoticGoodFramingLayerEnabled() { return true; }
inline bool IsChaoticGoodSendSupportedFeaturesEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_CUSTOM_FRAME_CHECK
inline bool IsCustomFrameCheckEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_ERROR_FLATTEN
inline bool IsErrorFlattenEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_CLIENT
inline bool IsEventEngineClientEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_DNS
inline bool IsEventEngineDnsEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_DNS_NON_CLIENT_CHANNEL
inline bool IsEventEngineDnsNonClientChannelEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_FORK
inline bool IsEventEngineForkEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_LISTENER
inline bool IsEventEngineListenerEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_CALLBACK_CQ
inline bool IsEventEngineCallbackCqEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_FOR_ALL_OTHER_ENDPOINTS
inline bool IsEventEngineForAllOtherEndpointsEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_POLLER_FOR_PYTHON
inline bool IsEventEnginePollerForPythonEnabled() { return true; }
inline bool IsFailRecvMetadataOnDeadlineExceededEnabled() { return false; }
inline bool IsFreeLargeAllocatorEnabled() { return false; }
inline bool IsFuseFiltersEnabled() { return false; }
inline bool IsH2MaxDeallocatingStreamsHeadroomEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_INPROC_CANCEL_STREAM
inline bool IsInprocCancelStreamEnabled() { return true; }
inline bool IsKeepAlivePingTimerBatchEnabled() { return false; }
inline bool IsLocalConnectorSecureEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_MAX_INFLIGHT_PINGS_STRICT_LIMIT
inline bool IsMaxInflightPingsStrictLimitEnabled() { return true; }
inline bool IsMemoryOptimization01Enabled() { return false; }
inline bool IsMemoryOptimization02Enabled() { return false; }
inline bool IsMessageSizeRefactoringEnabled() { return false; }
inline bool IsMetadataOutstandingTokenRefactorEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_METADATA_PUBLISH_TO_APP_TAG
inline bool IsMetadataPublishToAppTagEnabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_MONITORING_EXPERIMENT
inline bool IsMonitoringExperimentEnabled() { return true; }
inline bool IsMultipingEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_01
inline bool IsOptimization01Enabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_02
inline bool IsOptimization02Enabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_03
inline bool IsOptimization03Enabled() { return true; }
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_04
inline bool IsOptimization04Enabled() { return true; }
inline bool IsOtelExportTelemetryDomainsEnabled() { return false; }
inline bool IsPh2ClientEnabled() { return false; }
inline bool IsPh2ClientServerEnabled() { return false; }
inline bool IsPh2Perf01Enabled() { return false; }
inline bool IsPh2ServerEnabled() { return false; }
inline bool IsPickFirstIgnoreEmptyUpdatesEnabled() { return false; }
inline bool IsPipelinedReadSecureEndpointEnabled() { return false; }
inline bool IsPollsetAlternativeEnabled() { return false; }
inline bool IsPrioritizeFinishedRequestsEnabled() { return false; }
inline bool IsPromiseBasedInprocTransportEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_PROMISE_BATCH_CLEANUP_ON_CANCEL
inline bool IsPromiseBatchCleanupOnCancelEnabled() { return true; }
inline bool IsPromiseFilterSendCancelMetadataEnabled() { return false; }
inline bool IsRetryInCallv3Enabled() { return false; }
inline bool IsReturnPreexistingErrorsEnabled() { return false; }
inline bool IsScheduleCancellationOverWriteEnabled() { return false; }
inline bool IsSecureEndpointOffloadLargeReadsEnabled() { return false; }
inline bool IsSecureEndpointOffloadLargeWritesEnabled() { return false; }
inline bool IsSecureEndpointReadCoalescingEnabled() { return false; }
inline bool IsSkipClearPeerOnCancellationEnabled() { return false; }
inline bool IsSubchannelConnectionScalingEnabled() { return false; }
#define GRPC_EXPERIMENT_IS_INCLUDED_SUBCHANNEL_WRAPPER_CLEANUP_ON_ORPHAN
inline bool IsSubchannelWrapperCleanupOnOrphanEnabled() { return true; }
inline bool IsTcpFrameSizeTuningEnabled() { return false; }
inline bool IsTcpRcvLowatEnabled() { return false; }
inline bool IsTsiFrameProtectorWithoutLocksEnabled() { return false; }
inline bool IsUnconstrainedMaxQuotaBufferSizeEnabled() { return false; }
inline bool IsUseCallEventEngineInCompletionQueueEnabled() { return false; }
inline bool IsV2NonOwningWakerImplementationEnabled() { return false; }
inline bool IsVerboseChannelzConnectionLoggingEnabled() { return false; }
inline bool IsWildcardIpExpansionRestrictionEnabled() { return false; }
inline bool IsXdsServerFilterChainPerRouteEnabled() { return false; }
#endif

#else
enum ExperimentIds {
  kExperimentIdBufferListDeletionPrep,
  kExperimentIdCallTracerInTransport,
  kExperimentIdCallTracerSendInitialMetadataIsAnAnnotation,
  kExperimentIdCallTracerSendTrailingMetadataIsAnAnnotation,
  kExperimentIdCallv3BatchValidation,
  kExperimentIdChaoticGoodFramingLayer,
  kExperimentIdChaoticGoodSendSupportedFeatures,
  kExperimentIdCustomFrameCheck,
  kExperimentIdErrorFlatten,
  kExperimentIdEventEngineClient,
  kExperimentIdEventEngineDns,
  kExperimentIdEventEngineDnsNonClientChannel,
  kExperimentIdEventEngineFork,
  kExperimentIdEventEngineListener,
  kExperimentIdEventEngineCallbackCq,
  kExperimentIdEventEngineForAllOtherEndpoints,
  kExperimentIdEventEnginePollerForPython,
  kExperimentIdFailRecvMetadataOnDeadlineExceeded,
  kExperimentIdFreeLargeAllocator,
  kExperimentIdFuseFilters,
  kExperimentIdH2MaxDeallocatingStreamsHeadroom,
  kExperimentIdInprocCancelStream,
  kExperimentIdKeepAlivePingTimerBatch,
  kExperimentIdLocalConnectorSecure,
  kExperimentIdMaxInflightPingsStrictLimit,
  kExperimentIdMemoryOptimization01,
  kExperimentIdMemoryOptimization02,
  kExperimentIdMessageSizeRefactoring,
  kExperimentIdMetadataOutstandingTokenRefactor,
  kExperimentIdMetadataPublishToAppTag,
  kExperimentIdMonitoringExperiment,
  kExperimentIdMultiping,
  kExperimentIdOptimization01,
  kExperimentIdOptimization02,
  kExperimentIdOptimization03,
  kExperimentIdOptimization04,
  kExperimentIdOtelExportTelemetryDomains,
  kExperimentIdPh2Client,
  kExperimentIdPh2ClientServer,
  kExperimentIdPh2Perf01,
  kExperimentIdPh2Server,
  kExperimentIdPickFirstIgnoreEmptyUpdates,
  kExperimentIdPipelinedReadSecureEndpoint,
  kExperimentIdPollsetAlternative,
  kExperimentIdPrioritizeFinishedRequests,
  kExperimentIdPromiseBasedInprocTransport,
  kExperimentIdPromiseBatchCleanupOnCancel,
  kExperimentIdPromiseFilterSendCancelMetadata,
  kExperimentIdRetryInCallv3,
  kExperimentIdReturnPreexistingErrors,
  kExperimentIdScheduleCancellationOverWrite,
  kExperimentIdSecureEndpointOffloadLargeReads,
  kExperimentIdSecureEndpointOffloadLargeWrites,
  kExperimentIdSecureEndpointReadCoalescing,
  kExperimentIdSkipClearPeerOnCancellation,
  kExperimentIdSubchannelConnectionScaling,
  kExperimentIdSubchannelWrapperCleanupOnOrphan,
  kExperimentIdTcpFrameSizeTuning,
  kExperimentIdTcpRcvLowat,
  kExperimentIdTsiFrameProtectorWithoutLocks,
  kExperimentIdUnconstrainedMaxQuotaBufferSize,
  kExperimentIdUseCallEventEngineInCompletionQueue,
  kExperimentIdV2NonOwningWakerImplementation,
  kExperimentIdVerboseChannelzConnectionLogging,
  kExperimentIdWildcardIpExpansionRestriction,
  kExperimentIdXdsServerFilterChainPerRoute,
  kNumExperiments
};
#define GRPC_EXPERIMENT_IS_INCLUDED_BUFFER_LIST_DELETION_PREP
inline bool IsBufferListDeletionPrepEnabled() {
  return IsExperimentEnabled<kExperimentIdBufferListDeletionPrep>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_CALL_TRACER_IN_TRANSPORT
inline bool IsCallTracerInTransportEnabled() {
  return IsExperimentEnabled<kExperimentIdCallTracerInTransport>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_CALL_TRACER_SEND_INITIAL_METADATA_IS_AN_ANNOTATION
inline bool IsCallTracerSendInitialMetadataIsAnAnnotationEnabled() {
  return IsExperimentEnabled<
      kExperimentIdCallTracerSendInitialMetadataIsAnAnnotation>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_CALL_TRACER_SEND_TRAILING_METADATA_IS_AN_ANNOTATION
inline bool IsCallTracerSendTrailingMetadataIsAnAnnotationEnabled() {
  return IsExperimentEnabled<
      kExperimentIdCallTracerSendTrailingMetadataIsAnAnnotation>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_CALLV3_BATCH_VALIDATION
inline bool IsCallv3BatchValidationEnabled() {
  return IsExperimentEnabled<kExperimentIdCallv3BatchValidation>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_CHAOTIC_GOOD_FRAMING_LAYER
inline bool IsChaoticGoodFramingLayerEnabled() {
  return IsExperimentEnabled<kExperimentIdChaoticGoodFramingLayer>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_CHAOTIC_GOOD_SEND_SUPPORTED_FEATURES
inline bool IsChaoticGoodSendSupportedFeaturesEnabled() {
  return IsExperimentEnabled<kExperimentIdChaoticGoodSendSupportedFeatures>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_CUSTOM_FRAME_CHECK
inline bool IsCustomFrameCheckEnabled() {
  return IsExperimentEnabled<kExperimentIdCustomFrameCheck>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_ERROR_FLATTEN
inline bool IsErrorFlattenEnabled() {
  return IsExperimentEnabled<kExperimentIdErrorFlatten>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_CLIENT
inline bool IsEventEngineClientEnabled() {
  return IsExperimentEnabled<kExperimentIdEventEngineClient>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_DNS
inline bool IsEventEngineDnsEnabled() {
  return IsExperimentEnabled<kExperimentIdEventEngineDns>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_DNS_NON_CLIENT_CHANNEL
inline bool IsEventEngineDnsNonClientChannelEnabled() {
  return IsExperimentEnabled<kExperimentIdEventEngineDnsNonClientChannel>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_FORK
inline bool IsEventEngineForkEnabled() {
  return IsExperimentEnabled<kExperimentIdEventEngineFork>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_LISTENER
inline bool IsEventEngineListenerEnabled() {
  return IsExperimentEnabled<kExperimentIdEventEngineListener>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_CALLBACK_CQ
inline bool IsEventEngineCallbackCqEnabled() {
  return IsExperimentEnabled<kExperimentIdEventEngineCallbackCq>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_FOR_ALL_OTHER_ENDPOINTS
inline bool IsEventEngineForAllOtherEndpointsEnabled() {
  return IsExperimentEnabled<kExperimentIdEventEngineForAllOtherEndpoints>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_EVENT_ENGINE_POLLER_FOR_PYTHON
inline bool IsEventEnginePollerForPythonEnabled() {
  return IsExperimentEnabled<kExperimentIdEventEnginePollerForPython>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_FAIL_RECV_METADATA_ON_DEADLINE_EXCEEDED
inline bool IsFailRecvMetadataOnDeadlineExceededEnabled() {
  return IsExperimentEnabled<kExperimentIdFailRecvMetadataOnDeadlineExceeded>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_FREE_LARGE_ALLOCATOR
inline bool IsFreeLargeAllocatorEnabled() {
  return IsExperimentEnabled<kExperimentIdFreeLargeAllocator>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_FUSE_FILTERS
inline bool IsFuseFiltersEnabled() {
  return IsExperimentEnabled<kExperimentIdFuseFilters>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_H2_MAX_DEALLOCATING_STREAMS_HEADROOM
inline bool IsH2MaxDeallocatingStreamsHeadroomEnabled() {
  return IsExperimentEnabled<kExperimentIdH2MaxDeallocatingStreamsHeadroom>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_INPROC_CANCEL_STREAM
inline bool IsInprocCancelStreamEnabled() {
  return IsExperimentEnabled<kExperimentIdInprocCancelStream>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_KEEP_ALIVE_PING_TIMER_BATCH
inline bool IsKeepAlivePingTimerBatchEnabled() {
  return IsExperimentEnabled<kExperimentIdKeepAlivePingTimerBatch>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_LOCAL_CONNECTOR_SECURE
inline bool IsLocalConnectorSecureEnabled() {
  return IsExperimentEnabled<kExperimentIdLocalConnectorSecure>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_MAX_INFLIGHT_PINGS_STRICT_LIMIT
inline bool IsMaxInflightPingsStrictLimitEnabled() {
  return IsExperimentEnabled<kExperimentIdMaxInflightPingsStrictLimit>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_MEMORY_OPTIMIZATION_01
inline bool IsMemoryOptimization01Enabled() {
  return IsExperimentEnabled<kExperimentIdMemoryOptimization01>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_MEMORY_OPTIMIZATION_02
inline bool IsMemoryOptimization02Enabled() {
  return IsExperimentEnabled<kExperimentIdMemoryOptimization02>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_MESSAGE_SIZE_REFACTORING
inline bool IsMessageSizeRefactoringEnabled() {
  return IsExperimentEnabled<kExperimentIdMessageSizeRefactoring>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_METADATA_OUTSTANDING_TOKEN_REFACTOR
inline bool IsMetadataOutstandingTokenRefactorEnabled() {
  return IsExperimentEnabled<kExperimentIdMetadataOutstandingTokenRefactor>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_METADATA_PUBLISH_TO_APP_TAG
inline bool IsMetadataPublishToAppTagEnabled() {
  return IsExperimentEnabled<kExperimentIdMetadataPublishToAppTag>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_MONITORING_EXPERIMENT
inline bool IsMonitoringExperimentEnabled() {
  return IsExperimentEnabled<kExperimentIdMonitoringExperiment>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_MULTIPING
inline bool IsMultipingEnabled() {
  return IsExperimentEnabled<kExperimentIdMultiping>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_01
inline bool IsOptimization01Enabled() {
  return IsExperimentEnabled<kExperimentIdOptimization01>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_02
inline bool IsOptimization02Enabled() {
  return IsExperimentEnabled<kExperimentIdOptimization02>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_03
inline bool IsOptimization03Enabled() {
  return IsExperimentEnabled<kExperimentIdOptimization03>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_OPTIMIZATION_04
inline bool IsOptimization04Enabled() {
  return IsExperimentEnabled<kExperimentIdOptimization04>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_OTEL_EXPORT_TELEMETRY_DOMAINS
inline bool IsOtelExportTelemetryDomainsEnabled() {
  return IsExperimentEnabled<kExperimentIdOtelExportTelemetryDomains>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_PH2_CLIENT
inline bool IsPh2ClientEnabled() {
  return IsExperimentEnabled<kExperimentIdPh2Client>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_PH2_CLIENT_SERVER
inline bool IsPh2ClientServerEnabled() {
  return IsExperimentEnabled<kExperimentIdPh2ClientServer>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_PH2_PERF_01
inline bool IsPh2Perf01Enabled() {
  return IsExperimentEnabled<kExperimentIdPh2Perf01>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_PH2_SERVER
inline bool IsPh2ServerEnabled() {
  return IsExperimentEnabled<kExperimentIdPh2Server>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_PICK_FIRST_IGNORE_EMPTY_UPDATES
inline bool IsPickFirstIgnoreEmptyUpdatesEnabled() {
  return IsExperimentEnabled<kExperimentIdPickFirstIgnoreEmptyUpdates>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_PIPELINED_READ_SECURE_ENDPOINT
inline bool IsPipelinedReadSecureEndpointEnabled() {
  return IsExperimentEnabled<kExperimentIdPipelinedReadSecureEndpoint>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_POLLSET_ALTERNATIVE
inline bool IsPollsetAlternativeEnabled() {
  return IsExperimentEnabled<kExperimentIdPollsetAlternative>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_PRIORITIZE_FINISHED_REQUESTS
inline bool IsPrioritizeFinishedRequestsEnabled() {
  return IsExperimentEnabled<kExperimentIdPrioritizeFinishedRequests>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_PROMISE_BASED_INPROC_TRANSPORT
inline bool IsPromiseBasedInprocTransportEnabled() {
  return IsExperimentEnabled<kExperimentIdPromiseBasedInprocTransport>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_PROMISE_BATCH_CLEANUP_ON_CANCEL
inline bool IsPromiseBatchCleanupOnCancelEnabled() {
  return IsExperimentEnabled<kExperimentIdPromiseBatchCleanupOnCancel>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_PROMISE_FILTER_SEND_CANCEL_METADATA
inline bool IsPromiseFilterSendCancelMetadataEnabled() {
  return IsExperimentEnabled<kExperimentIdPromiseFilterSendCancelMetadata>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_RETRY_IN_CALLV3
inline bool IsRetryInCallv3Enabled() {
  return IsExperimentEnabled<kExperimentIdRetryInCallv3>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_RETURN_PREEXISTING_ERRORS
inline bool IsReturnPreexistingErrorsEnabled() {
  return IsExperimentEnabled<kExperimentIdReturnPreexistingErrors>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_SCHEDULE_CANCELLATION_OVER_WRITE
inline bool IsScheduleCancellationOverWriteEnabled() {
  return IsExperimentEnabled<kExperimentIdScheduleCancellationOverWrite>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_SECURE_ENDPOINT_OFFLOAD_LARGE_READS
inline bool IsSecureEndpointOffloadLargeReadsEnabled() {
  return IsExperimentEnabled<kExperimentIdSecureEndpointOffloadLargeReads>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_SECURE_ENDPOINT_OFFLOAD_LARGE_WRITES
inline bool IsSecureEndpointOffloadLargeWritesEnabled() {
  return IsExperimentEnabled<kExperimentIdSecureEndpointOffloadLargeWrites>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_SECURE_ENDPOINT_READ_COALESCING
inline bool IsSecureEndpointReadCoalescingEnabled() {
  return IsExperimentEnabled<kExperimentIdSecureEndpointReadCoalescing>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_SKIP_CLEAR_PEER_ON_CANCELLATION
inline bool IsSkipClearPeerOnCancellationEnabled() {
  return IsExperimentEnabled<kExperimentIdSkipClearPeerOnCancellation>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_SUBCHANNEL_CONNECTION_SCALING
inline bool IsSubchannelConnectionScalingEnabled() {
  return IsExperimentEnabled<kExperimentIdSubchannelConnectionScaling>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_SUBCHANNEL_WRAPPER_CLEANUP_ON_ORPHAN
inline bool IsSubchannelWrapperCleanupOnOrphanEnabled() {
  return IsExperimentEnabled<kExperimentIdSubchannelWrapperCleanupOnOrphan>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_TCP_FRAME_SIZE_TUNING
inline bool IsTcpFrameSizeTuningEnabled() {
  return IsExperimentEnabled<kExperimentIdTcpFrameSizeTuning>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_TCP_RCV_LOWAT
inline bool IsTcpRcvLowatEnabled() {
  return IsExperimentEnabled<kExperimentIdTcpRcvLowat>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_TSI_FRAME_PROTECTOR_WITHOUT_LOCKS
inline bool IsTsiFrameProtectorWithoutLocksEnabled() {
  return IsExperimentEnabled<kExperimentIdTsiFrameProtectorWithoutLocks>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_UNCONSTRAINED_MAX_QUOTA_BUFFER_SIZE
inline bool IsUnconstrainedMaxQuotaBufferSizeEnabled() {
  return IsExperimentEnabled<kExperimentIdUnconstrainedMaxQuotaBufferSize>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_USE_CALL_EVENT_ENGINE_IN_COMPLETION_QUEUE
inline bool IsUseCallEventEngineInCompletionQueueEnabled() {
  return IsExperimentEnabled<
      kExperimentIdUseCallEventEngineInCompletionQueue>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_V2_NON_OWNING_WAKER_IMPLEMENTATION
inline bool IsV2NonOwningWakerImplementationEnabled() {
  return IsExperimentEnabled<kExperimentIdV2NonOwningWakerImplementation>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_VERBOSE_CHANNELZ_CONNECTION_LOGGING
inline bool IsVerboseChannelzConnectionLoggingEnabled() {
  return IsExperimentEnabled<kExperimentIdVerboseChannelzConnectionLogging>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_WILDCARD_IP_EXPANSION_RESTRICTION
inline bool IsWildcardIpExpansionRestrictionEnabled() {
  return IsExperimentEnabled<kExperimentIdWildcardIpExpansionRestriction>();
}
#define GRPC_EXPERIMENT_IS_INCLUDED_XDS_SERVER_FILTER_CHAIN_PER_ROUTE
inline bool IsXdsServerFilterChainPerRouteEnabled() {
  return IsExperimentEnabled<kExperimentIdXdsServerFilterChainPerRoute>();
}

extern const ExperimentMetadata g_experiment_metadata[kNumExperiments];

#endif
}  // namespace grpc_core

#endif  // GRPC_SRC_CORE_LIB_EXPERIMENTS_EXPERIMENTS_H
