/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/route/v3/route_components.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef ENVOY_CONFIG_ROUTE_V3_ROUTE_COMPONENTS_PROTO_UPB_H__UPB_MINITABLE_H_
#define ENVOY_CONFIG_ROUTE_V3_ROUTE_COMPONENTS_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__config__route__v3__VirtualHost_msg_init;
extern const upb_MiniTable envoy__config__route__v3__VirtualHost__TypedPerFilterConfigEntry_msg_init;
extern const upb_MiniTable envoy__config__route__v3__FilterAction_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteList_msg_init;
extern const upb_MiniTable envoy__config__route__v3__Route_msg_init;
extern const upb_MiniTable envoy__config__route__v3__Route__TypedPerFilterConfigEntry_msg_init;
extern const upb_MiniTable envoy__config__route__v3__WeightedCluster_msg_init;
extern const upb_MiniTable envoy__config__route__v3__WeightedCluster__ClusterWeight_msg_init;
extern const upb_MiniTable envoy__config__route__v3__WeightedCluster__ClusterWeight__TypedPerFilterConfigEntry_msg_init;
extern const upb_MiniTable envoy__config__route__v3__ClusterSpecifierPlugin_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteMatch_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteMatch__GrpcRouteMatchOptions_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteMatch__TlsContextMatchOptions_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteMatch__ConnectMatcher_msg_init;
extern const upb_MiniTable envoy__config__route__v3__CorsPolicy_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction__RequestMirrorPolicy_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction__HashPolicy_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction__HashPolicy__Header_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction__HashPolicy__CookieAttribute_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction__HashPolicy__Cookie_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction__HashPolicy__ConnectionProperties_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction__HashPolicy__QueryParameter_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction__HashPolicy__FilterState_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction__UpgradeConfig_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction__UpgradeConfig__ConnectConfig_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RouteAction__MaxStreamDuration_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RetryPolicy_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RetryPolicy__RetryPriority_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RetryPolicy__RetryHostPredicate_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RetryPolicy__RetryBackOff_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RetryPolicy__ResetHeader_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RetryPolicy__RateLimitedRetryBackOff_msg_init;
extern const upb_MiniTable envoy__config__route__v3__HedgePolicy_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RedirectAction_msg_init;
extern const upb_MiniTable envoy__config__route__v3__DirectResponseAction_msg_init;
extern const upb_MiniTable envoy__config__route__v3__NonForwardingAction_msg_init;
extern const upb_MiniTable envoy__config__route__v3__Decorator_msg_init;
extern const upb_MiniTable envoy__config__route__v3__Tracing_msg_init;
extern const upb_MiniTable envoy__config__route__v3__VirtualCluster_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action__SourceCluster_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action__DestinationCluster_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action__RequestHeaders_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action__QueryParameters_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action__RemoteAddress_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action__MaskedRemoteAddress_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action__GenericKey_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action__HeaderValueMatch_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action__DynamicMetaData_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action__MetaData_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Action__QueryParameterValueMatch_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Override_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__Override__DynamicMetadata_msg_init;
extern const upb_MiniTable envoy__config__route__v3__RateLimit__HitsAddend_msg_init;
extern const upb_MiniTable envoy__config__route__v3__HeaderMatcher_msg_init;
extern const upb_MiniTable envoy__config__route__v3__QueryParameterMatcher_msg_init;
extern const upb_MiniTable envoy__config__route__v3__InternalRedirectPolicy_msg_init;
extern const upb_MiniTable envoy__config__route__v3__FilterConfig_msg_init;

extern const upb_MiniTableFile envoy_config_route_v3_route_components_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_CONFIG_ROUTE_V3_ROUTE_COMPONENTS_PROTO_UPB_H__UPB_MINITABLE_H_ */
