# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/api/v2/route.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/api/v2/route.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.api.v2.core import base_pb2 as envoy_dot_api_dot_v2_dot_core_dot_base__pb2
try:
  envoy_dot_api_dot_v2_dot_core_dot_socket__option__pb2 = envoy_dot_api_dot_v2_dot_core_dot_base__pb2.envoy_dot_api_dot_v2_dot_core_dot_socket__option__pb2
except AttributeError:
  envoy_dot_api_dot_v2_dot_core_dot_socket__option__pb2 = envoy_dot_api_dot_v2_dot_core_dot_base__pb2.envoy.api.v2.core.socket_option_pb2
from envoy.api.v2.core import config_source_pb2 as envoy_dot_api_dot_v2_dot_core_dot_config__source__pb2
from envoy.api.v2.route import route_components_pb2 as envoy_dot_api_dot_v2_dot_route_dot_route__components__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18\x65nvoy/api/v2/route.proto\x12\x0c\x65nvoy.api.v2\x1a\x1c\x65nvoy/api/v2/core/base.proto\x1a%envoy/api/v2/core/config_source.proto\x1a)envoy/api/v2/route/route_components.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x9f\x04\n\x12RouteConfiguration\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x36\n\rvirtual_hosts\x18\x02 \x03(\x0b\x32\x1f.envoy.api.v2.route.VirtualHost\x12 \n\x04vhds\x18\t \x01(\x0b\x32\x12.envoy.api.v2.Vhds\x12/\n\x15internal_only_headers\x18\x03 \x03(\tB\x10\xfa\x42\r\x92\x01\n\"\x08r\x06\xc0\x01\x01\xc8\x01\x00\x12P\n\x17response_headers_to_add\x18\x04 \x03(\x0b\x32$.envoy.api.v2.core.HeaderValueOptionB\t\xfa\x42\x06\x92\x01\x03\x10\xe8\x07\x12\x34\n\x1aresponse_headers_to_remove\x18\x05 \x03(\tB\x10\xfa\x42\r\x92\x01\n\"\x08r\x06\xc0\x01\x01\xc8\x01\x00\x12O\n\x16request_headers_to_add\x18\x06 \x03(\x0b\x32$.envoy.api.v2.core.HeaderValueOptionB\t\xfa\x42\x06\x92\x01\x03\x10\xe8\x07\x12\x33\n\x19request_headers_to_remove\x18\x08 \x03(\tB\x10\xfa\x42\r\x92\x01\n\"\x08r\x06\xc0\x01\x01\xc8\x01\x00\x12+\n#most_specific_header_mutations_wins\x18\n \x01(\x08\x12\x35\n\x11validate_clusters\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\"H\n\x04Vhds\x12@\n\rconfig_source\x18\x01 \x01(\x0b\x32\x1f.envoy.api.v2.core.ConfigSourceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x42\x8a\x01\n\x1aio.envoyproxy.envoy.api.v2B\nRouteProtoP\x01Z9github.com/envoyproxy/go-control-plane/envoy/api/v2;apiv2\xf2\x98\xfe\x8f\x05\x17\x12\x15\x65nvoy.config.route.v3\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.api.v2.route_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\032io.envoyproxy.envoy.api.v2B\nRouteProtoP\001Z9github.com/envoyproxy/go-control-plane/envoy/api/v2;apiv2\362\230\376\217\005\027\022\025envoy.config.route.v3\272\200\310\321\006\002\020\001'
  _globals['_ROUTECONFIGURATION'].fields_by_name['internal_only_headers']._loaded_options = None
  _globals['_ROUTECONFIGURATION'].fields_by_name['internal_only_headers']._serialized_options = b'\372B\r\222\001\n\"\010r\006\300\001\001\310\001\000'
  _globals['_ROUTECONFIGURATION'].fields_by_name['response_headers_to_add']._loaded_options = None
  _globals['_ROUTECONFIGURATION'].fields_by_name['response_headers_to_add']._serialized_options = b'\372B\006\222\001\003\020\350\007'
  _globals['_ROUTECONFIGURATION'].fields_by_name['response_headers_to_remove']._loaded_options = None
  _globals['_ROUTECONFIGURATION'].fields_by_name['response_headers_to_remove']._serialized_options = b'\372B\r\222\001\n\"\010r\006\300\001\001\310\001\000'
  _globals['_ROUTECONFIGURATION'].fields_by_name['request_headers_to_add']._loaded_options = None
  _globals['_ROUTECONFIGURATION'].fields_by_name['request_headers_to_add']._serialized_options = b'\372B\006\222\001\003\020\350\007'
  _globals['_ROUTECONFIGURATION'].fields_by_name['request_headers_to_remove']._loaded_options = None
  _globals['_ROUTECONFIGURATION'].fields_by_name['request_headers_to_remove']._serialized_options = b'\372B\r\222\001\n\"\010r\006\300\001\001\310\001\000'
  _globals['_VHDS'].fields_by_name['config_source']._loaded_options = None
  _globals['_VHDS'].fields_by_name['config_source']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_ROUTECONFIGURATION']._serialized_start=275
  _globals['_ROUTECONFIGURATION']._serialized_end=818
  _globals['_VHDS']._serialized_start=820
  _globals['_VHDS']._serialized_end=892
# @@protoc_insertion_point(module_scope)
