/* This file was generated by upb_generator from the input file:
 *
 *     xds/type/matcher/v3/matcher.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "xds/type/matcher/v3/matcher.upb_minitable.h"
#include "xds/core/v3/extension.upb_minitable.h"
#include "xds/type/matcher/v3/string.upb_minitable.h"
#include "validate/validate.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const UPB_PRIVATE(upb_GeneratedExtensionListEntry)* UPB_PRIVATE(upb_generated_extension_list);
typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[3];
} xds__type__matcher__v3__Matcher_msg_init_Fields;

static const xds__type__matcher__v3__Matcher_msg_init_Fields xds_type_matcher_v3_Matcher__fields = {{
  {1, UPB_SIZE(20, 24), UPB_SIZE(-17, -13), UPB_SIZE(9, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(20, 24), UPB_SIZE(-17, -13), UPB_SIZE(7, 9), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 16), 64, UPB_SIZE(5, 8), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherList_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherTree_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__OnMatch_msg_init},
}};

const upb_MiniTable xds__type__matcher__v3__Matcher_msg_init = {
  &xds_type_matcher_v3_Matcher__fields.fields[0],
  UPB_SIZE(24, 32), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Matcher",
#endif
};

typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[2];
} xds__type__matcher__v3__Matcher__OnMatch_msg_init_Fields;

static const xds__type__matcher__v3__Matcher__OnMatch_msg_init_Fields xds_type_matcher_v3_Matcher_OnMatch__fields = {{
  {1, 16, -13, UPB_SIZE(9, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, 16, -13, UPB_SIZE(7, 9), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, 8, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__core__v3__TypedExtensionConfig_msg_init},
}};

const upb_MiniTable xds__type__matcher__v3__Matcher__OnMatch_msg_init = {
  &xds_type_matcher_v3_Matcher_OnMatch__fields.fields[0],
  24, 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Matcher.OnMatch",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000018, &upb_DecodeFast_Bool_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[1];
  upb_MiniTableSubInternal subs[1];
} xds__type__matcher__v3__Matcher__MatcherList_msg_init_Fields;

static const xds__type__matcher__v3__Matcher__MatcherList_msg_init_Fields xds_type_matcher_v3_Matcher_MatcherList__fields = {{
  {1, 8, 0, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherList__FieldMatcher_msg_init},
}};

const upb_MiniTable xds__type__matcher__v3__Matcher__MatcherList_msg_init = {
  &xds_type_matcher_v3_Matcher_MatcherList__fields.fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Matcher.MatcherList",
#endif
};

typedef struct {
  upb_MiniTableField fields[4];
  upb_MiniTableSubInternal subs[4];
} xds__type__matcher__v3__Matcher__MatcherList__Predicate_msg_init_Fields;

static const xds__type__matcher__v3__Matcher__MatcherList__Predicate_msg_init_Fields xds_type_matcher_v3_Matcher_MatcherList_Predicate__fields = {{
  {1, UPB_SIZE(12, 16), -9, 12, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), -9, UPB_SIZE(10, 11), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 16), -9, UPB_SIZE(8, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(12, 16), -9, UPB_SIZE(6, 9), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherList__Predicate__SinglePredicate_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherList__Predicate__PredicateList_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherList__Predicate__PredicateList_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherList__Predicate_msg_init},
}};

const upb_MiniTable xds__type__matcher__v3__Matcher__MatcherList__Predicate_msg_init = {
  &xds_type_matcher_v3_Matcher_MatcherList_Predicate__fields.fields[0],
  UPB_SIZE(16, 24), 4, kUpb_ExtMode_NonExtendable, 4, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Matcher.MatcherList.Predicate",
#endif
};

typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[3];
} xds__type__matcher__v3__Matcher__MatcherList__Predicate__SinglePredicate_msg_init_Fields;

static const xds__type__matcher__v3__Matcher__MatcherList__Predicate__SinglePredicate_msg_init_Fields xds_type_matcher_v3_Matcher_MatcherList_Predicate_SinglePredicate__fields = {{
  {1, UPB_SIZE(12, 16), 64, UPB_SIZE(9, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(20, 24), UPB_SIZE(-17, -13), UPB_SIZE(7, 9), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 24), UPB_SIZE(-17, -13), UPB_SIZE(5, 8), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &xds__core__v3__TypedExtensionConfig_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__StringMatcher_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__core__v3__TypedExtensionConfig_msg_init},
}};

const upb_MiniTable xds__type__matcher__v3__Matcher__MatcherList__Predicate__SinglePredicate_msg_init = {
  &xds_type_matcher_v3_Matcher_MatcherList_Predicate_SinglePredicate__fields.fields[0],
  UPB_SIZE(24, 32), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Matcher.MatcherList.Predicate.SinglePredicate",
#endif
};

typedef struct {
  upb_MiniTableField fields[1];
  upb_MiniTableSubInternal subs[1];
} xds__type__matcher__v3__Matcher__MatcherList__Predicate__PredicateList_msg_init_Fields;

static const xds__type__matcher__v3__Matcher__MatcherList__Predicate__PredicateList_msg_init_Fields xds_type_matcher_v3_Matcher_MatcherList_Predicate_PredicateList__fields = {{
  {1, 8, 0, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherList__Predicate_msg_init},
}};

const upb_MiniTable xds__type__matcher__v3__Matcher__MatcherList__Predicate__PredicateList_msg_init = {
  &xds_type_matcher_v3_Matcher_MatcherList_Predicate_PredicateList__fields.fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Matcher.MatcherList.Predicate.PredicateList",
#endif
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[2];
} xds__type__matcher__v3__Matcher__MatcherList__FieldMatcher_msg_init_Fields;

static const xds__type__matcher__v3__Matcher__MatcherList__FieldMatcher_msg_init_Fields xds_type_matcher_v3_Matcher_MatcherList_FieldMatcher__fields = {{
  {1, UPB_SIZE(12, 16), 64, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 65, UPB_SIZE(4, 5), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherList__Predicate_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__OnMatch_msg_init},
}};

const upb_MiniTable xds__type__matcher__v3__Matcher__MatcherList__FieldMatcher_msg_init = {
  &xds_type_matcher_v3_Matcher_MatcherList_FieldMatcher__fields.fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Matcher.MatcherList.FieldMatcher",
#endif
};

typedef struct {
  upb_MiniTableField fields[4];
  upb_MiniTableSubInternal subs[4];
} xds__type__matcher__v3__Matcher__MatcherTree_msg_init_Fields;

static const xds__type__matcher__v3__Matcher__MatcherTree_msg_init_Fields xds_type_matcher_v3_Matcher_MatcherTree__fields = {{
  {1, UPB_SIZE(12, 16), 64, 12, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(20, 24), UPB_SIZE(-17, -13), UPB_SIZE(10, 11), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 24), UPB_SIZE(-17, -13), UPB_SIZE(8, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(20, 24), UPB_SIZE(-17, -13), UPB_SIZE(6, 9), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &xds__core__v3__TypedExtensionConfig_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherTree__MatchMap_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherTree__MatchMap_msg_init},
  {.UPB_PRIVATE(submsg) = &xds__core__v3__TypedExtensionConfig_msg_init},
}};

const upb_MiniTable xds__type__matcher__v3__Matcher__MatcherTree_msg_init = {
  &xds_type_matcher_v3_Matcher_MatcherTree__fields.fields[0],
  UPB_SIZE(24, 32), 4, kUpb_ExtMode_NonExtendable, 4, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Matcher.MatcherTree",
#endif
};

typedef struct {
  upb_MiniTableField fields[1];
  upb_MiniTableSubInternal subs[1];
} xds__type__matcher__v3__Matcher__MatcherTree__MatchMap_msg_init_Fields;

static const xds__type__matcher__v3__Matcher__MatcherTree__MatchMap_msg_init_Fields xds_type_matcher_v3_Matcher_MatcherTree_MatchMap__fields = {{
  {1, 8, 0, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Map | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__MatcherTree__MatchMap__MapEntry_msg_init},
}};

const upb_MiniTable xds__type__matcher__v3__Matcher__MatcherTree__MatchMap_msg_init = {
  &xds_type_matcher_v3_Matcher_MatcherTree_MatchMap__fields.fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Matcher.MatcherTree.MatchMap",
#endif
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} xds__type__matcher__v3__Matcher__MatcherTree__MatchMap__MapEntry_msg_init_Fields;

static const xds__type__matcher__v3__Matcher__MatcherTree__MatchMap__MapEntry_msg_init_Fields xds_type_matcher_v3_Matcher_MatcherTree_MatchMap_MapEntry__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, 32, 64, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &xds__type__matcher__v3__Matcher__OnMatch_msg_init},
}};

const upb_MiniTable xds__type__matcher__v3__Matcher__MatcherTree__MatchMap__MapEntry_msg_init = {
  &xds_type_matcher_v3_Matcher_MatcherTree_MatchMap_MapEntry__fields.fields[0],
  48, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "xds.type.matcher.v3.Matcher.MatcherTree.MatchMap.MapEntry",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

static const upb_MiniTable *messages_layout[10] = {
  &xds__type__matcher__v3__Matcher_msg_init,
  &xds__type__matcher__v3__Matcher__OnMatch_msg_init,
  &xds__type__matcher__v3__Matcher__MatcherList_msg_init,
  &xds__type__matcher__v3__Matcher__MatcherList__Predicate_msg_init,
  &xds__type__matcher__v3__Matcher__MatcherList__Predicate__SinglePredicate_msg_init,
  &xds__type__matcher__v3__Matcher__MatcherList__Predicate__PredicateList_msg_init,
  &xds__type__matcher__v3__Matcher__MatcherList__FieldMatcher_msg_init,
  &xds__type__matcher__v3__Matcher__MatcherTree_msg_init,
  &xds__type__matcher__v3__Matcher__MatcherTree__MatchMap_msg_init,
  &xds__type__matcher__v3__Matcher__MatcherTree__MatchMap__MapEntry_msg_init,
};

const upb_MiniTableFile xds_type_matcher_v3_matcher_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  10,
  0,
  0,
};

#include "upb/port/undef.inc"

