/* This file was generated by upb_generator from the input file:
 *
 *     src/proto/grpc/channelz/channelz.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */



#ifndef SRC_PROTO_GRPC_CHANNELZ_CHANNELZ_PROTO_UPB_H__UPBDEFS_H_
#define SRC_PROTO_GRPC_CHANNELZ_CHANNELZ_PROTO_UPB_H__UPBDEFS_H_

#include "upb/reflection/def.h"
#include "upb/reflection/internal/def_pool.h"

#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern _upb_DefPool_Init src_proto_grpc_channelz_channelz_proto_upbdefinit;

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_Channel_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.Channel");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_Subchannel_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.Subchannel");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_ChannelConnectivityState_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.ChannelConnectivityState");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_ChannelData_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.ChannelData");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_ChannelTraceEvent_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.ChannelTraceEvent");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_ChannelTrace_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.ChannelTrace");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_ChannelRef_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.ChannelRef");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_SubchannelRef_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.SubchannelRef");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_SocketRef_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.SocketRef");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_ServerRef_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.ServerRef");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_Server_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.Server");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_ServerData_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.ServerData");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_Socket_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.Socket");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_SocketData_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.SocketData");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_Address_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.Address");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_Address_TcpIpAddress_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.Address.TcpIpAddress");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_Address_UdsAddress_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.Address.UdsAddress");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_Address_OtherAddress_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.Address.OtherAddress");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_Security_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.Security");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_Security_Tls_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.Security.Tls");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_Security_OtherSecurity_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.Security.OtherSecurity");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_SocketOption_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.SocketOption");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_SocketOptionTimeout_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.SocketOptionTimeout");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_SocketOptionLinger_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.SocketOptionLinger");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_SocketOptionTcpInfo_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.SocketOptionTcpInfo");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetTopChannelsRequest_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetTopChannelsRequest");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetTopChannelsResponse_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetTopChannelsResponse");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetServersRequest_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetServersRequest");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetServersResponse_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetServersResponse");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetServerRequest_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetServerRequest");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetServerResponse_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetServerResponse");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetServerSocketsRequest_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetServerSocketsRequest");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetServerSocketsResponse_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetServerSocketsResponse");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetChannelRequest_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetChannelRequest");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetChannelResponse_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetChannelResponse");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetSubchannelRequest_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetSubchannelRequest");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetSubchannelResponse_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetSubchannelResponse");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetSocketRequest_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetSocketRequest");
}

UPB_INLINE const upb_MessageDef *grpc_channelz_v1_GetSocketResponse_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &src_proto_grpc_channelz_channelz_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "grpc.channelz.v1.GetSocketResponse");
}


#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* SRC_PROTO_GRPC_CHANNELZ_CHANNELZ_PROTO_UPB_H__UPBDEFS_H_ */
