// Copyright 2026 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

edition = "2023";

package grpc.testing;

import "google/protobuf/cpp_features.proto";
import "google/protobuf/go_features.proto";
import "google/protobuf/java_features.proto";

option go_package = "foo.com/demo";
option java_package = "com.foo.demo";

option features.(pb.go).api_level = API_OPAQUE;
option features.(pb.cpp).string_type = VIEW;
option features.(pb.java).utf8_validation = VERIFY;

message Demo {
  string name = 1;
}
