# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/router/cluster_specifiers/matcher/v3/matcher.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/router/cluster_specifiers/matcher/v3/matcher.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.type.matcher.v3 import matcher_pb2 as xds_dot_type_dot_matcher_dot_v3_dot_matcher__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nCenvoy/extensions/router/cluster_specifiers/matcher/v3/matcher.proto\x12\x35\x65nvoy.extensions.router.cluster_specifiers.matcher.v3\x1a!xds/type/matcher/v3/matcher.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\")\n\rClusterAction\x12\x18\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\"Z\n\x17MatcherClusterSpecifier\x12?\n\x0f\x63luster_matcher\x18\x01 \x01(\x0b\x32\x1c.xds.type.matcher.v3.MatcherB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x42\xc5\x01\nCio.envoyproxy.envoy.extensions.router.cluster_specifiers.matcher.v3B\x0cMatcherProtoP\x01Zfgithub.com/envoyproxy/go-control-plane/envoy/extensions/router/cluster_specifiers/matcher/v3;matcherv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.router.cluster_specifiers.matcher.v3.matcher_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nCio.envoyproxy.envoy.extensions.router.cluster_specifiers.matcher.v3B\014MatcherProtoP\001Zfgithub.com/envoyproxy/go-control-plane/envoy/extensions/router/cluster_specifiers/matcher/v3;matcherv3\272\200\310\321\006\002\020\002'
  _globals['_CLUSTERACTION'].fields_by_name['cluster']._loaded_options = None
  _globals['_CLUSTERACTION'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_MATCHERCLUSTERSPECIFIER'].fields_by_name['cluster_matcher']._loaded_options = None
  _globals['_MATCHERCLUSTERSPECIFIER'].fields_by_name['cluster_matcher']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_CLUSTERACTION']._serialized_start=217
  _globals['_CLUSTERACTION']._serialized_end=258
  _globals['_MATCHERCLUSTERSPECIFIER']._serialized_start=260
  _globals['_MATCHERCLUSTERSPECIFIER']._serialized_end=350
# @@protoc_insertion_point(module_scope)
