/* This file was generated by upb_generator from the input file:
 *
 *     envoy/extensions/clusters/aggregate/v3/cluster.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */



#ifndef ENVOY_EXTENSIONS_CLUSTERS_AGGREGATE_V3_CLUSTER_PROTO_UPB_H__UPBDEFS_H_
#define ENVOY_EXTENSIONS_CLUSTERS_AGGREGATE_V3_CLUSTER_PROTO_UPB_H__UPBDEFS_H_

#include "upb/reflection/def.h"
#include "upb/reflection/internal/def_pool.h"

#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern _upb_DefPool_Init envoy_extensions_clusters_aggregate_v3_cluster_proto_upbdefinit;

UPB_INLINE const upb_MessageDef *envoy_extensions_clusters_aggregate_v3_ClusterConfig_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &envoy_extensions_clusters_aggregate_v3_cluster_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "envoy.extensions.clusters.aggregate.v3.ClusterConfig");
}

UPB_INLINE const upb_MessageDef *envoy_extensions_clusters_aggregate_v3_AggregateClusterResource_getmsgdef(upb_DefPool *s) {
  _upb_DefPool_LoadDefInit(s, &envoy_extensions_clusters_aggregate_v3_cluster_proto_upbdefinit);
  return upb_DefPool_FindMessageByName(s, "envoy.extensions.clusters.aggregate.v3.AggregateClusterResource");
}


#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_EXTENSIONS_CLUSTERS_AGGREGATE_V3_CLUSTER_PROTO_UPB_H__UPBDEFS_H_ */
