// Copyright 2021 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package avl_fuzzer;

message Empty{}

message Action {
    int32 key = 1;
    oneof action {
        int32 set = 2;
        Empty get = 3;
        Empty del = 4;
    }
}

message KeyValue {
    int32 key = 1;
    int32 value = 2;
}

message Compares {
    repeated KeyValue left = 1;
    repeated KeyValue right = 2;
}

message Msg {
    repeated Action actions = 2;
    repeated Compares compares = 3;
}
