# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/transform/v3/transform.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/transform/v3/transform.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.common.mutation_rules.v3 import mutation_rules_pb2 as envoy_dot_config_dot_common_dot_mutation__rules_dot_v3_dot_mutation__rules__pb2
from envoy.config.core.v3 import substitution_format_string_pb2 as envoy_dot_config_dot_core_dot_v3_dot_substitution__format__string__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n:envoy/extensions/filters/http/transform/v3/transform.proto\x12*envoy.extensions.filters.http.transform.v3\x1a:envoy/config/common/mutation_rules/v3/mutation_rules.proto\x1a\x35\x65nvoy/config/core/v3/substitution_format_string.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x82\x02\n\x0fTransformConfig\x12Z\n\x16request_transformation\x18\x01 \x01(\x0b\x32:.envoy.extensions.filters.http.transform.v3.Transformation\x12[\n\x17response_transformation\x18\x02 \x01(\x0b\x32:.envoy.extensions.filters.http.transform.v3.Transformation\x12\x1b\n\x13\x63lear_cluster_cache\x18\x03 \x01(\x08\x12\x19\n\x11\x63lear_route_cache\x18\x04 \x01(\x08\"\xbf\x01\n\x0eTransformation\x12P\n\x11headers_mutations\x18\x01 \x03(\x0b\x32\x35.envoy.config.common.mutation_rules.v3.HeaderMutation\x12[\n\x13\x62ody_transformation\x18\x02 \x01(\x0b\x32>.envoy.extensions.filters.http.transform.v3.BodyTransformation\"\xee\x01\n\x12\x42odyTransformation\x12M\n\x0b\x62ody_format\x18\x01 \x01(\x0b\x32..envoy.config.core.v3.SubstitutionFormatStringB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12^\n\x06\x61\x63tion\x18\x02 \x01(\x0e\x32N.envoy.extensions.filters.http.transform.v3.BodyTransformation.TransformAction\")\n\x0fTransformAction\x12\t\n\x05MERGE\x10\x00\x12\x0b\n\x07REPLACE\x10\x01\x42\xb3\x01\n8io.envoyproxy.envoy.extensions.filters.http.transform.v3B\x0eTransformProtoP\x01Z]github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/transform/v3;transformv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.transform.v3.transform_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n8io.envoyproxy.envoy.extensions.filters.http.transform.v3B\016TransformProtoP\001Z]github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/transform/v3;transformv3\272\200\310\321\006\002\020\002'
  _globals['_BODYTRANSFORMATION'].fields_by_name['body_format']._loaded_options = None
  _globals['_BODYTRANSFORMATION'].fields_by_name['body_format']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_TRANSFORMCONFIG']._serialized_start=278
  _globals['_TRANSFORMCONFIG']._serialized_end=536
  _globals['_TRANSFORMATION']._serialized_start=539
  _globals['_TRANSFORMATION']._serialized_end=730
  _globals['_BODYTRANSFORMATION']._serialized_start=733
  _globals['_BODYTRANSFORMATION']._serialized_end=971
  _globals['_BODYTRANSFORMATION_TRANSFORMACTION']._serialized_start=930
  _globals['_BODYTRANSFORMATION_TRANSFORMACTION']._serialized_end=971
# @@protoc_insertion_point(module_scope)
