<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: math.proto

namespace Math;

use Google\Protobuf\Internal\GPBType;
use Google\Protobuf\Internal\GPBUtil;
use Google\Protobuf\RepeatedField;

/**
 * Generated from protobuf message <code>math.DivArgs</code>
 */
class DivArgs extends \Google\Protobuf\Internal\Message
{
    /**
     * Generated from protobuf field <code>int64 dividend = 1;</code>
     */
    protected $dividend = 0;
    /**
     * Generated from protobuf field <code>int64 divisor = 2;</code>
     */
    protected $divisor = 0;

    /**
     * Constructor.
     *
     * @param array $data {
     *     Optional. Data for populating the Message object.
     *
     *     @type int|string $dividend
     *     @type int|string $divisor
     * }
     */
    public function __construct($data = NULL) {
        \GPBMetadata\Math::initOnce();
        parent::__construct($data);
    }

    /**
     * Generated from protobuf field <code>int64 dividend = 1;</code>
     * @return int|string
     */
    public function getDividend()
    {
        return $this->dividend;
    }

    /**
     * Generated from protobuf field <code>int64 dividend = 1;</code>
     * @param int|string $var
     * @return $this
     */
    public function setDividend(int|string $var)
    {
        GPBUtil::checkInt64($var);
        $this->dividend = $var;

        return $this;
    }

    /**
     * Generated from protobuf field <code>int64 divisor = 2;</code>
     * @return int|string
     */
    public function getDivisor()
    {
        return $this->divisor;
    }

    /**
     * Generated from protobuf field <code>int64 divisor = 2;</code>
     * @param int|string $var
     * @return $this
     */
    public function setDivisor(int|string $var)
    {
        GPBUtil::checkInt64($var);
        $this->divisor = $var;

        return $this;
    }

}

