// Copyright 2018 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package keyvaluestore;

// A simple key-value storage service
service KeyValueStore {
  // Provides a value for each key request
  rpc GetValues (stream Request) returns (stream Response) {}
}

// The request message containing the key
message Request {
  string key = 1;
}

// The response message containing the value associated with the key
message Response {
  string value = 1;
}
