// Copyright 2022 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
syntax = "proto3";

package work_queue_fuzzer;

enum CallbackType {
    CALLBACK_TYPE_CLOSURE = 0;
    CALLBACK_TYPE_ANY_INVOCABLE = 1;
}

message Add {
    CallbackType type = 1;
    int32 key = 2;
}

message PopMostRecent {
}

message PopOldest {
}

message Empty {
}

message Action {
    oneof action_type {
        Add add = 1;
        PopMostRecent pop_most_recent = 2;
        PopOldest pop_oldest = 3;
        Empty empty = 4;
    }
}

message Msg {
    repeated Action actions = 1;
}
