# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/proto_api_scrubber/v3/config.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/proto_api_scrubber/v3/config.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from xds.type.matcher.v3 import matcher_pb2 as xds_dot_type_dot_matcher_dot_v3_dot_matcher__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n@envoy/extensions/filters/http/proto_api_scrubber/v3/config.proto\x12\x33\x65nvoy.extensions.filters.http.proto_api_scrubber.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\x1fxds/annotations/v3/status.proto\x1a!xds/type/matcher/v3/matcher.proto\x1a\x1dudpa/annotations/status.proto\"\xdf\x02\n\x16ProtoApiScrubberConfig\x12Z\n\x0e\x64\x65scriptor_set\x18\x01 \x01(\x0b\x32\x42.envoy.extensions.filters.http.proto_api_scrubber.v3.DescriptorSet\x12W\n\x0crestrictions\x18\x02 \x01(\x0b\x32\x41.envoy.extensions.filters.http.proto_api_scrubber.v3.Restrictions\x12q\n\x0e\x66iltering_mode\x18\x03 \x01(\x0e\x32Y.envoy.extensions.filters.http.proto_api_scrubber.v3.ProtoApiScrubberConfig.FilteringMode\"\x1d\n\rFilteringMode\x12\x0c\n\x08OVERRIDE\x10\x00\"F\n\rDescriptorSet\x12\x35\n\x0b\x64\x61ta_source\x18\x01 \x01(\x0b\x32 .envoy.config.core.v3.DataSource\"\x8c\x04\n\x0cRestrictions\x12v\n\x13method_restrictions\x18\x01 \x03(\x0b\x32Y.envoy.extensions.filters.http.proto_api_scrubber.v3.Restrictions.MethodRestrictionsEntry\x12x\n\x14message_restrictions\x18\x02 \x03(\x0b\x32Z.envoy.extensions.filters.http.proto_api_scrubber.v3.Restrictions.MessageRestrictionsEntry\x1a\x82\x01\n\x17MethodRestrictionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12V\n\x05value\x18\x02 \x01(\x0b\x32G.envoy.extensions.filters.http.proto_api_scrubber.v3.MethodRestrictions:\x02\x38\x01\x1a\x84\x01\n\x18MessageRestrictionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12W\n\x05value\x18\x02 \x01(\x0b\x32H.envoy.extensions.filters.http.proto_api_scrubber.v3.MessageRestrictions:\x02\x38\x01\"\xa7\x05\n\x12MethodRestrictions\x12\x89\x01\n\x1arequest_field_restrictions\x18\x01 \x03(\x0b\x32\x65.envoy.extensions.filters.http.proto_api_scrubber.v3.MethodRestrictions.RequestFieldRestrictionsEntry\x12\x8b\x01\n\x1bresponse_field_restrictions\x18\x02 \x03(\x0b\x32\x66.envoy.extensions.filters.http.proto_api_scrubber.v3.MethodRestrictions.ResponseFieldRestrictionsEntry\x12\x62\n\x12method_restriction\x18\x03 \x01(\x0b\x32\x46.envoy.extensions.filters.http.proto_api_scrubber.v3.RestrictionConfig\x1a\x87\x01\n\x1dRequestFieldRestrictionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12U\n\x05value\x18\x02 \x01(\x0b\x32\x46.envoy.extensions.filters.http.proto_api_scrubber.v3.RestrictionConfig:\x02\x38\x01\x1a\x88\x01\n\x1eResponseFieldRestrictionsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12U\n\x05value\x18\x02 \x01(\x0b\x32\x46.envoy.extensions.filters.http.proto_api_scrubber.v3.RestrictionConfig:\x02\x38\x01\"m\n\x13MessageRestrictions\x12V\n\x06\x63onfig\x18\x01 \x01(\x0b\x32\x46.envoy.extensions.filters.http.proto_api_scrubber.v3.RestrictionConfig\"B\n\x11RestrictionConfig\x12-\n\x07matcher\x18\x01 \x01(\x0b\x32\x1c.xds.type.matcher.v3.MatcherB\xd3\x01\nAio.envoyproxy.envoy.extensions.filters.http.proto_api_scrubber.v3B\x0b\x43onfigProtoP\x01Zogithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/proto_api_scrubber/v3;proto_api_scrubberv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.proto_api_scrubber.v3.config_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nAio.envoyproxy.envoy.extensions.filters.http.proto_api_scrubber.v3B\013ConfigProtoP\001Zogithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/proto_api_scrubber/v3;proto_api_scrubberv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_RESTRICTIONS_METHODRESTRICTIONSENTRY']._loaded_options = None
  _globals['_RESTRICTIONS_METHODRESTRICTIONSENTRY']._serialized_options = b'8\001'
  _globals['_RESTRICTIONS_MESSAGERESTRICTIONSENTRY']._loaded_options = None
  _globals['_RESTRICTIONS_MESSAGERESTRICTIONSENTRY']._serialized_options = b'8\001'
  _globals['_METHODRESTRICTIONS_REQUESTFIELDRESTRICTIONSENTRY']._loaded_options = None
  _globals['_METHODRESTRICTIONS_REQUESTFIELDRESTRICTIONSENTRY']._serialized_options = b'8\001'
  _globals['_METHODRESTRICTIONS_RESPONSEFIELDRESTRICTIONSENTRY']._loaded_options = None
  _globals['_METHODRESTRICTIONS_RESPONSEFIELDRESTRICTIONSENTRY']._serialized_options = b'8\001'
  _globals['_PROTOAPISCRUBBERCONFIG']._serialized_start=254
  _globals['_PROTOAPISCRUBBERCONFIG']._serialized_end=605
  _globals['_PROTOAPISCRUBBERCONFIG_FILTERINGMODE']._serialized_start=576
  _globals['_PROTOAPISCRUBBERCONFIG_FILTERINGMODE']._serialized_end=605
  _globals['_DESCRIPTORSET']._serialized_start=607
  _globals['_DESCRIPTORSET']._serialized_end=677
  _globals['_RESTRICTIONS']._serialized_start=680
  _globals['_RESTRICTIONS']._serialized_end=1204
  _globals['_RESTRICTIONS_METHODRESTRICTIONSENTRY']._serialized_start=939
  _globals['_RESTRICTIONS_METHODRESTRICTIONSENTRY']._serialized_end=1069
  _globals['_RESTRICTIONS_MESSAGERESTRICTIONSENTRY']._serialized_start=1072
  _globals['_RESTRICTIONS_MESSAGERESTRICTIONSENTRY']._serialized_end=1204
  _globals['_METHODRESTRICTIONS']._serialized_start=1207
  _globals['_METHODRESTRICTIONS']._serialized_end=1886
  _globals['_METHODRESTRICTIONS_REQUESTFIELDRESTRICTIONSENTRY']._serialized_start=1612
  _globals['_METHODRESTRICTIONS_REQUESTFIELDRESTRICTIONSENTRY']._serialized_end=1747
  _globals['_METHODRESTRICTIONS_RESPONSEFIELDRESTRICTIONSENTRY']._serialized_start=1750
  _globals['_METHODRESTRICTIONS_RESPONSEFIELDRESTRICTIONSENTRY']._serialized_end=1886
  _globals['_MESSAGERESTRICTIONS']._serialized_start=1888
  _globals['_MESSAGERESTRICTIONS']._serialized_end=1997
  _globals['_RESTRICTIONCONFIG']._serialized_start=1999
  _globals['_RESTRICTIONCONFIG']._serialized_end=2065
# @@protoc_insertion_point(module_scope)
