/* This file was generated by upb_generator from the input file:
 *
 *     google/api/expr/v1alpha1/syntax.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef GOOGLE_API_EXPR_V1ALPHA1_SYNTAX_PROTO_UPB_H__UPB_MINITABLE_H_
#define GOOGLE_API_EXPR_V1ALPHA1_SYNTAX_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable google__api__expr__v1alpha1__ParsedExpr_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__Expr_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__Expr__Ident_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__Expr__Select_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__Expr__Call_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__Expr__CreateList_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__Expr__CreateStruct_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__Expr__CreateStruct__Entry_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__Expr__Comprehension_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__Constant_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__SourceInfo_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__SourceInfo__Extension_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__SourceInfo__Extension__Version_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__SourceInfo__PositionsEntry_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__SourceInfo__MacroCallsEntry_msg_init;
extern const upb_MiniTable google__api__expr__v1alpha1__SourcePosition_msg_init;

extern const upb_MiniTableFile google_api_expr_v1alpha1_syntax_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* GOOGLE_API_EXPR_V1ALPHA1_SYNTAX_PROTO_UPB_H__UPB_MINITABLE_H_ */
