ul {
  list-style-type: none;
}
a{
  text-decoration: none;
}
a:hover {
  text-decoration: underline;
}
ul li a {
  font-family: 'SF Mono', 'Menlo', 'Monaco', 'Consolas', 'Courier New', Courier, monospace
}
h1 {
  font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, Oxygen, Ubuntu, Cantarell, 'Open Sans', 'Helvetica Neue', sans-serif
}

