/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/cluster/v3/cluster.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef ENVOY_CONFIG_CLUSTER_V3_CLUSTER_PROTO_UPB_H__UPB_MINITABLE_H_
#define ENVOY_CONFIG_CLUSTER_V3_CLUSTER_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__config__cluster__v3__ClusterCollection_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__TransportSocketMatch_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__CustomClusterType_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__EdsClusterConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__LbSubsetConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__LbSubsetConfig__LbSubsetSelector_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__SlowStartConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__RoundRobinLbConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__LeastRequestLbConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__RingHashLbConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__MaglevLbConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__OriginalDstLbConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__CommonLbConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__CommonLbConfig__ZoneAwareLbConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__CommonLbConfig__LocalityWeightedLbConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__CommonLbConfig__ConsistentHashingLbConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__RefreshRate_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__PreconnectPolicy_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__Cluster__TypedExtensionProtocolOptionsEntry_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__LoadBalancingPolicy_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__LoadBalancingPolicy__Policy_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__UpstreamConnectionOptions_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__UpstreamConnectionOptions__HappyEyeballsConfig_msg_init;
extern const upb_MiniTable envoy__config__cluster__v3__TrackClusterStats_msg_init;

extern const upb_MiniTableFile envoy_config_cluster_v3_cluster_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_CONFIG_CLUSTER_V3_CLUSTER_PROTO_UPB_H__UPB_MINITABLE_H_ */
