/* This file was generated by upb_generator from the input file:
 *
 *     envoy/extensions/filters/common/fault/v3/fault.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "envoy/extensions/filters/common/fault/v3/fault.upb_minitable.h"
#include "envoy/type/v3/percent.upb_minitable.h"
#include "google/protobuf/duration.upb_minitable.h"
#include "udpa/annotations/status.upb_minitable.h"
#include "udpa/annotations/versioning.upb_minitable.h"
#include "validate/validate.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const UPB_PRIVATE(upb_GeneratedExtensionListEntry)* UPB_PRIVATE(upb_generated_extension_list);
typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[3];
} envoy__extensions__filters__common__fault__v3__FaultDelay_msg_init_Fields;

static const envoy__extensions__filters__common__fault__v3__FaultDelay_msg_init_Fields envoy_extensions_filters_common_fault_v3_FaultDelay__fields = {{
  {3, UPB_SIZE(20, 24), UPB_SIZE(-17, -13), UPB_SIZE(9, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(12, 16), 64, UPB_SIZE(7, 9), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(20, 24), UPB_SIZE(-17, -13), UPB_SIZE(5, 8), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Duration_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__type__v3__FractionalPercent_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__extensions__filters__common__fault__v3__FaultDelay__HeaderDelay_msg_init},
}};

const upb_MiniTable envoy__extensions__filters__common__fault__v3__FaultDelay_msg_init = {
  &envoy_extensions_filters_common_fault_v3_FaultDelay__fields.fields[0],
  UPB_SIZE(24, 32), 3, kUpb_ExtMode_NonExtendable, 0, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.extensions.filters.common.fault.v3.FaultDelay",
#endif
};

const upb_MiniTable envoy__extensions__filters__common__fault__v3__FaultDelay__HeaderDelay_msg_init = {
  NULL,
  8, 0, kUpb_ExtMode_NonExtendable, 0, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.extensions.filters.common.fault.v3.FaultDelay.HeaderDelay",
#endif
};

typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[3];
} envoy__extensions__filters__common__fault__v3__FaultRateLimit_msg_init_Fields;

static const envoy__extensions__filters__common__fault__v3__FaultRateLimit_msg_init_Fields envoy_extensions_filters_common_fault_v3_FaultRateLimit__fields = {{
  {1, UPB_SIZE(20, 24), UPB_SIZE(-17, -13), UPB_SIZE(9, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), 64, UPB_SIZE(7, 9), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 24), UPB_SIZE(-17, -13), UPB_SIZE(5, 8), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__extensions__filters__common__fault__v3__FaultRateLimit__FixedLimit_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__type__v3__FractionalPercent_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__extensions__filters__common__fault__v3__FaultRateLimit__HeaderLimit_msg_init},
}};

const upb_MiniTable envoy__extensions__filters__common__fault__v3__FaultRateLimit_msg_init = {
  &envoy_extensions_filters_common_fault_v3_FaultRateLimit__fields.fields[0],
  UPB_SIZE(24, 32), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.extensions.filters.common.fault.v3.FaultRateLimit",
#endif
};

typedef struct {
  upb_MiniTableField fields[1];
} envoy__extensions__filters__common__fault__v3__FaultRateLimit__FixedLimit_msg_init_Fields;

static const envoy__extensions__filters__common__fault__v3__FaultRateLimit__FixedLimit_msg_init_Fields envoy_extensions_filters_common_fault_v3_FaultRateLimit_FixedLimit__fields = {{
  {1, 8, 0, kUpb_NoSub, 4, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__extensions__filters__common__fault__v3__FaultRateLimit__FixedLimit_msg_init = {
  &envoy_extensions_filters_common_fault_v3_FaultRateLimit_FixedLimit__fields.fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.extensions.filters.common.fault.v3.FaultRateLimit.FixedLimit",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000008, &upb_DecodeFast_Varint64_Scalar_Tag1Byte},
  })
};

const upb_MiniTable envoy__extensions__filters__common__fault__v3__FaultRateLimit__HeaderLimit_msg_init = {
  NULL,
  8, 0, kUpb_ExtMode_NonExtendable, 0, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.extensions.filters.common.fault.v3.FaultRateLimit.HeaderLimit",
#endif
};

static const upb_MiniTable *messages_layout[5] = {
  &envoy__extensions__filters__common__fault__v3__FaultDelay_msg_init,
  &envoy__extensions__filters__common__fault__v3__FaultDelay__HeaderDelay_msg_init,
  &envoy__extensions__filters__common__fault__v3__FaultRateLimit_msg_init,
  &envoy__extensions__filters__common__fault__v3__FaultRateLimit__FixedLimit_msg_init,
  &envoy__extensions__filters__common__fault__v3__FaultRateLimit__HeaderLimit_msg_init,
};

const upb_MiniTableFile envoy_extensions_filters_common_fault_v3_fault_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  5,
  0,
  0,
};

#include "upb/port/undef.inc"

