//
//
// Copyright 2015 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
//

#ifndef GRPC_TEST_CORE_CREDENTIALS_CALL_OAUTH2_OAUTH2_UTILS_H
#define GRPC_TEST_CORE_CREDENTIALS_CALL_OAUTH2_OAUTH2_UTILS_H

#include "src/core/credentials/call/call_credentials.h"

// Fetch oauth2 access token with a credentials object. Does not take ownership.
// Returns NULL on a failure. The caller should call gpr_free on the token.
char* grpc_test_fetch_oauth2_token_with_credentials(
    grpc_call_credentials* creds);

#endif  // GRPC_TEST_CORE_CREDENTIALS_CALL_OAUTH2_OAUTH2_UTILS_H
