# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/network/sip_proxy/v3alpha/route.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/network/sip_proxy/v3alpha/route.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nFcontrib/envoy/extensions/filters/network/sip_proxy/v3alpha/route.proto\x12\x32\x65nvoy.extensions.filters.network.sip_proxy.v3alpha\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"m\n\x12RouteConfiguration\x12\x0c\n\x04name\x18\x01 \x01(\t\x12I\n\x06routes\x18\x02 \x03(\x0b\x32\x39.envoy.extensions.filters.network.sip_proxy.v3alpha.Route\"\xba\x01\n\x05Route\x12W\n\x05match\x18\x01 \x01(\x0b\x32>.envoy.extensions.filters.network.sip_proxy.v3alpha.RouteMatchB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12X\n\x05route\x18\x02 \x01(\x0b\x32?.envoy.extensions.filters.network.sip_proxy.v3alpha.RouteActionB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\"Y\n\nRouteMatch\x12\x10\n\x06\x64omain\x18\x01 \x01(\tH\x00\x12\x0e\n\x06header\x18\x02 \x01(\t\x12\x11\n\tparameter\x18\x03 \x01(\tB\x16\n\x0fmatch_specifier\x12\x03\xf8\x42\x01\"C\n\x0bRouteAction\x12\x1a\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01H\x00\x42\x18\n\x11\x63luster_specifier\x12\x03\xf8\x42\x01\x42\xbb\x01\n@io.envoyproxy.envoy.extensions.filters.network.sip_proxy.v3alphaB\nRouteProtoP\x01Zagithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/sip_proxy/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.sip_proxy.v3alpha.route_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n@io.envoyproxy.envoy.extensions.filters.network.sip_proxy.v3alphaB\nRouteProtoP\001Zagithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/sip_proxy/v3alpha\272\200\310\321\006\002\020\002'
  _globals['_ROUTE'].fields_by_name['match']._loaded_options = None
  _globals['_ROUTE'].fields_by_name['match']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_ROUTE'].fields_by_name['route']._loaded_options = None
  _globals['_ROUTE'].fields_by_name['route']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_ROUTEMATCH'].oneofs_by_name['match_specifier']._loaded_options = None
  _globals['_ROUTEMATCH'].oneofs_by_name['match_specifier']._serialized_options = b'\370B\001'
  _globals['_ROUTEACTION'].oneofs_by_name['cluster_specifier']._loaded_options = None
  _globals['_ROUTEACTION'].oneofs_by_name['cluster_specifier']._serialized_options = b'\370B\001'
  _globals['_ROUTEACTION'].fields_by_name['cluster']._loaded_options = None
  _globals['_ROUTEACTION'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_ROUTECONFIGURATION']._serialized_start=182
  _globals['_ROUTECONFIGURATION']._serialized_end=291
  _globals['_ROUTE']._serialized_start=294
  _globals['_ROUTE']._serialized_end=480
  _globals['_ROUTEMATCH']._serialized_start=482
  _globals['_ROUTEMATCH']._serialized_end=571
  _globals['_ROUTEACTION']._serialized_start=573
  _globals['_ROUTEACTION']._serialized_end=640
# @@protoc_insertion_point(module_scope)
