# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/compression/qatzstd/compressor/v3alpha/qatzstd.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/compression/qatzstd/compressor/v3alpha/qatzstd.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nMcontrib/envoy/extensions/compression/qatzstd/compressor/v3alpha/qatzstd.proto\x12\x37\x65nvoy.extensions.compression.qatzstd.compressor.v3alpha\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xf9\x03\n\x07Qatzstd\x12\x42\n\x11\x63ompression_level\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\t\xfa\x42\x06*\x04\x18\x16(\x01\x12\x17\n\x0f\x65nable_checksum\x18\x02 \x01(\x08\x12\x65\n\x08strategy\x18\x03 \x01(\x0e\x32I.envoy.extensions.compression.qatzstd.compressor.v3alpha.Qatzstd.StrategyB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12>\n\nchunk_size\x18\x05 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x0c\xfa\x42\t*\x07\x18\x80\x80\x04(\x80 \x12\x17\n\x0f\x65nable_qat_zstd\x18\x06 \x01(\x08\x12N\n\x1bqat_zstd_fallback_threshold\x18\x07 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x0b\xfa\x42\x08*\x06\x18\x80\x80\x04(\x00\"\x80\x01\n\x08Strategy\x12\x0b\n\x07\x44\x45\x46\x41ULT\x10\x00\x12\x08\n\x04\x46\x41ST\x10\x01\x12\t\n\x05\x44\x46\x41ST\x10\x02\x12\n\n\x06GREEDY\x10\x03\x12\x08\n\x04LAZY\x10\x04\x12\t\n\x05LAZY2\x10\x05\x12\x0b\n\x07\x42TLAZY2\x10\x06\x12\t\n\x05\x42TOPT\x10\x07\x12\x0b\n\x07\x42TULTRA\x10\x08\x12\x0c\n\x08\x42TULTRA2\x10\tB\xc7\x01\nEio.envoyproxy.envoy.extensions.compression.qatzstd.compressor.v3alphaB\x0cQatzstdProtoP\x01Zfgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/compression/qatzstd/compressor/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.compression.qatzstd.compressor.v3alpha.qatzstd_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nEio.envoyproxy.envoy.extensions.compression.qatzstd.compressor.v3alphaB\014QatzstdProtoP\001Zfgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/compression/qatzstd/compressor/v3alpha\272\200\310\321\006\002\020\002'
  _globals['_QATZSTD'].fields_by_name['compression_level']._loaded_options = None
  _globals['_QATZSTD'].fields_by_name['compression_level']._serialized_options = b'\372B\006*\004\030\026(\001'
  _globals['_QATZSTD'].fields_by_name['strategy']._loaded_options = None
  _globals['_QATZSTD'].fields_by_name['strategy']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_QATZSTD'].fields_by_name['chunk_size']._loaded_options = None
  _globals['_QATZSTD'].fields_by_name['chunk_size']._serialized_options = b'\372B\t*\007\030\200\200\004(\200 '
  _globals['_QATZSTD'].fields_by_name['qat_zstd_fallback_threshold']._loaded_options = None
  _globals['_QATZSTD'].fields_by_name['qat_zstd_fallback_threshold']._serialized_options = b'\372B\010*\006\030\200\200\004(\000'
  _globals['_QATZSTD']._serialized_start=227
  _globals['_QATZSTD']._serialized_end=732
  _globals['_QATZSTD_STRATEGY']._serialized_start=604
  _globals['_QATZSTD_STRATEGY']._serialized_end=732
# @@protoc_insertion_point(module_scope)
