// Copyright 2025 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "src/core/lib/promise/wait_set.h"

#include "absl/strings/str_join.h"

namespace grpc_core {

std::string WaitSet::ToString() {
  return absl::StrJoin(pending_, ", ",
                       [](std::string* out, const Waker& waker) {
                         absl::StrAppend(out, waker.DebugString());
                       });
}

}  // namespace grpc_core
