# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: contrib/envoy/extensions/filters/network/generic_proxy/codecs/dubbo/v3/dubbo.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nRcontrib/envoy/extensions/filters/network/generic_proxy/codecs/dubbo/v3/dubbo.proto\x12>envoy.extensions.filters.network.generic_proxy.codecs.dubbo.v3\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"\x12\n\x10\x44ubboCodecConfigB\xe3\x01\nLio.envoyproxy.envoy.extensions.filters.network.generic_proxy.codecs.dubbo.v3B\nDubboProtoP\x01Zugithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/generic_proxy/codecs/dubbo/v3;dubbov3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.generic_proxy.codecs.dubbo.v3.dubbo_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\nLio.envoyproxy.envoy.extensions.filters.network.generic_proxy.codecs.dubbo.v3B\nDubboProtoP\001Zugithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/generic_proxy/codecs/dubbo/v3;dubbov3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_DUBBOCODECCONFIG']._serialized_start=214
  _globals['_DUBBOCODECCONFIG']._serialized_end=232
# @@protoc_insertion_point(module_scope)
