/* This file was generated by upb_generator from the input file:
 *
 *     src/proto/grpc/channelz/v2/property_list.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "src/proto/grpc/channelz/v2/property_list.upb_minitable.h"
#include "google/protobuf/any.upb_minitable.h"
#include "google/protobuf/duration.upb_minitable.h"
#include "google/protobuf/empty.upb_minitable.h"
#include "google/protobuf/timestamp.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const UPB_PRIVATE(upb_GeneratedExtensionListEntry)* UPB_PRIVATE(upb_generated_extension_list);
typedef struct {
  upb_MiniTableField fields[1];
  upb_MiniTableSubInternal subs[1];
} grpc__channelz__v2__PropertyList_msg_init_Fields;

static const grpc__channelz__v2__PropertyList_msg_init_Fields grpc_channelz_v2_PropertyList__fields = {{
  {1, 8, 0, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__PropertyList__Element_msg_init},
}};

const upb_MiniTable grpc__channelz__v2__PropertyList_msg_init = {
  &grpc_channelz_v2_PropertyList__fields.fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.PropertyList",
#endif
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} grpc__channelz__v2__PropertyList__Element_msg_init_Fields;

static const grpc__channelz__v2__PropertyList__Element_msg_init_Fields grpc_channelz_v2_PropertyList_Element__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), 64, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__PropertyValue_msg_init},
}};

const upb_MiniTable grpc__channelz__v2__PropertyList__Element_msg_init = {
  &grpc_channelz_v2_PropertyList_Element__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.PropertyList.Element",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} grpc__channelz__v2__PropertyGrid_msg_init_Fields;

static const grpc__channelz__v2__PropertyGrid_msg_init_Fields grpc_channelz_v2_PropertyGrid__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), 0, 3, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__PropertyGrid__Row_msg_init},
}};

const upb_MiniTable grpc__channelz__v2__PropertyGrid_msg_init = {
  &grpc_channelz_v2_PropertyGrid__fields.fields[0],
  UPB_SIZE(16, 24), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.PropertyGrid",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Repeated_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} grpc__channelz__v2__PropertyGrid__Row_msg_init_Fields;

static const grpc__channelz__v2__PropertyGrid__Row_msg_init_Fields grpc_channelz_v2_PropertyGrid_Row__fields = {{
  {1, UPB_SIZE(12, 8), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(8, 24), 0, 3, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__PropertyValue_msg_init},
}};

const upb_MiniTable grpc__channelz__v2__PropertyGrid__Row_msg_init = {
  &grpc_channelz_v2_PropertyGrid_Row__fields.fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.PropertyGrid.Row",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} grpc__channelz__v2__PropertyTable_msg_init_Fields;

static const grpc__channelz__v2__PropertyTable_msg_init_Fields grpc_channelz_v2_PropertyTable__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), 0, 3, 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__PropertyTable__Row_msg_init},
}};

const upb_MiniTable grpc__channelz__v2__PropertyTable_msg_init = {
  &grpc_channelz_v2_PropertyTable__fields.fields[0],
  UPB_SIZE(16, 24), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.PropertyTable",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Repeated_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[1];
  upb_MiniTableSubInternal subs[1];
} grpc__channelz__v2__PropertyTable__Row_msg_init_Fields;

static const grpc__channelz__v2__PropertyTable__Row_msg_init_Fields grpc_channelz_v2_PropertyTable_Row__fields = {{
  {1, 8, 0, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &grpc__channelz__v2__PropertyValue_msg_init},
}};

const upb_MiniTable grpc__channelz__v2__PropertyTable__Row_msg_init = {
  &grpc_channelz_v2_PropertyTable_Row__fields.fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.PropertyTable.Row",
#endif
};

typedef struct {
  upb_MiniTableField fields[9];
  upb_MiniTableSubInternal subs[4];
} grpc__channelz__v2__PropertyValue_msg_init_Fields;

static const grpc__channelz__v2__PropertyValue_msg_init_Fields grpc_channelz_v2_PropertyValue__fields = {{
  {1, 16, -9, UPB_SIZE(27, 28), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, 16, -9, UPB_SIZE(25, 27), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, 16, -9, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {4, 16, -9, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {5, 16, -9, kUpb_NoSub, 4, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {6, 16, -9, kUpb_NoSub, 1, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {7, 16, -9, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
  {8, 16, -9, UPB_SIZE(8, 11), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {9, 16, -9, UPB_SIZE(6, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Empty_msg_init},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Duration_msg_init},
}};

const upb_MiniTable grpc__channelz__v2__PropertyValue_msg_init = {
  &grpc_channelz_v2_PropertyValue__fields.fields[0],
  UPB_SIZE(24, 32), 9, kUpb_ExtMode_NonExtendable, 9, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "grpc.channelz.v2.PropertyValue",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000080300001a, &upb_DecodeFast_String_Oneof_Tag1Byte},
    {0x0010000804000020, &upb_DecodeFast_Varint64_Oneof_Tag1Byte},
    {0x0010000805000028, &upb_DecodeFast_Varint64_Oneof_Tag1Byte},
    {0x0010000806000031, &upb_DecodeFast_Fixed64_Oneof_Tag1Byte},
    {0x0010000807000038, &upb_DecodeFast_Bool_Oneof_Tag1Byte},
  })
};

static const upb_MiniTable *messages_layout[7] = {
  &grpc__channelz__v2__PropertyList_msg_init,
  &grpc__channelz__v2__PropertyList__Element_msg_init,
  &grpc__channelz__v2__PropertyGrid_msg_init,
  &grpc__channelz__v2__PropertyGrid__Row_msg_init,
  &grpc__channelz__v2__PropertyTable_msg_init,
  &grpc__channelz__v2__PropertyTable__Row_msg_init,
  &grpc__channelz__v2__PropertyValue_msg_init,
};

const upb_MiniTableFile src_proto_grpc_channelz_v2_property_list_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  7,
  0,
  0,
};

#include "upb/port/undef.inc"

