/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/core/v3/base.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef ENVOY_CONFIG_CORE_V3_BASE_PROTO_UPB_H__UPB_MINITABLE_H_
#define ENVOY_CONFIG_CORE_V3_BASE_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__config__core__v3__Locality_msg_init;
extern const upb_MiniTable envoy__config__core__v3__BuildVersion_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Extension_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Node_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Node__DynamicParametersEntry_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Metadata_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Metadata__FilterMetadataEntry_msg_init;
extern const upb_MiniTable envoy__config__core__v3__Metadata__TypedFilterMetadataEntry_msg_init;
extern const upb_MiniTable envoy__config__core__v3__RuntimeUInt32_msg_init;
extern const upb_MiniTable envoy__config__core__v3__RuntimePercent_msg_init;
extern const upb_MiniTable envoy__config__core__v3__RuntimeDouble_msg_init;
extern const upb_MiniTable envoy__config__core__v3__RuntimeFeatureFlag_msg_init;
extern const upb_MiniTable envoy__config__core__v3__KeyValue_msg_init;
extern const upb_MiniTable envoy__config__core__v3__KeyValuePair_msg_init;
extern const upb_MiniTable envoy__config__core__v3__KeyValueAppend_msg_init;
extern const upb_MiniTable envoy__config__core__v3__KeyValueMutation_msg_init;
extern const upb_MiniTable envoy__config__core__v3__QueryParameter_msg_init;
extern const upb_MiniTable envoy__config__core__v3__HeaderValue_msg_init;
extern const upb_MiniTable envoy__config__core__v3__HeaderValueOption_msg_init;
extern const upb_MiniTable envoy__config__core__v3__HeaderMap_msg_init;
extern const upb_MiniTable envoy__config__core__v3__WatchedDirectory_msg_init;
extern const upb_MiniTable envoy__config__core__v3__DataSource_msg_init;
extern const upb_MiniTable envoy__config__core__v3__RetryPolicy_msg_init;
extern const upb_MiniTable envoy__config__core__v3__RetryPolicy__RetryPriority_msg_init;
extern const upb_MiniTable envoy__config__core__v3__RetryPolicy__RetryHostPredicate_msg_init;
extern const upb_MiniTable envoy__config__core__v3__RemoteDataSource_msg_init;
extern const upb_MiniTable envoy__config__core__v3__AsyncDataSource_msg_init;
extern const upb_MiniTable envoy__config__core__v3__TransportSocket_msg_init;
extern const upb_MiniTable envoy__config__core__v3__RuntimeFractionalPercent_msg_init;
extern const upb_MiniTable envoy__config__core__v3__ControlPlane_msg_init;

extern const upb_MiniTableFile envoy_config_core_v3_base_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_CONFIG_CORE_V3_BASE_PROTO_UPB_H__UPB_MINITABLE_H_ */
