/* This file was generated by upb_generator from the input file:
 *
 *     envoy/type/matcher/v3/http_inputs.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "envoy/type/matcher/v3/http_inputs.upb_minitable.h"
#include "udpa/annotations/status.upb_minitable.h"
#include "validate/validate.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const UPB_PRIVATE(upb_GeneratedExtensionListEntry)* UPB_PRIVATE(upb_generated_extension_list);
typedef struct {
  upb_MiniTableField fields[1];
} envoy__type__matcher__v3__HttpRequestHeaderMatchInput_msg_init_Fields;

static const envoy__type__matcher__v3__HttpRequestHeaderMatchInput_msg_init_Fields envoy_type_matcher_v3_HttpRequestHeaderMatchInput__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__type__matcher__v3__HttpRequestHeaderMatchInput_msg_init = {
  &envoy_type_matcher_v3_HttpRequestHeaderMatchInput__fields.fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.type.matcher.v3.HttpRequestHeaderMatchInput",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[1];
} envoy__type__matcher__v3__HttpRequestTrailerMatchInput_msg_init_Fields;

static const envoy__type__matcher__v3__HttpRequestTrailerMatchInput_msg_init_Fields envoy_type_matcher_v3_HttpRequestTrailerMatchInput__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__type__matcher__v3__HttpRequestTrailerMatchInput_msg_init = {
  &envoy_type_matcher_v3_HttpRequestTrailerMatchInput__fields.fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.type.matcher.v3.HttpRequestTrailerMatchInput",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[1];
} envoy__type__matcher__v3__HttpResponseHeaderMatchInput_msg_init_Fields;

static const envoy__type__matcher__v3__HttpResponseHeaderMatchInput_msg_init_Fields envoy_type_matcher_v3_HttpResponseHeaderMatchInput__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__type__matcher__v3__HttpResponseHeaderMatchInput_msg_init = {
  &envoy_type_matcher_v3_HttpResponseHeaderMatchInput__fields.fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.type.matcher.v3.HttpResponseHeaderMatchInput",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[1];
} envoy__type__matcher__v3__HttpResponseTrailerMatchInput_msg_init_Fields;

static const envoy__type__matcher__v3__HttpResponseTrailerMatchInput_msg_init_Fields envoy_type_matcher_v3_HttpResponseTrailerMatchInput__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__type__matcher__v3__HttpResponseTrailerMatchInput_msg_init = {
  &envoy_type_matcher_v3_HttpResponseTrailerMatchInput__fields.fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.type.matcher.v3.HttpResponseTrailerMatchInput",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[1];
} envoy__type__matcher__v3__HttpRequestQueryParamMatchInput_msg_init_Fields;

static const envoy__type__matcher__v3__HttpRequestQueryParamMatchInput_msg_init_Fields envoy_type_matcher_v3_HttpRequestQueryParamMatchInput__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__type__matcher__v3__HttpRequestQueryParamMatchInput_msg_init = {
  &envoy_type_matcher_v3_HttpRequestQueryParamMatchInput__fields.fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.type.matcher.v3.HttpRequestQueryParamMatchInput",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

static const upb_MiniTable *messages_layout[5] = {
  &envoy__type__matcher__v3__HttpRequestHeaderMatchInput_msg_init,
  &envoy__type__matcher__v3__HttpRequestTrailerMatchInput_msg_init,
  &envoy__type__matcher__v3__HttpResponseHeaderMatchInput_msg_init,
  &envoy__type__matcher__v3__HttpResponseTrailerMatchInput_msg_init,
  &envoy__type__matcher__v3__HttpRequestQueryParamMatchInput_msg_init,
};

const upb_MiniTableFile envoy_type_matcher_v3_http_inputs_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  5,
  0,
  0,
};

#include "upb/port/undef.inc"

