# Copyright 2026 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# absl Utilities

cdef extern from "absl/log/initialize.h" namespace "absl":
  void InitializeLog() nogil

cdef extern from "absl/base/log_severity.h" namespace "absl":
  cpdef enum class LogSeverityAtLeast(int):
    kInfo = 0
    kWarning = 1
    kError = 2
    kFatal = 3

cdef extern from "absl/log/globals.h" namespace "absl":
  void SetStderrThreshold(LogSeverityAtLeast threshold) nogil
