# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/filter/http/dynamo/v2/dynamo.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/filter/http/dynamo/v2/dynamo.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/envoy/config/filter/http/dynamo/v2/dynamo.proto\x12\"envoy.config.filter.http.dynamo.v2\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\"\x08\n\x06\x44ynamoB\xcc\x01\n0io.envoyproxy.envoy.config.filter.http.dynamo.v2B\x0b\x44ynamoProtoP\x01ZRgithub.com/envoyproxy/go-control-plane/envoy/config/filter/http/dynamo/v2;dynamov2\xf2\x98\xfe\x8f\x05)\x12\'envoy.extensions.filters.http.dynamo.v3\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.filter.http.dynamo.v2.dynamo_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n0io.envoyproxy.envoy.config.filter.http.dynamo.v2B\013DynamoProtoP\001ZRgithub.com/envoyproxy/go-control-plane/envoy/config/filter/http/dynamo/v2;dynamov2\362\230\376\217\005)\022\'envoy.extensions.filters.http.dynamo.v3\272\200\310\321\006\002\020\001'
  _globals['_DYNAMO']._serialized_start=150
  _globals['_DYNAMO']._serialized_end=158
# @@protoc_insertion_point(module_scope)
