# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: cel/expr/conformance/proto2/test_all_types_extensions.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'cel/expr/conformance/proto2/test_all_types_extensions.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cel.expr.conformance.proto2 import test_all_types_pb2 as cel_dot_expr_dot_conformance_dot_proto2_dot_test__all__types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;cel/expr/conformance/proto2/test_all_types_extensions.proto\x12\x1b\x63\x65l.expr.conformance.proto2\x1a\x30\x63\x65l/expr/conformance/proto2/test_all_types.proto\"\xda\x03\n\x1cProto2ExtensionScopedMessage2=\n\tint64_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xed\x07 \x01(\x03\x32x\n\x19message_scoped_nested_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xee\x07 \x01(\x0b\x32).cel.expr.conformance.proto2.TestAllTypes2y\n\x0fnested_enum_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xef\x07 \x01(\x0e\x32\x34.cel.expr.conformance.proto2.TestAllTypes.NestedEnum2\x85\x01\n&message_scoped_repeated_test_all_types\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xf0\x07 \x03(\x0b\x32).cel.expr.conformance.proto2.TestAllTypes:=\n\tint32_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xe8\x07 \x01(\x05:i\n\nnested_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xe9\x07 \x01(\x0b\x32).cel.expr.conformance.proto2.TestAllTypes:q\n\x12test_all_types_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xea\x07 \x01(\x0b\x32).cel.expr.conformance.proto2.TestAllTypes:y\n\x0fnested_enum_ext\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xeb\x07 \x01(\x0e\x32\x34.cel.expr.conformance.proto2.TestAllTypes.NestedEnum:v\n\x17repeated_test_all_types\x12).cel.expr.conformance.proto2.TestAllTypes\x18\xec\x07 \x03(\x0b\x32).cel.expr.conformance.proto2.TestAllTypesB_\n\x1f\x64\x65v.cel.expr.conformance.proto2B\x16TestAllTypesExtensionsP\x01Z\x1f\x63\x65l.dev/expr/conformance/proto2\xf8\x01\x01')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'cel.expr.conformance.proto2.test_all_types_extensions_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\037dev.cel.expr.conformance.proto2B\026TestAllTypesExtensionsP\001Z\037cel.dev/expr/conformance/proto2\370\001\001'
  _globals['_PROTO2EXTENSIONSCOPEDMESSAGE']._serialized_start=143
  _globals['_PROTO2EXTENSIONSCOPEDMESSAGE']._serialized_end=617
# @@protoc_insertion_point(module_scope)
