# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/network/dubbo_proxy/v3/dubbo_proxy.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/network/dubbo_proxy/v3/dubbo_proxy.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import config_source_pb2 as envoy_dot_config_dot_core_dot_v3_dot_config__source__pb2
from envoy.extensions.filters.network.dubbo_proxy.v3 import route_pb2 as envoy_dot_extensions_dot_filters_dot_network_dot_dubbo__proxy_dot_v3_dot_route__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAenvoy/extensions/filters/network/dubbo_proxy/v3/dubbo_proxy.proto\x12/envoy.extensions.filters.network.dubbo_proxy.v3\x1a(envoy/config/core/v3/config_source.proto\x1a;envoy/extensions/filters/network/dubbo_proxy/v3/route.proto\x1a\x19google/protobuf/any.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"f\n\x04\x44rds\x12\x43\n\rconfig_source\x18\x01 \x01(\x0b\x32\".envoy.config.core.v3.ConfigSourceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x19\n\x11route_config_name\x18\x02 \x01(\t\"\xd6\x05\n\nDubboProxy\x12\x1c\n\x0bstat_prefix\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12^\n\rprotocol_type\x18\x02 \x01(\x0e\x32=.envoy.extensions.filters.network.dubbo_proxy.v3.ProtocolTypeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12h\n\x12serialization_type\x18\x03 \x01(\x0e\x32\x42.envoy.extensions.filters.network.dubbo_proxy.v3.SerializationTypeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x66\n\x0croute_config\x18\x04 \x03(\x0b\x32\x43.envoy.extensions.filters.network.dubbo_proxy.v3.RouteConfigurationB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12^\n\x04\x64rds\x18\x06 \x01(\x0b\x32\x35.envoy.extensions.filters.network.dubbo_proxy.v3.DrdsB\x17\xf2\x98\xfe\x8f\x05\x11\x12\x0froute_specifierH\x00\x12l\n\x15multiple_route_config\x18\x07 \x01(\x0b\x32K.envoy.extensions.filters.network.dubbo_proxy.v3.MultipleRouteConfigurationH\x00\x12S\n\rdubbo_filters\x18\x05 \x03(\x0b\x32<.envoy.extensions.filters.network.dubbo_proxy.v3.DubboFilter:B\x9a\xc5\x88\x1e=\n;envoy.config.filter.network.dubbo_proxy.v2alpha1.DubboProxyB\x11\n\x0froute_specifier\"\x8f\x01\n\x0b\x44ubboFilter\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12$\n\x06\x63onfig\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any:C\x9a\xc5\x88\x1e>\n<envoy.config.filter.network.dubbo_proxy.v2alpha1.DubboFilter*\x19\n\x0cProtocolType\x12\t\n\x05\x44ubbo\x10\x00*!\n\x11SerializationType\x12\x0c\n\x08Hessian2\x10\x00\x42\xc0\x01\n=io.envoyproxy.envoy.extensions.filters.network.dubbo_proxy.v3B\x0f\x44ubboProxyProtoP\x01Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/dubbo_proxy/v3;dubbo_proxyv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.network.dubbo_proxy.v3.dubbo_proxy_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n=io.envoyproxy.envoy.extensions.filters.network.dubbo_proxy.v3B\017DubboProxyProtoP\001Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/dubbo_proxy/v3;dubbo_proxyv3\272\200\310\321\006\002\020\002'
  _globals['_DRDS'].fields_by_name['config_source']._loaded_options = None
  _globals['_DRDS'].fields_by_name['config_source']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_DUBBOPROXY'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_DUBBOPROXY'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_DUBBOPROXY'].fields_by_name['protocol_type']._loaded_options = None
  _globals['_DUBBOPROXY'].fields_by_name['protocol_type']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_DUBBOPROXY'].fields_by_name['serialization_type']._loaded_options = None
  _globals['_DUBBOPROXY'].fields_by_name['serialization_type']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_DUBBOPROXY'].fields_by_name['route_config']._loaded_options = None
  _globals['_DUBBOPROXY'].fields_by_name['route_config']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_DUBBOPROXY'].fields_by_name['drds']._loaded_options = None
  _globals['_DUBBOPROXY'].fields_by_name['drds']._serialized_options = b'\362\230\376\217\005\021\022\017route_specifier'
  _globals['_DUBBOPROXY']._loaded_options = None
  _globals['_DUBBOPROXY']._serialized_options = b'\232\305\210\036=\n;envoy.config.filter.network.dubbo_proxy.v2alpha1.DubboProxy'
  _globals['_DUBBOFILTER'].fields_by_name['name']._loaded_options = None
  _globals['_DUBBOFILTER'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_DUBBOFILTER']._loaded_options = None
  _globals['_DUBBOFILTER']._serialized_options = b'\232\305\210\036>\n<envoy.config.filter.network.dubbo_proxy.v2alpha1.DubboFilter'
  _globals['_PROTOCOLTYPE']._serialized_start=1387
  _globals['_PROTOCOLTYPE']._serialized_end=1412
  _globals['_SERIALIZATIONTYPE']._serialized_start=1414
  _globals['_SERIALIZATIONTYPE']._serialized_end=1447
  _globals['_DRDS']._serialized_start=408
  _globals['_DRDS']._serialized_end=510
  _globals['_DUBBOPROXY']._serialized_start=513
  _globals['_DUBBOPROXY']._serialized_end=1239
  _globals['_DUBBOFILTER']._serialized_start=1242
  _globals['_DUBBOFILTER']._serialized_end=1385
# @@protoc_insertion_point(module_scope)
