#!/bin/bash
# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

gource                          \
  --multi-sampling              \
  -s 0.1                        \
  --max-file-lag 0.05           \
  --max-files 0                 \
  -e 0.01                       \
  --hide filenames,dirnames,mouse,progress     \
  --disable-auto-rotate         \
  --file-filter '/grpc/doc/ref' \
  $*
