# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: bazel/cc_proto_descriptor_library/testdata/test.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'bazel/cc_proto_descriptor_library/testdata/test.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5bazel/cc_proto_descriptor_library/testdata/test.proto\x12\x1ctestdata.dynamic_descriptors\x1a\x19google/protobuf/any.proto\";\n\x03\x46oo\x12\x0b\n\x03\x62\x61r\x18\x01 \x01(\t\x12!\n\x03\x62\x61z\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any*\x04\x08\n\x10\x15')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bazel.cc_proto_descriptor_library.testdata.test_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_FOO']._serialized_start=114
  _globals['_FOO']._serialized_end=173
# @@protoc_insertion_point(module_scope)
