# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/access_loggers/fluentd/v3/fluentd.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/access_loggers/fluentd/v3/fluentd.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import backoff_pb2 as envoy_dot_config_dot_core_dot_v3_dot_backoff__pb2
from envoy.config.core.v3 import extension_pb2 as envoy_dot_config_dot_core_dot_v3_dot_extension__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n8envoy/extensions/access_loggers/fluentd/v3/fluentd.proto\x12*envoy.extensions.access_loggers.fluentd.v3\x1a\"envoy/config/core/v3/backoff.proto\x1a$envoy/config/core/v3/extension.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xcb\x04\n\x16\x46luentdAccessLogConfig\x12\x18\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x14\n\x03tag\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x1c\n\x0bstat_prefix\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x42\n\x15\x62uffer_flush_interval\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationB\x08\xfa\x42\x05\xaa\x01\x02*\x00\x12\x37\n\x11\x62uffer_size_bytes\x18\x05 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x12\x31\n\x06record\x18\x06 \x01(\x0b\x32\x17.google.protobuf.StructB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x66\n\rretry_options\x18\x07 \x01(\x0b\x32O.envoy.extensions.access_loggers.fluentd.v3.FluentdAccessLogConfig.RetryOptions\x12>\n\nformatters\x18\x08 \x03(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfig\x1a\x8a\x01\n\x0cRetryOptions\x12:\n\x14max_connect_attempts\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x12>\n\x0f\x62\x61\x63koff_options\x18\x02 \x01(\x0b\x32%.envoy.config.core.v3.BackoffStrategyB\xaf\x01\n8io.envoyproxy.envoy.extensions.access_loggers.fluentd.v3B\x0c\x46luentdProtoP\x01Z[github.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/fluentd/v3;fluentdv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.access_loggers.fluentd.v3.fluentd_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n8io.envoyproxy.envoy.extensions.access_loggers.fluentd.v3B\014FluentdProtoP\001Z[github.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/fluentd/v3;fluentdv3\272\200\310\321\006\002\020\002'
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['cluster']._loaded_options = None
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['tag']._loaded_options = None
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['tag']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['buffer_flush_interval']._loaded_options = None
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['buffer_flush_interval']._serialized_options = b'\372B\005\252\001\002*\000'
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['record']._loaded_options = None
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['record']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_FLUENTDACCESSLOGCONFIG']._serialized_start=329
  _globals['_FLUENTDACCESSLOGCONFIG']._serialized_end=916
  _globals['_FLUENTDACCESSLOGCONFIG_RETRYOPTIONS']._serialized_start=778
  _globals['_FLUENTDACCESSLOGCONFIG_RETRYOPTIONS']._serialized_end=916
# @@protoc_insertion_point(module_scope)
