# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/http/stateful_session/envelope/v3/envelope.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/http/stateful_session/envelope/v3/envelope.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAenvoy/extensions/http/stateful_session/envelope/v3/envelope.proto\x12\x32\x65nvoy.extensions.http.stateful_session.envelope.v3\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xa2\x01\n\x14\x45nvelopeSessionState\x12i\n\x06header\x18\x01 \x01(\x0b\x32O.envoy.extensions.http.stateful_session.envelope.v3.EnvelopeSessionState.HeaderB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x1a\x1f\n\x06Header\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x42\xc1\x01\n@io.envoyproxy.envoy.extensions.http.stateful_session.envelope.v3B\rEnvelopeProtoP\x01Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/http/stateful_session/envelope/v3;envelopev3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.http.stateful_session.envelope.v3.envelope_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n@io.envoyproxy.envoy.extensions.http.stateful_session.envelope.v3B\rEnvelopeProtoP\001Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/http/stateful_session/envelope/v3;envelopev3\272\200\310\321\006\002\020\002'
  _globals['_ENVELOPESESSIONSTATE_HEADER'].fields_by_name['name']._loaded_options = None
  _globals['_ENVELOPESESSIONSTATE_HEADER'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_ENVELOPESESSIONSTATE'].fields_by_name['header']._loaded_options = None
  _globals['_ENVELOPESESSIONSTATE'].fields_by_name['header']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_ENVELOPESESSIONSTATE']._serialized_start=178
  _globals['_ENVELOPESESSIONSTATE']._serialized_end=340
  _globals['_ENVELOPESESSIONSTATE_HEADER']._serialized_start=309
  _globals['_ENVELOPESESSIONSTATE_HEADER']._serialized_end=340
# @@protoc_insertion_point(module_scope)
