<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/proto/grpc/testing/messages.proto

namespace Grpc\Testing;

use Google\Protobuf\Internal\GPBType;
use Google\Protobuf\Internal\RepeatedField;
use Google\Protobuf\Internal\GPBUtil;

/**
 * A block of data, to simply increase gRPC message size.
 *
 * Generated from protobuf message <code>grpc.testing.Payload</code>
 */
class Payload extends \Google\Protobuf\Internal\Message
{
    /**
     * The type of data in body.
     *
     * Generated from protobuf field <code>.grpc.testing.PayloadType type = 1;</code>
     */
    protected $type = 0;
    /**
     * Primary contents of payload.
     *
     * Generated from protobuf field <code>bytes body = 2;</code>
     */
    protected $body = '';

    /**
     * Constructor.
     *
     * @param array $data {
     *     Optional. Data for populating the Message object.
     *
     *     @type int $type
     *           The type of data in body.
     *     @type string $body
     *           Primary contents of payload.
     * }
     */
    public function __construct($data = NULL) {
        \GPBMetadata\Src\Proto\Grpc\Testing\Messages::initOnce();
        parent::__construct($data);
    }

    /**
     * The type of data in body.
     *
     * Generated from protobuf field <code>.grpc.testing.PayloadType type = 1;</code>
     * @return int
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * The type of data in body.
     *
     * Generated from protobuf field <code>.grpc.testing.PayloadType type = 1;</code>
     * @param int $var
     * @return $this
     */
    public function setType($var)
    {
        GPBUtil::checkEnum($var, \Grpc\Testing\PayloadType::class);
        $this->type = $var;

        return $this;
    }

    /**
     * Primary contents of payload.
     *
     * Generated from protobuf field <code>bytes body = 2;</code>
     * @return string
     */
    public function getBody()
    {
        return $this->body;
    }

    /**
     * Primary contents of payload.
     *
     * Generated from protobuf field <code>bytes body = 2;</code>
     * @param string $var
     * @return $this
     */
    public function setBody($var)
    {
        GPBUtil::checkString($var, False);
        $this->body = $var;

        return $this;
    }

}

