<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: src/proto/grpc/testing/empty.proto

namespace GPBMetadata\Src\Proto\Grpc\Testing;

class PBEmpty
{
    public static $is_initialized = false;

    public static function initOnce() {
        $pool = \Google\Protobuf\Internal\DescriptorPool::getGeneratedPool();

        if (static::$is_initialized == true) {
          return;
        }
        $pool->internalAddGeneratedFile(
            "\x0AJ\x0A\"src/proto/grpc/testing/empty.proto\x12\x0Cgrpc.testing\"\x0E\x0A\x0CEmptyMessageb\x06proto3"
        , true);

        static::$is_initialized = true;
    }
}

