<?php
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: src/proto/grpc/testing/control.proto

namespace Grpc\Testing;

use Google\Protobuf\Internal\GPBType;
use Google\Protobuf\Internal\RepeatedField;
use Google\Protobuf\Internal\GPBUtil;

/**
 * Parameters of poisson process distribution, which is a good representation
 * of activity coming in from independent identical stationary sources.
 *
 * Generated from protobuf message <code>grpc.testing.PoissonParams</code>
 */
class PoissonParams extends \Google\Protobuf\Internal\Message
{
    /**
     * The rate of arrivals (a.k.a. lambda parameter of the exp distribution).
     *
     * Generated from protobuf field <code>double offered_load = 1;</code>
     */
    protected $offered_load = 0.0;

    /**
     * Constructor.
     *
     * @param array $data {
     *     Optional. Data for populating the Message object.
     *
     *     @type float $offered_load
     *           The rate of arrivals (a.k.a. lambda parameter of the exp distribution).
     * }
     */
    public function __construct($data = NULL) {
        \GPBMetadata\Src\Proto\Grpc\Testing\Control::initOnce();
        parent::__construct($data);
    }

    /**
     * The rate of arrivals (a.k.a. lambda parameter of the exp distribution).
     *
     * Generated from protobuf field <code>double offered_load = 1;</code>
     * @return float
     */
    public function getOfferedLoad()
    {
        return $this->offered_load;
    }

    /**
     * The rate of arrivals (a.k.a. lambda parameter of the exp distribution).
     *
     * Generated from protobuf field <code>double offered_load = 1;</code>
     * @param float $var
     * @return $this
     */
    public function setOfferedLoad($var)
    {
        GPBUtil::checkDouble($var);
        $this->offered_load = $var;

        return $this;
    }

}

