/* This file was generated by upb_generator from the input file:
 *
 *     cel/expr/syntax.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef CEL_EXPR_SYNTAX_PROTO_UPB_H__UPB_MINITABLE_H_
#define CEL_EXPR_SYNTAX_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable cel__expr__ParsedExpr_msg_init;
extern const upb_MiniTable cel__expr__Expr_msg_init;
extern const upb_MiniTable cel__expr__Expr__Ident_msg_init;
extern const upb_MiniTable cel__expr__Expr__Select_msg_init;
extern const upb_MiniTable cel__expr__Expr__Call_msg_init;
extern const upb_MiniTable cel__expr__Expr__CreateList_msg_init;
extern const upb_MiniTable cel__expr__Expr__CreateStruct_msg_init;
extern const upb_MiniTable cel__expr__Expr__CreateStruct__Entry_msg_init;
extern const upb_MiniTable cel__expr__Expr__Comprehension_msg_init;
extern const upb_MiniTable cel__expr__Constant_msg_init;
extern const upb_MiniTable cel__expr__SourceInfo_msg_init;
extern const upb_MiniTable cel__expr__SourceInfo__PositionsEntry_msg_init;
extern const upb_MiniTable cel__expr__SourceInfo__MacroCallsEntry_msg_init;
extern const upb_MiniTable cel__expr__SourceInfo__Extension_msg_init;
extern const upb_MiniTable cel__expr__SourceInfo__Extension__Version_msg_init;

extern const upb_MiniTableFile cel_expr_syntax_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* CEL_EXPR_SYNTAX_PROTO_UPB_H__UPB_MINITABLE_H_ */
