# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/http/sxg/v3alpha/sxg.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/http/sxg/v3alpha/sxg.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.extensions.transport_sockets.tls.v3 import secret_pb2 as envoy_dot_extensions_dot_transport__sockets_dot_tls_dot_v3_dot_secret__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;contrib/envoy/extensions/filters/http/sxg/v3alpha/sxg.proto\x12)envoy.extensions.filters.http.sxg.v3alpha\x1a\x36\x65nvoy/extensions/transport_sockets/tls/v3/secret.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xc5\x03\n\x03SXG\x12O\n\x0b\x63\x65rtificate\x18\x01 \x01(\x0b\x32:.envoy.extensions.transport_sockets.tls.v3.SdsSecretConfig\x12O\n\x0bprivate_key\x18\x02 \x01(\x0b\x32:.envoy.extensions.transport_sockets.tls.v3.SdsSecretConfig\x12+\n\x08\x64uration\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x16\n\x0emi_record_size\x18\x04 \x01(\x04\x12\x1c\n\x08\x63\x62or_url\x18\x05 \x01(\tB\n\xfa\x42\x07r\x05\x10\x01:\x01/\x12 \n\x0cvalidity_url\x18\x06 \x01(\tB\n\xfa\x42\x07r\x05\x10\x01:\x01/\x12\x34\n\x1c\x63lient_can_accept_sxg_header\x18\x07 \x01(\tB\x0e\xfa\x42\x0br\t\xc0\x01\x01\xc8\x01\x00\xd0\x01\x01\x12\x30\n\x18should_encode_sxg_header\x18\x08 \x01(\tB\x0e\xfa\x42\x0br\t\xc0\x01\x01\xc8\x01\x00\xd0\x01\x01\x12/\n\x15header_prefix_filters\x18\t \x03(\tB\x10\xfa\x42\r\x92\x01\n\"\x08r\x06\xc0\x01\x01\xc8\x01\x00\x42\xa9\x01\n7io.envoyproxy.envoy.extensions.filters.http.sxg.v3alphaB\x08SxgProtoP\x01ZXgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/http/sxg/v3alpha\xba\x80\xc8\xd1\x06\x04\x08\x01\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.http.sxg.v3alpha.sxg_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n7io.envoyproxy.envoy.extensions.filters.http.sxg.v3alphaB\010SxgProtoP\001ZXgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/http/sxg/v3alpha\272\200\310\321\006\004\010\001\020\002'
  _globals['_SXG'].fields_by_name['cbor_url']._loaded_options = None
  _globals['_SXG'].fields_by_name['cbor_url']._serialized_options = b'\372B\007r\005\020\001:\001/'
  _globals['_SXG'].fields_by_name['validity_url']._loaded_options = None
  _globals['_SXG'].fields_by_name['validity_url']._serialized_options = b'\372B\007r\005\020\001:\001/'
  _globals['_SXG'].fields_by_name['client_can_accept_sxg_header']._loaded_options = None
  _globals['_SXG'].fields_by_name['client_can_accept_sxg_header']._serialized_options = b'\372B\013r\t\300\001\001\310\001\000\320\001\001'
  _globals['_SXG'].fields_by_name['should_encode_sxg_header']._loaded_options = None
  _globals['_SXG'].fields_by_name['should_encode_sxg_header']._serialized_options = b'\372B\013r\t\300\001\001\310\001\000\320\001\001'
  _globals['_SXG'].fields_by_name['header_prefix_filters']._loaded_options = None
  _globals['_SXG'].fields_by_name['header_prefix_filters']._serialized_options = b'\372B\r\222\001\n\"\010r\006\300\001\001\310\001\000'
  _globals['_SXG']._serialized_start=251
  _globals['_SXG']._serialized_end=704
# @@protoc_insertion_point(module_scope)
