# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/service/accesslog/v3/als.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/service/accesslog/v3/als.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.data.accesslog.v3 import accesslog_pb2 as envoy_dot_data_dot_accesslog_dot_v3_dot_accesslog__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$envoy/service/accesslog/v3/als.proto\x12\x1a\x65nvoy.service.accesslog.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\'envoy/data/accesslog/v3/accesslog.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"V\n\x18StreamAccessLogsResponse::\x9a\xc5\x88\x1e\x35\n3envoy.service.accesslog.v2.StreamAccessLogsResponse\"\xff\x06\n\x17StreamAccessLogsMessage\x12R\n\nidentifier\x18\x01 \x01(\x0b\x32>.envoy.service.accesslog.v3.StreamAccessLogsMessage.Identifier\x12]\n\thttp_logs\x18\x02 \x01(\x0b\x32H.envoy.service.accesslog.v3.StreamAccessLogsMessage.HTTPAccessLogEntriesH\x00\x12[\n\x08tcp_logs\x18\x03 \x01(\x0b\x32G.envoy.service.accesslog.v3.StreamAccessLogsMessage.TCPAccessLogEntriesH\x00\x1a\xa1\x01\n\nIdentifier\x12\x32\n\x04node\x18\x01 \x01(\x0b\x32\x1a.envoy.config.core.v3.NodeB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x19\n\x08log_name\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01:D\x9a\xc5\x88\x1e?\n=envoy.service.accesslog.v2.StreamAccessLogsMessage.Identifier\x1a\xb0\x01\n\x14HTTPAccessLogEntries\x12H\n\tlog_entry\x18\x01 \x03(\x0b\x32+.envoy.data.accesslog.v3.HTTPAccessLogEntryB\x08\xfa\x42\x05\x92\x01\x02\x08\x01:N\x9a\xc5\x88\x1eI\nGenvoy.service.accesslog.v2.StreamAccessLogsMessage.HTTPAccessLogEntries\x1a\xad\x01\n\x13TCPAccessLogEntries\x12G\n\tlog_entry\x18\x01 \x03(\x0b\x32*.envoy.data.accesslog.v3.TCPAccessLogEntryB\x08\xfa\x42\x05\x92\x01\x02\x08\x01:M\x9a\xc5\x88\x1eH\nFenvoy.service.accesslog.v2.StreamAccessLogsMessage.TCPAccessLogEntries:9\x9a\xc5\x88\x1e\x34\n2envoy.service.accesslog.v2.StreamAccessLogsMessageB\x12\n\x0blog_entries\x12\x03\xf8\x42\x01\x32\x96\x01\n\x10\x41\x63\x63\x65ssLogService\x12\x81\x01\n\x10StreamAccessLogs\x12\x33.envoy.service.accesslog.v3.StreamAccessLogsMessage\x1a\x34.envoy.service.accesslog.v3.StreamAccessLogsResponse\"\x00(\x01\x42\x8d\x01\n(io.envoyproxy.envoy.service.accesslog.v3B\x08\x41lsProtoP\x01ZMgithub.com/envoyproxy/go-control-plane/envoy/service/accesslog/v3;accesslogv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.service.accesslog.v3.als_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n(io.envoyproxy.envoy.service.accesslog.v3B\010AlsProtoP\001ZMgithub.com/envoyproxy/go-control-plane/envoy/service/accesslog/v3;accesslogv3\272\200\310\321\006\002\020\002'
  _globals['_STREAMACCESSLOGSRESPONSE']._loaded_options = None
  _globals['_STREAMACCESSLOGSRESPONSE']._serialized_options = b'\232\305\210\0365\n3envoy.service.accesslog.v2.StreamAccessLogsResponse'
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER'].fields_by_name['node']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER'].fields_by_name['node']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER'].fields_by_name['log_name']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER'].fields_by_name['log_name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER']._serialized_options = b'\232\305\210\036?\n=envoy.service.accesslog.v2.StreamAccessLogsMessage.Identifier'
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES'].fields_by_name['log_entry']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES'].fields_by_name['log_entry']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES']._serialized_options = b'\232\305\210\036I\nGenvoy.service.accesslog.v2.StreamAccessLogsMessage.HTTPAccessLogEntries'
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES'].fields_by_name['log_entry']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES'].fields_by_name['log_entry']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES']._serialized_options = b'\232\305\210\036H\nFenvoy.service.accesslog.v2.StreamAccessLogsMessage.TCPAccessLogEntries'
  _globals['_STREAMACCESSLOGSMESSAGE'].oneofs_by_name['log_entries']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE'].oneofs_by_name['log_entries']._serialized_options = b'\370B\001'
  _globals['_STREAMACCESSLOGSMESSAGE']._loaded_options = None
  _globals['_STREAMACCESSLOGSMESSAGE']._serialized_options = b'\232\305\210\0364\n2envoy.service.accesslog.v2.StreamAccessLogsMessage'
  _globals['_STREAMACCESSLOGSRESPONSE']._serialized_start=233
  _globals['_STREAMACCESSLOGSRESPONSE']._serialized_end=319
  _globals['_STREAMACCESSLOGSMESSAGE']._serialized_start=322
  _globals['_STREAMACCESSLOGSMESSAGE']._serialized_end=1217
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER']._serialized_start=622
  _globals['_STREAMACCESSLOGSMESSAGE_IDENTIFIER']._serialized_end=783
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES']._serialized_start=786
  _globals['_STREAMACCESSLOGSMESSAGE_HTTPACCESSLOGENTRIES']._serialized_end=962
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES']._serialized_start=965
  _globals['_STREAMACCESSLOGSMESSAGE_TCPACCESSLOGENTRIES']._serialized_end=1138
  _globals['_ACCESSLOGSERVICE']._serialized_start=1220
  _globals['_ACCESSLOGSERVICE']._serialized_end=1370
# @@protoc_insertion_point(module_scope)
