# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/load_balancing_policies/subset/v3/subset.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/load_balancing_policies/subset/v3/subset.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.cluster.v3 import cluster_pb2 as envoy_dot_config_dot_cluster_dot_v3_dot_cluster__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?envoy/extensions/load_balancing_policies/subset/v3/subset.proto\x12\x32\x65nvoy.extensions.load_balancing_policies.subset.v3\x1a%envoy/config/cluster/v3/cluster.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xb4\t\n\x06Subset\x12t\n\x0f\x66\x61llback_policy\x18\x01 \x01(\x0e\x32Q.envoy.extensions.load_balancing_policies.subset.v3.Subset.LbSubsetFallbackPolicyB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12/\n\x0e\x64\x65\x66\x61ult_subset\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x65\n\x10subset_selectors\x18\x03 \x03(\x0b\x32K.envoy.extensions.load_balancing_policies.subset.v3.Subset.LbSubsetSelector\x12\x1c\n\x14\x61llow_redundant_keys\x18\n \x01(\x08\x12\x1d\n\x15locality_weight_aware\x18\x04 \x01(\x08\x12\x1d\n\x15scale_locality_weight\x18\x05 \x01(\x08\x12\x16\n\x0epanic_mode_any\x18\x06 \x01(\x08\x12\x13\n\x0blist_as_any\x18\x07 \x01(\x08\x12\x85\x01\n\x18metadata_fallback_policy\x18\x08 \x01(\x0e\x32Y.envoy.extensions.load_balancing_policies.subset.v3.Subset.LbSubsetMetadataFallbackPolicyB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12P\n\x10subset_lb_policy\x18\t \x01(\x0b\x32,.envoy.config.cluster.v3.LoadBalancingPolicyB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x1a\xe9\x02\n\x10LbSubsetSelector\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\x1e\n\x16single_host_per_subset\x18\x04 \x01(\x08\x12\x8d\x01\n\x0f\x66\x61llback_policy\x18\x02 \x01(\x0e\x32j.envoy.extensions.load_balancing_policies.subset.v3.Subset.LbSubsetSelector.LbSubsetSelectorFallbackPolicyB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x1c\n\x14\x66\x61llback_keys_subset\x18\x03 \x03(\t\"y\n\x1eLbSubsetSelectorFallbackPolicy\x12\x0f\n\x0bNOT_DEFINED\x10\x00\x12\x0f\n\x0bNO_FALLBACK\x10\x01\x12\x10\n\x0c\x41NY_ENDPOINT\x10\x02\x12\x12\n\x0e\x44\x45\x46\x41ULT_SUBSET\x10\x03\x12\x0f\n\x0bKEYS_SUBSET\x10\x04\"O\n\x16LbSubsetFallbackPolicy\x12\x0f\n\x0bNO_FALLBACK\x10\x00\x12\x10\n\x0c\x41NY_ENDPOINT\x10\x01\x12\x12\n\x0e\x44\x45\x46\x41ULT_SUBSET\x10\x02\"M\n\x1eLbSubsetMetadataFallbackPolicy\x12\x18\n\x14METADATA_NO_FALLBACK\x10\x00\x12\x11\n\rFALLBACK_LIST\x10\x01:-\x9a\xc5\x88\x1e(\n&envoy.config.cluster.v3.LbSubsetConfigB\xbd\x01\n@io.envoyproxy.envoy.extensions.load_balancing_policies.subset.v3B\x0bSubsetProtoP\x01Zbgithub.com/envoyproxy/go-control-plane/envoy/extensions/load_balancing_policies/subset/v3;subsetv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.load_balancing_policies.subset.v3.subset_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n@io.envoyproxy.envoy.extensions.load_balancing_policies.subset.v3B\013SubsetProtoP\001Zbgithub.com/envoyproxy/go-control-plane/envoy/extensions/load_balancing_policies/subset/v3;subsetv3\272\200\310\321\006\002\020\002'
  _globals['_SUBSET_LBSUBSETSELECTOR'].fields_by_name['fallback_policy']._loaded_options = None
  _globals['_SUBSET_LBSUBSETSELECTOR'].fields_by_name['fallback_policy']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_SUBSET'].fields_by_name['fallback_policy']._loaded_options = None
  _globals['_SUBSET'].fields_by_name['fallback_policy']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_SUBSET'].fields_by_name['metadata_fallback_policy']._loaded_options = None
  _globals['_SUBSET'].fields_by_name['metadata_fallback_policy']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_SUBSET'].fields_by_name['subset_lb_policy']._loaded_options = None
  _globals['_SUBSET'].fields_by_name['subset_lb_policy']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_SUBSET']._loaded_options = None
  _globals['_SUBSET']._serialized_options = b'\232\305\210\036(\n&envoy.config.cluster.v3.LbSubsetConfig'
  _globals['_SUBSET']._serialized_start=280
  _globals['_SUBSET']._serialized_end=1484
  _globals['_SUBSET_LBSUBSETSELECTOR']._serialized_start=916
  _globals['_SUBSET_LBSUBSETSELECTOR']._serialized_end=1277
  _globals['_SUBSET_LBSUBSETSELECTOR_LBSUBSETSELECTORFALLBACKPOLICY']._serialized_start=1156
  _globals['_SUBSET_LBSUBSETSELECTOR_LBSUBSETSELECTORFALLBACKPOLICY']._serialized_end=1277
  _globals['_SUBSET_LBSUBSETFALLBACKPOLICY']._serialized_start=1279
  _globals['_SUBSET_LBSUBSETFALLBACKPOLICY']._serialized_end=1358
  _globals['_SUBSET_LBSUBSETMETADATAFALLBACKPOLICY']._serialized_start=1360
  _globals['_SUBSET_LBSUBSETMETADATAFALLBACKPOLICY']._serialized_end=1437
# @@protoc_insertion_point(module_scope)
