# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/network/sip_proxy/tra/v3alpha/tra.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/network/sip_proxy/tra/v3alpha/tra.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import config_source_pb2 as envoy_dot_config_dot_core_dot_v3_dot_config__source__pb2
from envoy.config.core.v3 import grpc_service_pb2 as envoy_dot_config_dot_core_dot_v3_dot_grpc__service__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nHcontrib/envoy/extensions/filters/network/sip_proxy/tra/v3alpha/tra.proto\x12\x36\x65nvoy.extensions.filters.network.sip_proxy.tra.v3alpha\x1a(envoy/config/core/v3/config_source.proto\x1a\'envoy/config/core/v3/grpc_service.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xcc\x01\n\x10TraServiceConfig\x12\x41\n\x0cgrpc_service\x18\x01 \x01(\x0b\x32!.envoy.config.core.v3.GrpcServiceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12I\n\x15transport_api_version\x18\x02 \x01(\x0e\x32 .envoy.config.core.v3.ApiVersionB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12*\n\x07timeout\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\"\x9b\x04\n\x11TraServiceRequest\x12\x0c\n\x04type\x18\x01 \x01(\t\x12_\n\x0e\x63reate_request\x18\x02 \x01(\x0b\x32\x45.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.CreateRequestH\x00\x12_\n\x0eupdate_request\x18\x03 \x01(\x0b\x32\x45.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.UpdateRequestH\x00\x12\x63\n\x10retrieve_request\x18\x04 \x01(\x0b\x32G.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.RetrieveRequestH\x00\x12_\n\x0e\x64\x65lete_request\x18\x05 \x01(\x0b\x32\x45.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.DeleteRequestH\x00\x12\x65\n\x11subscribe_request\x18\x06 \x01(\x0b\x32H.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.SubscribeRequestH\x00\x42\t\n\x07request\"\xc4\x04\n\x12TraServiceResponse\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x0b\n\x03ret\x18\x02 \x01(\x05\x12\x0e\n\x06reason\x18\x03 \x01(\t\x12\x61\n\x0f\x63reate_response\x18\x04 \x01(\x0b\x32\x46.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.CreateResponseH\x00\x12\x61\n\x0fupdate_response\x18\x05 \x01(\x0b\x32\x46.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.UpdateResponseH\x00\x12\x65\n\x11retrieve_response\x18\x06 \x01(\x0b\x32H.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.RetrieveResponseH\x00\x12\x61\n\x0f\x64\x65lete_response\x18\x07 \x01(\x0b\x32\x46.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.DeleteResponseH\x00\x12g\n\x12subscribe_response\x18\x08 \x01(\x0b\x32I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.SubscribeResponseH\x00\x42\n\n\x08response\"\xb0\x02\n\rCreateRequest\x12]\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32O.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.CreateRequest.DataEntry\x12\x63\n\x07\x63ontext\x18\x02 \x03(\x0b\x32R.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.CreateRequest.ContextEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a.\n\x0c\x43ontextEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x10\n\x0e\x43reateResponse\"\xb0\x02\n\rUpdateRequest\x12]\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32O.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.UpdateRequest.DataEntry\x12\x63\n\x07\x63ontext\x18\x02 \x03(\x0b\x32R.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.UpdateRequest.ContextEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a.\n\x0c\x43ontextEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x10\n\x0eUpdateResponse\"\xb5\x01\n\x0fRetrieveRequest\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x65\n\x07\x63ontext\x18\x02 \x03(\x0b\x32T.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.RetrieveRequest.ContextEntry\x1a.\n\x0c\x43ontextEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xa1\x01\n\x10RetrieveResponse\x12`\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32R.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.RetrieveResponse.DataEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xb1\x01\n\rDeleteRequest\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x63\n\x07\x63ontext\x18\x02 \x03(\x0b\x32R.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.DeleteRequest.ContextEntry\x1a.\n\x0c\x43ontextEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x10\n\x0e\x44\x65leteResponse\"\x12\n\x10SubscribeRequest\"\xa3\x01\n\x11SubscribeResponse\x12\x61\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32S.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.SubscribeResponse.DataEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x32\xc7\x06\n\nTraService\x12\xa1\x01\n\x06\x43reate\x12I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceRequest\x1aJ.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceResponse\"\x00\x12\xa1\x01\n\x06Update\x12I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceRequest\x1aJ.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceResponse\"\x00\x12\xa3\x01\n\x08Retrieve\x12I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceRequest\x1aJ.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceResponse\"\x00\x12\xa1\x01\n\x06\x44\x65lete\x12I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceRequest\x1aJ.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceResponse\"\x00\x12\xa6\x01\n\tSubscribe\x12I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceRequest\x1aJ.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceResponse\"\x00\x30\x01\x42\xc1\x01\nDio.envoyproxy.envoy.extensions.filters.network.sip_proxy.tra.v3alphaB\x08TraProtoP\x01Zegithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/sip_proxy/tra/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.tra_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nDio.envoyproxy.envoy.extensions.filters.network.sip_proxy.tra.v3alphaB\010TraProtoP\001Zegithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/sip_proxy/tra/v3alpha\272\200\310\321\006\002\020\002'
  _globals['_TRASERVICECONFIG'].fields_by_name['grpc_service']._loaded_options = None
  _globals['_TRASERVICECONFIG'].fields_by_name['grpc_service']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_TRASERVICECONFIG'].fields_by_name['transport_api_version']._loaded_options = None
  _globals['_TRASERVICECONFIG'].fields_by_name['transport_api_version']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_CREATEREQUEST_DATAENTRY']._loaded_options = None
  _globals['_CREATEREQUEST_DATAENTRY']._serialized_options = b'8\001'
  _globals['_CREATEREQUEST_CONTEXTENTRY']._loaded_options = None
  _globals['_CREATEREQUEST_CONTEXTENTRY']._serialized_options = b'8\001'
  _globals['_UPDATEREQUEST_DATAENTRY']._loaded_options = None
  _globals['_UPDATEREQUEST_DATAENTRY']._serialized_options = b'8\001'
  _globals['_UPDATEREQUEST_CONTEXTENTRY']._loaded_options = None
  _globals['_UPDATEREQUEST_CONTEXTENTRY']._serialized_options = b'8\001'
  _globals['_RETRIEVEREQUEST_CONTEXTENTRY']._loaded_options = None
  _globals['_RETRIEVEREQUEST_CONTEXTENTRY']._serialized_options = b'8\001'
  _globals['_RETRIEVERESPONSE_DATAENTRY']._loaded_options = None
  _globals['_RETRIEVERESPONSE_DATAENTRY']._serialized_options = b'8\001'
  _globals['_DELETEREQUEST_CONTEXTENTRY']._loaded_options = None
  _globals['_DELETEREQUEST_CONTEXTENTRY']._serialized_options = b'8\001'
  _globals['_SUBSCRIBERESPONSE_DATAENTRY']._loaded_options = None
  _globals['_SUBSCRIBERESPONSE_DATAENTRY']._serialized_options = b'8\001'
  _globals['_TRASERVICECONFIG']._serialized_start=304
  _globals['_TRASERVICECONFIG']._serialized_end=508
  _globals['_TRASERVICEREQUEST']._serialized_start=511
  _globals['_TRASERVICEREQUEST']._serialized_end=1050
  _globals['_TRASERVICERESPONSE']._serialized_start=1053
  _globals['_TRASERVICERESPONSE']._serialized_end=1633
  _globals['_CREATEREQUEST']._serialized_start=1636
  _globals['_CREATEREQUEST']._serialized_end=1940
  _globals['_CREATEREQUEST_DATAENTRY']._serialized_start=1849
  _globals['_CREATEREQUEST_DATAENTRY']._serialized_end=1892
  _globals['_CREATEREQUEST_CONTEXTENTRY']._serialized_start=1894
  _globals['_CREATEREQUEST_CONTEXTENTRY']._serialized_end=1940
  _globals['_CREATERESPONSE']._serialized_start=1942
  _globals['_CREATERESPONSE']._serialized_end=1958
  _globals['_UPDATEREQUEST']._serialized_start=1961
  _globals['_UPDATEREQUEST']._serialized_end=2265
  _globals['_UPDATEREQUEST_DATAENTRY']._serialized_start=1849
  _globals['_UPDATEREQUEST_DATAENTRY']._serialized_end=1892
  _globals['_UPDATEREQUEST_CONTEXTENTRY']._serialized_start=1894
  _globals['_UPDATEREQUEST_CONTEXTENTRY']._serialized_end=1940
  _globals['_UPDATERESPONSE']._serialized_start=2267
  _globals['_UPDATERESPONSE']._serialized_end=2283
  _globals['_RETRIEVEREQUEST']._serialized_start=2286
  _globals['_RETRIEVEREQUEST']._serialized_end=2467
  _globals['_RETRIEVEREQUEST_CONTEXTENTRY']._serialized_start=1894
  _globals['_RETRIEVEREQUEST_CONTEXTENTRY']._serialized_end=1940
  _globals['_RETRIEVERESPONSE']._serialized_start=2470
  _globals['_RETRIEVERESPONSE']._serialized_end=2631
  _globals['_RETRIEVERESPONSE_DATAENTRY']._serialized_start=1849
  _globals['_RETRIEVERESPONSE_DATAENTRY']._serialized_end=1892
  _globals['_DELETEREQUEST']._serialized_start=2634
  _globals['_DELETEREQUEST']._serialized_end=2811
  _globals['_DELETEREQUEST_CONTEXTENTRY']._serialized_start=1894
  _globals['_DELETEREQUEST_CONTEXTENTRY']._serialized_end=1940
  _globals['_DELETERESPONSE']._serialized_start=2813
  _globals['_DELETERESPONSE']._serialized_end=2829
  _globals['_SUBSCRIBEREQUEST']._serialized_start=2831
  _globals['_SUBSCRIBEREQUEST']._serialized_end=2849
  _globals['_SUBSCRIBERESPONSE']._serialized_start=2852
  _globals['_SUBSCRIBERESPONSE']._serialized_end=3015
  _globals['_SUBSCRIBERESPONSE_DATAENTRY']._serialized_start=1849
  _globals['_SUBSCRIBERESPONSE_DATAENTRY']._serialized_end=1892
  _globals['_TRASERVICE']._serialized_start=3018
  _globals['_TRASERVICE']._serialized_end=3857
# @@protoc_insertion_point(module_scope)
