# Copyright 2022 The gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Retrieves supported bazel versions from plaintext file."""

_SUPPORTED_VERSIONS_FILE = "bazel/supported_versions.txt"


def _get_supported_bazel_versions():
    versions = []
    with open(_SUPPORTED_VERSIONS_FILE, "r") as f:
        for line in f:
            versions.append(line.strip())
    return versions


def mako_plugin(dictionary):
    versions = _get_supported_bazel_versions()
    dictionary["supported_bazel_versions"] = versions
    # The entry listed in the "supported_version.txt" file is considered
    # to be our repo-wide bazel version.
    dictionary["primary_bazel_version"] = versions[0]
