# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/http/ext_proc/response_processors/save_processing_response/v3/save_processing_response.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/http/ext_proc/response_processors/save_processing_response/v3/save_processing_response.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nmenvoy/extensions/http/ext_proc/response_processors/save_processing_response/v3/save_processing_response.proto\x12Nenvoy.extensions.http.ext_proc.response_processors.save_processing_response.v3\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"\xdd\x06\n\x16SaveProcessingResponse\x12 \n\x18\x66ilter_state_name_suffix\x18\x01 \x01(\t\x12\x90\x01\n\x14save_request_headers\x18\x02 \x01(\x0b\x32r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions\x12\x91\x01\n\x15save_response_headers\x18\x03 \x01(\x0b\x32r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions\x12\x91\x01\n\x15save_request_trailers\x18\x04 \x01(\x0b\x32r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions\x12\x92\x01\n\x16save_response_trailers\x18\x05 \x01(\x0b\x32r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions\x12\x93\x01\n\x17save_immediate_response\x18\x06 \x01(\x0b\x32r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions\x1a;\n\x0bSaveOptions\x12\x15\n\rsave_response\x18\x01 \x01(\x08\x12\x15\n\rsave_on_error\x18\x02 \x01(\x08\x42\xa0\x02\n\\io.envoyproxy.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3B\x1bSaveProcessingResponseProtoP\x01Z\x90\x01github.com/envoyproxy/go-control-plane/envoy/extensions/http/ext_proc/response_processors/save_processing_response/v3;save_processing_responsev3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.save_processing_response_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\\io.envoyproxy.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3B\033SaveProcessingResponseProtoP\001Z\220\001github.com/envoyproxy/go-control-plane/envoy/extensions/http/ext_proc/response_processors/save_processing_response/v3;save_processing_responsev3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_SAVEPROCESSINGRESPONSE']._serialized_start=258
  _globals['_SAVEPROCESSINGRESPONSE']._serialized_end=1119
  _globals['_SAVEPROCESSINGRESPONSE_SAVEOPTIONS']._serialized_start=1060
  _globals['_SAVEPROCESSINGRESPONSE_SAVEOPTIONS']._serialized_end=1119
# @@protoc_insertion_point(module_scope)
