/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/core/v3/base.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "envoy/config/core/v3/base.upb_minitable.h"
#include "envoy/config/core/v3/address.upb_minitable.h"
#include "envoy/config/core/v3/backoff.upb_minitable.h"
#include "envoy/config/core/v3/http_uri.upb_minitable.h"
#include "envoy/type/v3/percent.upb_minitable.h"
#include "envoy/type/v3/semantic_version.upb_minitable.h"
#include "google/protobuf/any.upb_minitable.h"
#include "google/protobuf/struct.upb_minitable.h"
#include "google/protobuf/wrappers.upb_minitable.h"
#include "xds/core/v3/context_params.upb_minitable.h"
#include "envoy/annotations/deprecation.upb_minitable.h"
#include "udpa/annotations/migrate.upb_minitable.h"
#include "udpa/annotations/status.upb_minitable.h"
#include "udpa/annotations/versioning.upb_minitable.h"
#include "validate/validate.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const UPB_PRIVATE(upb_GeneratedExtensionListEntry)* UPB_PRIVATE(upb_generated_extension_list);
typedef struct {
  upb_MiniTableField fields[3];
} envoy__config__core__v3__Locality_msg_init_Fields;

static const envoy__config__core__v3__Locality_msg_init_Fields envoy_config_core_v3_Locality__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(24, 40), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__config__core__v3__Locality_msg_init = {
  &envoy_config_core_v3_Locality__fields.fields[0],
  UPB_SIZE(32, 56), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.Locality",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x001800003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x002800003f00001a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[2];
} envoy__config__core__v3__BuildVersion_msg_init_Fields;

static const envoy__config__core__v3__BuildVersion_msg_init_Fields envoy_config_core_v3_BuildVersion__fields = {{
  {1, UPB_SIZE(12, 16), 64, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 65, UPB_SIZE(4, 5), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__type__v3__SemanticVersion_msg_init},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Struct_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__BuildVersion_msg_init = {
  &envoy_config_core_v3_BuildVersion__fields.fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.BuildVersion",
#endif
};

typedef struct {
  upb_MiniTableField fields[6];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__Extension_msg_init_Fields;

static const envoy__config__core__v3__Extension_msg_init_Fields envoy_config_core_v3_Extension__fields = {{
  {1, UPB_SIZE(20, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(28, 32), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(36, 48), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(12, 64), 64, 9, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, 9, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(16, 72), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__BuildVersion_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__Extension_msg_init = {
  &envoy_config_core_v3_Extension__fields.fields[0],
  UPB_SIZE(48, 80), 6, kUpb_ExtMode_NonExtendable, 6, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.Extension",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x002000003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x003000003f00001a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000900003f000028, &upb_DecodeFast_Bool_Scalar_Tag1Byte},
    {0x004800003f000032, &upb_DecodeFast_String_Repeated_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[11];
  upb_MiniTableSubInternal subs[6];
} envoy__config__core__v3__Node_msg_init_Fields;

static const envoy__config__core__v3__Node_msg_init_Fields envoy_config_core_v3_Node__fields = {{
  {1, UPB_SIZE(40, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(48, 32), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 80), 64, UPB_SIZE(27, 28), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(16, 88), 65, UPB_SIZE(25, 27), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(56, 48), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {7, 64, UPB_SIZE(-37, -13), kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {8, 64, UPB_SIZE(-37, -13), UPB_SIZE(17, 20), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {9, UPB_SIZE(20, 96), 0, UPB_SIZE(15, 19), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {10, UPB_SIZE(24, 104), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {11, UPB_SIZE(28, 112), 0, UPB_SIZE(10, 15), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {12, UPB_SIZE(32, 120), 0, UPB_SIZE(8, 14), 11, (int)kUpb_FieldMode_Map | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Struct_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__Locality_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__BuildVersion_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__Extension_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__Address_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__Node__DynamicParametersEntry_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__Node_msg_init = {
  &envoy_config_core_v3_Node__fields.fields[0],
  UPB_SIZE(72, 128), 11, kUpb_ExtMode_NonExtendable, 4, UPB_FASTTABLE_MASK(120), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.Node",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x002000003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x003000003f000032, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0040000c0700003a, &upb_DecodeFast_String_Oneof_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x006800003f000052, &upb_DecodeFast_String_Repeated_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__Node__DynamicParametersEntry_msg_init_Fields;

static const envoy__config__core__v3__Node__DynamicParametersEntry_msg_init_Fields envoy_config_core_v3_Node_DynamicParametersEntry__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, 32, 64, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &xds__core__v3__ContextParams_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__Node__DynamicParametersEntry_msg_init = {
  &envoy_config_core_v3_Node_DynamicParametersEntry__fields.fields[0],
  48, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.Node.DynamicParametersEntry",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[2];
} envoy__config__core__v3__Metadata_msg_init_Fields;

static const envoy__config__core__v3__Metadata_msg_init_Fields envoy_config_core_v3_Metadata__fields = {{
  {1, 8, 0, 6, 11, (int)kUpb_FieldMode_Map | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), 0, UPB_SIZE(4, 5), 11, (int)kUpb_FieldMode_Map | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__Metadata__FilterMetadataEntry_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__Metadata__TypedFilterMetadataEntry_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__Metadata_msg_init = {
  &envoy_config_core_v3_Metadata__fields.fields[0],
  UPB_SIZE(16, 24), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.Metadata",
#endif
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__Metadata__FilterMetadataEntry_msg_init_Fields;

static const envoy__config__core__v3__Metadata__FilterMetadataEntry_msg_init_Fields envoy_config_core_v3_Metadata_FilterMetadataEntry__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, 32, 64, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Struct_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__Metadata__FilterMetadataEntry_msg_init = {
  &envoy_config_core_v3_Metadata_FilterMetadataEntry__fields.fields[0],
  48, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.Metadata.FilterMetadataEntry",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__Metadata__TypedFilterMetadataEntry_msg_init_Fields;

static const envoy__config__core__v3__Metadata__TypedFilterMetadataEntry_msg_init_Fields envoy_config_core_v3_Metadata_TypedFilterMetadataEntry__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, 32, 64, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__Metadata__TypedFilterMetadataEntry_msg_init = {
  &envoy_config_core_v3_Metadata_TypedFilterMetadataEntry__fields.fields[0],
  48, 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.Metadata.TypedFilterMetadataEntry",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
} envoy__config__core__v3__RuntimeUInt32_msg_init_Fields;

static const envoy__config__core__v3__RuntimeUInt32_msg_init_Fields envoy_config_core_v3_RuntimeUInt32__fields = {{
  {2, 8, 0, kUpb_NoSub, 13, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__config__core__v3__RuntimeUInt32_msg_init = {
  &envoy_config_core_v3_RuntimeUInt32__fields.fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 0, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.RuntimeUInt32",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f000010, &upb_DecodeFast_Varint32_Scalar_Tag1Byte},
    {0x001000003f00001a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__RuntimePercent_msg_init_Fields;

static const envoy__config__core__v3__RuntimePercent_msg_init_Fields envoy_config_core_v3_RuntimePercent__fields = {{
  {1, UPB_SIZE(12, 32), 64, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__type__v3__Percent_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__RuntimePercent_msg_init = {
  &envoy_config_core_v3_RuntimePercent__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.RuntimePercent",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
} envoy__config__core__v3__RuntimeDouble_msg_init_Fields;

static const envoy__config__core__v3__RuntimeDouble_msg_init_Fields envoy_config_core_v3_RuntimeDouble__fields = {{
  {1, UPB_SIZE(16, 24), 0, kUpb_NoSub, 1, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
  {2, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__config__core__v3__RuntimeDouble_msg_init = {
  &envoy_config_core_v3_RuntimeDouble__fields.fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.RuntimeDouble",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001800003f000009, &upb_DecodeFast_Fixed64_Scalar_Tag1Byte},
    {0x000800003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__RuntimeFeatureFlag_msg_init_Fields;

static const envoy__config__core__v3__RuntimeFeatureFlag_msg_init_Fields envoy_config_core_v3_RuntimeFeatureFlag__fields = {{
  {1, UPB_SIZE(12, 32), 64, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__BoolValue_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__RuntimeFeatureFlag_msg_init = {
  &envoy_config_core_v3_RuntimeFeatureFlag__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.RuntimeFeatureFlag",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
} envoy__config__core__v3__KeyValue_msg_init_Fields;

static const envoy__config__core__v3__KeyValue_msg_init_Fields envoy_config_core_v3_KeyValue__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 0, kUpb_NoSub, 12, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__config__core__v3__KeyValue_msg_init = {
  &envoy_config_core_v3_KeyValue__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.KeyValue",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x001800003f000012, &upb_DecodeFast_Bytes_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__KeyValuePair_msg_init_Fields;

static const envoy__config__core__v3__KeyValuePair_msg_init_Fields envoy_config_core_v3_KeyValuePair__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), 64, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Value_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__KeyValuePair_msg_init = {
  &envoy_config_core_v3_KeyValuePair__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.KeyValuePair",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[2];
} envoy__config__core__v3__KeyValueAppend_msg_init_Fields;

static const envoy__config__core__v3__KeyValueAppend_msg_init_Fields envoy_config_core_v3_KeyValueAppend__fields = {{
  {1, UPB_SIZE(12, 16), 64, UPB_SIZE(9, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 12), 0, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 24), 65, UPB_SIZE(4, 6), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__KeyValue_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__KeyValuePair_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__KeyValueAppend_msg_init = {
  &envoy_config_core_v3_KeyValueAppend__fields.fields[0],
  UPB_SIZE(24, 32), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.KeyValueAppend",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000c00003f000010, &upb_DecodeFast_Varint32_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__KeyValueMutation_msg_init_Fields;

static const envoy__config__core__v3__KeyValueMutation_msg_init_Fields envoy_config_core_v3_KeyValueMutation__fields = {{
  {1, UPB_SIZE(12, 32), 64, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__KeyValueAppend_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__KeyValueMutation_msg_init = {
  &envoy_config_core_v3_KeyValueMutation__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.KeyValueMutation",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
} envoy__config__core__v3__QueryParameter_msg_init_Fields;

static const envoy__config__core__v3__QueryParameter_msg_init_Fields envoy_config_core_v3_QueryParameter__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__config__core__v3__QueryParameter_msg_init = {
  &envoy_config_core_v3_QueryParameter__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.QueryParameter",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x001800003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[3];
} envoy__config__core__v3__HeaderValue_msg_init_Fields;

static const envoy__config__core__v3__HeaderValue_msg_init_Fields envoy_config_core_v3_HeaderValue__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(24, 40), 0, kUpb_NoSub, 12, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__config__core__v3__HeaderValue_msg_init = {
  &envoy_config_core_v3_HeaderValue__fields.fields[0],
  UPB_SIZE(32, 56), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.HeaderValue",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x001800003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x002800003f00001a, &upb_DecodeFast_Bytes_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[4];
  upb_MiniTableSubInternal subs[2];
} envoy__config__core__v3__HeaderValueOption_msg_init_Fields;

static const envoy__config__core__v3__HeaderValueOption_msg_init_Fields envoy_config_core_v3_HeaderValueOption__fields = {{
  {1, UPB_SIZE(12, 16), 64, 12, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 65, UPB_SIZE(10, 11), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 12), 0, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
  {4, 9, 0, kUpb_NoSub, 8, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_1Byte << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__HeaderValue_msg_init},
  {.UPB_PRIVATE(submsg) = &google__protobuf__BoolValue_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__HeaderValueOption_msg_init = {
  &envoy_config_core_v3_HeaderValueOption__fields.fields[0],
  UPB_SIZE(24, 32), 4, kUpb_ExtMode_NonExtendable, 4, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.HeaderValueOption",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000c00003f000018, &upb_DecodeFast_Varint32_Scalar_Tag1Byte},
    {0x000900003f000020, &upb_DecodeFast_Bool_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[1];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__HeaderMap_msg_init_Fields;

static const envoy__config__core__v3__HeaderMap_msg_init_Fields envoy_config_core_v3_HeaderMap__fields = {{
  {1, 8, 0, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__HeaderValue_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__HeaderMap_msg_init = {
  &envoy_config_core_v3_HeaderMap__fields.fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.HeaderMap",
#endif
};

typedef struct {
  upb_MiniTableField fields[1];
} envoy__config__core__v3__WatchedDirectory_msg_init_Fields;

static const envoy__config__core__v3__WatchedDirectory_msg_init_Fields envoy_config_core_v3_WatchedDirectory__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__config__core__v3__WatchedDirectory_msg_init = {
  &envoy_config_core_v3_WatchedDirectory__fields.fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.WatchedDirectory",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[5];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__DataSource_msg_init_Fields;

static const envoy__config__core__v3__DataSource_msg_init_Fields envoy_config_core_v3_DataSource__fields = {{
  {1, UPB_SIZE(20, 16), UPB_SIZE(-17, -13), kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(20, 16), UPB_SIZE(-17, -13), kUpb_NoSub, 12, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(20, 16), UPB_SIZE(-17, -13), kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(20, 16), UPB_SIZE(-17, -13), kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(12, 32), 64, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__WatchedDirectory_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__DataSource_msg_init = {
  &envoy_config_core_v3_DataSource__fields.fields[0],
  UPB_SIZE(32, 40), 5, kUpb_ExtMode_NonExtendable, 5, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.DataSource",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0010000c0100000a, &upb_DecodeFast_String_Oneof_Tag1Byte},
    {0x0010000c02000012, &upb_DecodeFast_Bytes_Oneof_Tag1Byte},
    {0x0010000c0300001a, &upb_DecodeFast_String_Oneof_Tag1Byte},
    {0x0010000c04000022, &upb_DecodeFast_String_Oneof_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[6];
  upb_MiniTableSubInternal subs[4];
} envoy__config__core__v3__RetryPolicy_msg_init_Fields;

static const envoy__config__core__v3__RetryPolicy_msg_init_Fields envoy_config_core_v3_RetryPolicy__fields = {{
  {1, UPB_SIZE(12, 32), 64, 18, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 40), 65, UPB_SIZE(16, 17), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(28, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(20, 48), 66, UPB_SIZE(11, 13), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(24, 56), 0, UPB_SIZE(9, 12), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(40, 64), 0, kUpb_NoSub, 3, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_8Byte << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__BackoffStrategy_msg_init},
  {.UPB_PRIVATE(submsg) = &google__protobuf__UInt32Value_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__RetryPolicy__RetryPriority_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__RetryPolicy__RetryHostPredicate_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__RetryPolicy_msg_init = {
  &envoy_config_core_v3_RetryPolicy__fields.fields[0],
  UPB_SIZE(48, 72), 6, kUpb_ExtMode_NonExtendable, 6, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.RetryPolicy",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00001a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x004000003f000030, &upb_DecodeFast_Varint64_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__RetryPolicy__RetryPriority_msg_init_Fields;

static const envoy__config__core__v3__RetryPolicy__RetryPriority_msg_init_Fields envoy_config_core_v3_RetryPolicy_RetryPriority__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), -9, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__RetryPolicy__RetryPriority_msg_init = {
  &envoy_config_core_v3_RetryPolicy_RetryPriority__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.RetryPolicy.RetryPriority",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__RetryPolicy__RetryHostPredicate_msg_init_Fields;

static const envoy__config__core__v3__RetryPolicy__RetryHostPredicate_msg_init_Fields envoy_config_core_v3_RetryPolicy_RetryHostPredicate__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), -9, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__RetryPolicy__RetryHostPredicate_msg_init = {
  &envoy_config_core_v3_RetryPolicy_RetryHostPredicate__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.RetryPolicy.RetryHostPredicate",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[2];
} envoy__config__core__v3__RemoteDataSource_msg_init_Fields;

static const envoy__config__core__v3__RemoteDataSource_msg_init_Fields envoy_config_core_v3_RemoteDataSource__fields = {{
  {1, UPB_SIZE(12, 32), 64, UPB_SIZE(9, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(20, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(16, 40), 65, UPB_SIZE(4, 6), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__HttpUri_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__RetryPolicy_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__RemoteDataSource_msg_init = {
  &envoy_config_core_v3_RemoteDataSource__fields.fields[0],
  UPB_SIZE(32, 48), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.RemoteDataSource",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[2];
} envoy__config__core__v3__AsyncDataSource_msg_init_Fields;

static const envoy__config__core__v3__AsyncDataSource_msg_init_Fields envoy_config_core_v3_AsyncDataSource__fields = {{
  {1, UPB_SIZE(12, 16), -9, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), -9, UPB_SIZE(4, 5), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__DataSource_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__config__core__v3__RemoteDataSource_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__AsyncDataSource_msg_init = {
  &envoy_config_core_v3_AsyncDataSource__fields.fields[0],
  UPB_SIZE(16, 24), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.AsyncDataSource",
#endif
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__TransportSocket_msg_init_Fields;

static const envoy__config__core__v3__TransportSocket_msg_init_Fields envoy_config_core_v3_TransportSocket__fields = {{
  {1, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 32), -9, 3, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__TransportSocket_msg_init = {
  &envoy_config_core_v3_TransportSocket__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.TransportSocket",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[1];
} envoy__config__core__v3__RuntimeFractionalPercent_msg_init_Fields;

static const envoy__config__core__v3__RuntimeFractionalPercent_msg_init_Fields envoy_config_core_v3_RuntimeFractionalPercent__fields = {{
  {1, UPB_SIZE(12, 32), 64, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, 16, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__type__v3__FractionalPercent_msg_init},
}};

const upb_MiniTable envoy__config__core__v3__RuntimeFractionalPercent_msg_init = {
  &envoy_config_core_v3_RuntimeFractionalPercent__fields.fields[0],
  UPB_SIZE(24, 40), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(24), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.RuntimeFractionalPercent",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[1];
} envoy__config__core__v3__ControlPlane_msg_init_Fields;

static const envoy__config__core__v3__ControlPlane_msg_init_Fields envoy_config_core_v3_ControlPlane__fields = {{
  {1, 8, 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
}};

const upb_MiniTable envoy__config__core__v3__ControlPlane_msg_init = {
  &envoy_config_core_v3_ControlPlane__fields.fields[0],
  UPB_SIZE(16, 24), 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.config.core.v3.ControlPlane",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000800003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

static const upb_MiniTable *messages_layout[30] = {
  &envoy__config__core__v3__Locality_msg_init,
  &envoy__config__core__v3__BuildVersion_msg_init,
  &envoy__config__core__v3__Extension_msg_init,
  &envoy__config__core__v3__Node_msg_init,
  &envoy__config__core__v3__Node__DynamicParametersEntry_msg_init,
  &envoy__config__core__v3__Metadata_msg_init,
  &envoy__config__core__v3__Metadata__FilterMetadataEntry_msg_init,
  &envoy__config__core__v3__Metadata__TypedFilterMetadataEntry_msg_init,
  &envoy__config__core__v3__RuntimeUInt32_msg_init,
  &envoy__config__core__v3__RuntimePercent_msg_init,
  &envoy__config__core__v3__RuntimeDouble_msg_init,
  &envoy__config__core__v3__RuntimeFeatureFlag_msg_init,
  &envoy__config__core__v3__KeyValue_msg_init,
  &envoy__config__core__v3__KeyValuePair_msg_init,
  &envoy__config__core__v3__KeyValueAppend_msg_init,
  &envoy__config__core__v3__KeyValueMutation_msg_init,
  &envoy__config__core__v3__QueryParameter_msg_init,
  &envoy__config__core__v3__HeaderValue_msg_init,
  &envoy__config__core__v3__HeaderValueOption_msg_init,
  &envoy__config__core__v3__HeaderMap_msg_init,
  &envoy__config__core__v3__WatchedDirectory_msg_init,
  &envoy__config__core__v3__DataSource_msg_init,
  &envoy__config__core__v3__RetryPolicy_msg_init,
  &envoy__config__core__v3__RetryPolicy__RetryPriority_msg_init,
  &envoy__config__core__v3__RetryPolicy__RetryHostPredicate_msg_init,
  &envoy__config__core__v3__RemoteDataSource_msg_init,
  &envoy__config__core__v3__AsyncDataSource_msg_init,
  &envoy__config__core__v3__TransportSocket_msg_init,
  &envoy__config__core__v3__RuntimeFractionalPercent_msg_init,
  &envoy__config__core__v3__ControlPlane_msg_init,
};

const upb_MiniTableFile envoy_config_core_v3_base_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  30,
  0,
  0,
};

#include "upb/port/undef.inc"

