# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/grpc_service/channel_credentials/xds/v3/xds_credentials.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/grpc_service/channel_credentials/xds/v3/xds_credentials.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nNenvoy/extensions/grpc_service/channel_credentials/xds/v3/xds_credentials.proto\x12\x38\x65nvoy.extensions.grpc_service.channel_credentials.xds.v3\x1a\x19google/protobuf/any.proto\x1a\x1dudpa/annotations/status.proto\"D\n\x0eXdsCredentials\x12\x32\n\x14\x66\x61llback_credentials\x18\x01 \x01(\x0b\x32\x14.google.protobuf.AnyB\xce\x01\nFio.envoyproxy.envoy.extensions.grpc_service.channel_credentials.xds.v3B\x13XdsCredentialsProtoP\x01Zegithub.com/envoyproxy/go-control-plane/envoy/extensions/grpc_service/channel_credentials/xds/v3;xdsv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.grpc_service.channel_credentials.xds.v3.xds_credentials_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nFio.envoyproxy.envoy.extensions.grpc_service.channel_credentials.xds.v3B\023XdsCredentialsProtoP\001Zegithub.com/envoyproxy/go-control-plane/envoy/extensions/grpc_service/channel_credentials/xds/v3;xdsv3\272\200\310\321\006\002\020\002'
  _globals['_XDSCREDENTIALS']._serialized_start=198
  _globals['_XDSCREDENTIALS']._serialized_end=266
# @@protoc_insertion_point(module_scope)
