# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/network/redis_proxy/v3/redis_proxy.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/network/redis_proxy/v3/redis_proxy.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import address_pb2 as envoy_dot_config_dot_core_dot_v3_dot_address__pb2
from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.config.core.v3 import grpc_service_pb2 as envoy_dot_config_dot_core_dot_v3_dot_grpc__service__pb2
from envoy.extensions.common.aws.v3 import credential_provider_pb2 as envoy_dot_extensions_dot_common_dot_aws_dot_v3_dot_credential__provider__pb2
from envoy.extensions.common.dynamic_forward_proxy.v3 import dns_cache_pb2 as envoy_dot_extensions_dot_common_dot_dynamic__forward__proxy_dot_v3_dot_dns__cache__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import sensitive_pb2 as udpa_dot_annotations_dot_sensitive__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAenvoy/extensions/filters/network/redis_proxy/v3/redis_proxy.proto\x12/envoy.extensions.filters.network.redis_proxy.v3\x1a\"envoy/config/core/v3/address.proto\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\'envoy/config/core/v3/grpc_service.proto\x1a\x38\x65nvoy/extensions/common/aws/v3/credential_provider.proto\x1a@envoy/extensions/common/dynamic_forward_proxy/v3/dns_cache.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a udpa/annotations/sensitive.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xcf\x18\n\nRedisProxy\x12\x1c\n\x0bstat_prefix\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12h\n\x08settings\x18\x03 \x01(\x0b\x32L.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.ConnPoolSettingsB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x19\n\x11latency_in_micros\x18\x04 \x01(\x08\x12_\n\rprefix_routes\x18\x05 \x01(\x0b\x32H.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.PrefixRoutes\x12U\n\x18\x64ownstream_auth_password\x18\x06 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x11\x18\x01\xb8\xb7\x8b\xa4\x02\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12K\n\x19\x64ownstream_auth_passwords\x18\t \x03(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12V\n\x06\x66\x61ults\x18\x08 \x03(\x0b\x32\x46.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.RedisFault\x12J\n\x18\x64ownstream_auth_username\x18\x07 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12j\n\x16\x65xternal_auth_provider\x18\n \x01(\x0b\x32J.envoy.extensions.filters.network.redis_proxy.v3.RedisExternalAuthProvider\x12\x17\n\x0f\x63ustom_commands\x18\x0b \x03(\t\x1a\xdb\x06\n\x10\x43onnPoolSettings\x12\x37\n\nop_timeout\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationB\x08\xfa\x42\x05\xaa\x01\x02\x08\x01\x12\x1a\n\x12\x65nable_hashtagging\x18\x02 \x01(\x08\x12\x1a\n\x12\x65nable_redirection\x18\x03 \x01(\x08\x12Z\n\x10\x64ns_cache_config\x18\t \x01(\x0b\x32@.envoy.extensions.common.dynamic_forward_proxy.v3.DnsCacheConfig\x12$\n\x1cmax_buffer_size_before_flush\x18\x04 \x01(\r\x12\x37\n\x14\x62uffer_flush_timeout\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x46\n max_upstream_unknown_connections\x18\x06 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x12\x1c\n\x14\x65nable_command_stats\x18\x08 \x01(\x08\x12v\n\x0bread_policy\x18\x07 \x01(\x0e\x32W.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.ConnPoolSettings.ReadPolicyB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12n\n\x15\x63onnection_rate_limit\x18\n \x01(\x0b\x32O.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.ConnectionRateLimit\"~\n\nReadPolicy\x12\x1b\n\x06MASTER\x10\x00\x1a\x0f\xf2\x98\xfe\x8f\x05\t\n\x07PRIMARY\x12)\n\rPREFER_MASTER\x10\x01\x1a\x16\xf2\x98\xfe\x8f\x05\x10\n\x0ePREFER_PRIMARY\x12\x0b\n\x07REPLICA\x10\x02\x12\x12\n\x0ePREFER_REPLICA\x10\x03\x12\x07\n\x03\x41NY\x10\x04:M\x9a\xc5\x88\x1eH\nFenvoy.config.filter.network.redis_proxy.v2.RedisProxy.ConnPoolSettings\x1a\xc4\x08\n\x0cPrefixRoutes\x12^\n\x06routes\x18\x01 \x03(\x0b\x32N.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.PrefixRoutes.Route\x12\x18\n\x10\x63\x61se_insensitive\x18\x02 \x01(\x08\x12g\n\x0f\x63\x61tch_all_route\x18\x04 \x01(\x0b\x32N.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.PrefixRoutes.Route\x1a\xec\x05\n\x05Route\x12\x18\n\x06prefix\x18\x01 \x01(\tB\x08\xfa\x42\x05r\x03(\xe8\x07\x12\x15\n\rremove_prefix\x18\x02 \x01(\x08\x12\x18\n\x07\x63luster\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x81\x01\n\x15request_mirror_policy\x18\x04 \x03(\x0b\x32\x62.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.PrefixRoutes.Route.RequestMirrorPolicy\x12\x15\n\rkey_formatter\x18\x05 \x01(\t\x12}\n\x13read_command_policy\x18\x06 \x01(\x0b\x32`.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.PrefixRoutes.Route.ReadCommandPolicy\x1a\xfd\x01\n\x13RequestMirrorPolicy\x12\x18\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12H\n\x10runtime_fraction\x18\x02 \x01(\x0b\x32..envoy.config.core.v3.RuntimeFractionalPercent\x12\x1d\n\x15\x65xclude_read_commands\x18\x03 \x01(\x08:c\x9a\xc5\x88\x1e^\n\\envoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutes.Route.RequestMirrorPolicy\x1a-\n\x11ReadCommandPolicy\x12\x18\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01:O\x9a\xc5\x88\x1eJ\nHenvoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutes.Route:I\x9a\xc5\x88\x1e\x44\nBenvoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutesJ\x04\x08\x03\x10\x04R\x11\x63\x61tch_all_cluster\x1a\xb6\x02\n\nRedisFault\x12s\n\nfault_type\x18\x01 \x01(\x0e\x32U.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.RedisFault.RedisFaultTypeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12O\n\rfault_enabled\x18\x02 \x01(\x0b\x32..envoy.config.core.v3.RuntimeFractionalPercentB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12(\n\x05\x64\x65lay\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x10\n\x08\x63ommands\x18\x04 \x03(\t\"&\n\x0eRedisFaultType\x12\t\n\x05\x44\x45LAY\x10\x00\x12\t\n\x05\x45RROR\x10\x01\x1a\x45\n\x13\x43onnectionRateLimit\x12.\n\x1d\x63onnection_rate_limit_per_sec\x18\x01 \x01(\rB\x07\xfa\x42\x04*\x02 \x00:<\x9a\xc5\x88\x1e\x37\n5envoy.config.filter.network.redis_proxy.v2.RedisProxyJ\x04\x08\x02\x10\x03R\x07\x63luster\"\xd2\x04\n\x14RedisProtocolOptions\x12?\n\rauth_password\x18\x01 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12?\n\rauth_username\x18\x02 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12H\n\x07\x61ws_iam\x18\x03 \x01(\x0b\x32\x37.envoy.extensions.filters.network.redis_proxy.v3.AwsIam\x12\x65\n\x0b\x63redentials\x18\x04 \x03(\x0b\x32P.envoy.extensions.filters.network.redis_proxy.v3.RedisProtocolOptions.Credential\x1a\xbe\x01\n\nCredential\x12.\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0b\x32\x1d.envoy.config.core.v3.Address\x12?\n\rauth_password\x18\x02 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12?\n\rauth_username\x18\x03 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01:F\x9a\xc5\x88\x1e\x41\n?envoy.config.filter.network.redis_proxy.v2.RedisProtocolOptions\"\xe2\x01\n\x06\x41wsIam\x12R\n\x13\x63redential_provider\x18\x01 \x01(\x0b\x32\x35.envoy.extensions.common.aws.v3.AwsCredentialProvider\x12\x1b\n\ncache_name\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x14\n\x0cservice_name\x18\x03 \x01(\t\x12\x0e\n\x06region\x18\x04 \x01(\t\x12\x41\n\x0f\x65xpiration_time\x18\x05 \x01(\x0b\x32\x19.google.protobuf.DurationB\r\xfa\x42\n\xaa\x01\x07\"\x03\x08\x84\x07\x32\x00\"t\n\x19RedisExternalAuthProvider\x12\x37\n\x0cgrpc_service\x18\x01 \x01(\x0b\x32!.envoy.config.core.v3.GrpcService\x12\x1e\n\x16\x65nable_auth_expiration\x18\x02 \x01(\x08\x42\xc0\x01\n=io.envoyproxy.envoy.extensions.filters.network.redis_proxy.v3B\x0fRedisProxyProtoP\x01Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/redis_proxy/v3;redis_proxyv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.network.redis_proxy.v3.redis_proxy_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n=io.envoyproxy.envoy.extensions.filters.network.redis_proxy.v3B\017RedisProxyProtoP\001Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/redis_proxy/v3;redis_proxyv3\272\200\310\321\006\002\020\002'
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY'].values_by_name["MASTER"]._loaded_options = None
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY'].values_by_name["MASTER"]._serialized_options = b'\362\230\376\217\005\t\n\007PRIMARY'
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY'].values_by_name["PREFER_MASTER"]._loaded_options = None
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY'].values_by_name["PREFER_MASTER"]._serialized_options = b'\362\230\376\217\005\020\n\016PREFER_PRIMARY'
  _globals['_REDISPROXY_CONNPOOLSETTINGS'].fields_by_name['op_timeout']._loaded_options = None
  _globals['_REDISPROXY_CONNPOOLSETTINGS'].fields_by_name['op_timeout']._serialized_options = b'\372B\005\252\001\002\010\001'
  _globals['_REDISPROXY_CONNPOOLSETTINGS'].fields_by_name['read_policy']._loaded_options = None
  _globals['_REDISPROXY_CONNPOOLSETTINGS'].fields_by_name['read_policy']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_REDISPROXY_CONNPOOLSETTINGS']._loaded_options = None
  _globals['_REDISPROXY_CONNPOOLSETTINGS']._serialized_options = b'\232\305\210\036H\nFenvoy.config.filter.network.redis_proxy.v2.RedisProxy.ConnPoolSettings'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY'].fields_by_name['cluster']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY']._serialized_options = b'\232\305\210\036^\n\\envoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutes.Route.RequestMirrorPolicy'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_READCOMMANDPOLICY'].fields_by_name['cluster']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_READCOMMANDPOLICY'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE'].fields_by_name['prefix']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE'].fields_by_name['prefix']._serialized_options = b'\372B\005r\003(\350\007'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE'].fields_by_name['cluster']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE']._serialized_options = b'\232\305\210\036J\nHenvoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutes.Route'
  _globals['_REDISPROXY_PREFIXROUTES']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES']._serialized_options = b'\232\305\210\036D\nBenvoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutes'
  _globals['_REDISPROXY_REDISFAULT'].fields_by_name['fault_type']._loaded_options = None
  _globals['_REDISPROXY_REDISFAULT'].fields_by_name['fault_type']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_REDISPROXY_REDISFAULT'].fields_by_name['fault_enabled']._loaded_options = None
  _globals['_REDISPROXY_REDISFAULT'].fields_by_name['fault_enabled']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_REDISPROXY_CONNECTIONRATELIMIT'].fields_by_name['connection_rate_limit_per_sec']._loaded_options = None
  _globals['_REDISPROXY_CONNECTIONRATELIMIT'].fields_by_name['connection_rate_limit_per_sec']._serialized_options = b'\372B\004*\002 \000'
  _globals['_REDISPROXY'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_REDISPROXY'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_REDISPROXY'].fields_by_name['settings']._loaded_options = None
  _globals['_REDISPROXY'].fields_by_name['settings']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_password']._loaded_options = None
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_password']._serialized_options = b'\030\001\270\267\213\244\002\001\222\307\206\330\004\0033.0'
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_passwords']._loaded_options = None
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_passwords']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_username']._loaded_options = None
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_username']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROXY']._loaded_options = None
  _globals['_REDISPROXY']._serialized_options = b'\232\305\210\0367\n5envoy.config.filter.network.redis_proxy.v2.RedisProxy'
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL'].fields_by_name['auth_password']._loaded_options = None
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL'].fields_by_name['auth_password']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL'].fields_by_name['auth_username']._loaded_options = None
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL'].fields_by_name['auth_username']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROTOCOLOPTIONS'].fields_by_name['auth_password']._loaded_options = None
  _globals['_REDISPROTOCOLOPTIONS'].fields_by_name['auth_password']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROTOCOLOPTIONS'].fields_by_name['auth_username']._loaded_options = None
  _globals['_REDISPROTOCOLOPTIONS'].fields_by_name['auth_username']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROTOCOLOPTIONS']._loaded_options = None
  _globals['_REDISPROTOCOLOPTIONS']._serialized_options = b'\232\305\210\036A\n?envoy.config.filter.network.redis_proxy.v2.RedisProtocolOptions'
  _globals['_AWSIAM'].fields_by_name['cache_name']._loaded_options = None
  _globals['_AWSIAM'].fields_by_name['cache_name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_AWSIAM'].fields_by_name['expiration_time']._loaded_options = None
  _globals['_AWSIAM'].fields_by_name['expiration_time']._serialized_options = b'\372B\n\252\001\007\"\003\010\204\0072\000'
  _globals['_REDISPROXY']._serialized_start=611
  _globals['_REDISPROXY']._serialized_end=3762
  _globals['_REDISPROXY_CONNPOOLSETTINGS']._serialized_start=1347
  _globals['_REDISPROXY_CONNPOOLSETTINGS']._serialized_end=2206
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY']._serialized_start=2001
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY']._serialized_end=2127
  _globals['_REDISPROXY_PREFIXROUTES']._serialized_start=2209
  _globals['_REDISPROXY_PREFIXROUTES']._serialized_end=3301
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE']._serialized_start=2453
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE']._serialized_end=3201
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY']._serialized_start=2820
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY']._serialized_end=3073
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_READCOMMANDPOLICY']._serialized_start=3075
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_READCOMMANDPOLICY']._serialized_end=3120
  _globals['_REDISPROXY_REDISFAULT']._serialized_start=3304
  _globals['_REDISPROXY_REDISFAULT']._serialized_end=3614
  _globals['_REDISPROXY_REDISFAULT_REDISFAULTTYPE']._serialized_start=3576
  _globals['_REDISPROXY_REDISFAULT_REDISFAULTTYPE']._serialized_end=3614
  _globals['_REDISPROXY_CONNECTIONRATELIMIT']._serialized_start=3616
  _globals['_REDISPROXY_CONNECTIONRATELIMIT']._serialized_end=3685
  _globals['_REDISPROTOCOLOPTIONS']._serialized_start=3765
  _globals['_REDISPROTOCOLOPTIONS']._serialized_end=4359
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL']._serialized_start=4097
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL']._serialized_end=4287
  _globals['_AWSIAM']._serialized_start=4362
  _globals['_AWSIAM']._serialized_end=4588
  _globals['_REDISEXTERNALAUTHPROVIDER']._serialized_start=4590
  _globals['_REDISEXTERNALAUTHPROVIDER']._serialized_end=4706
# @@protoc_insertion_point(module_scope)
