# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/compressor/v3/compressor.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/compressor/v3/compressor.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.config.core.v3 import extension_pb2 as envoy_dot_config_dot_core_dot_v3_dot_extension__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<envoy/extensions/filters/http/compressor/v3/compressor.proto\x12+envoy.extensions.filters.http.compressor.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a$envoy/config/core/v3/extension.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xf3\t\n\nCompressor\x12\x41\n\x0e\x63ontent_length\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12!\n\x0c\x63ontent_type\x18\x02 \x03(\tB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12+\n\x16\x64isable_on_etag_header\x18\x03 \x01(\x08\x42\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12\x32\n\x1dremove_accept_encoding_header\x18\x04 \x01(\x08\x42\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12N\n\x0fruntime_enabled\x18\x05 \x01(\x0b\x32(.envoy.config.core.v3.RuntimeFeatureFlagB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12P\n\x12\x63ompressor_library\x18\x06 \x01(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfigB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12p\n\x18request_direction_config\x18\x07 \x01(\x0b\x32N.envoy.extensions.filters.http.compressor.v3.Compressor.RequestDirectionConfig\x12r\n\x19response_direction_config\x18\x08 \x01(\x0b\x32O.envoy.extensions.filters.http.compressor.v3.Compressor.ResponseDirectionConfig\x12\x14\n\x0c\x63hoose_first\x18\t \x01(\x08\x1a\xa2\x01\n\x15\x43ommonDirectionConfig\x12\x39\n\x07\x65nabled\x18\x01 \x01(\x0b\x32(.envoy.config.core.v3.RuntimeFeatureFlag\x12\x38\n\x12min_content_length\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x12\x14\n\x0c\x63ontent_type\x18\x03 \x03(\t\x1a~\n\x16RequestDirectionConfig\x12\x64\n\rcommon_config\x18\x01 \x01(\x0b\x32M.envoy.extensions.filters.http.compressor.v3.Compressor.CommonDirectionConfig\x1a\xa0\x02\n\x17ResponseDirectionConfig\x12\x64\n\rcommon_config\x18\x01 \x01(\x0b\x32M.envoy.extensions.filters.http.compressor.v3.Compressor.CommonDirectionConfig\x12\x1e\n\x16\x64isable_on_etag_header\x18\x02 \x01(\x08\x12%\n\x1dremove_accept_encoding_header\x18\x03 \x01(\x08\x12\x39\n\x1duncompressible_response_codes\x18\x04 \x03(\rB\x12\xfa\x42\x0f\x92\x01\x0c\x18\x01\"\x08*\x06\x10\xd8\x04(\xc8\x01\x12\x1d\n\x15status_header_enabled\x18\x05 \x01(\x08:8\x9a\xc5\x88\x1e\x33\n1envoy.config.filter.http.compressor.v2.Compressor\"_\n\x1aResponseDirectionOverrides\x12\x41\n\x1dremove_accept_encoding_header\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\"\xc9\x01\n\x13\x43ompressorOverrides\x12j\n\x19response_direction_config\x18\x01 \x01(\x0b\x32G.envoy.extensions.filters.http.compressor.v3.ResponseDirectionOverrides\x12\x46\n\x12\x63ompressor_library\x18\x02 \x01(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfig\"\x99\x01\n\x12\x43ompressorPerRoute\x12\x1b\n\x08\x64isabled\x18\x01 \x01(\x08\x42\x07\xfa\x42\x04j\x02\x08\x01H\x00\x12U\n\toverrides\x18\x02 \x01(\x0b\x32@.envoy.extensions.filters.http.compressor.v3.CompressorOverridesH\x00\x42\x0f\n\x08override\x12\x03\xf8\x42\x01\x42\xb7\x01\n9io.envoyproxy.envoy.extensions.filters.http.compressor.v3B\x0f\x43ompressorProtoP\x01Z_github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/compressor/v3;compressorv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.compressor.v3.compressor_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n9io.envoyproxy.envoy.extensions.filters.http.compressor.v3B\017CompressorProtoP\001Z_github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/compressor/v3;compressorv3\272\200\310\321\006\002\020\002'
  _globals['_COMPRESSOR_RESPONSEDIRECTIONCONFIG'].fields_by_name['uncompressible_response_codes']._loaded_options = None
  _globals['_COMPRESSOR_RESPONSEDIRECTIONCONFIG'].fields_by_name['uncompressible_response_codes']._serialized_options = b'\372B\017\222\001\014\030\001\"\010*\006\020\330\004(\310\001'
  _globals['_COMPRESSOR'].fields_by_name['content_length']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['content_length']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_COMPRESSOR'].fields_by_name['content_type']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['content_type']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_COMPRESSOR'].fields_by_name['disable_on_etag_header']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['disable_on_etag_header']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_COMPRESSOR'].fields_by_name['remove_accept_encoding_header']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['remove_accept_encoding_header']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_COMPRESSOR'].fields_by_name['runtime_enabled']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['runtime_enabled']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_COMPRESSOR'].fields_by_name['compressor_library']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['compressor_library']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_COMPRESSOR']._loaded_options = None
  _globals['_COMPRESSOR']._serialized_options = b'\232\305\210\0363\n1envoy.config.filter.http.compressor.v2.Compressor'
  _globals['_COMPRESSORPERROUTE'].oneofs_by_name['override']._loaded_options = None
  _globals['_COMPRESSORPERROUTE'].oneofs_by_name['override']._serialized_options = b'\370B\001'
  _globals['_COMPRESSORPERROUTE'].fields_by_name['disabled']._loaded_options = None
  _globals['_COMPRESSORPERROUTE'].fields_by_name['disabled']._serialized_options = b'\372B\004j\002\010\001'
  _globals['_COMPRESSOR']._serialized_start=341
  _globals['_COMPRESSOR']._serialized_end=1608
  _globals['_COMPRESSOR_COMMONDIRECTIONCONFIG']._serialized_start=969
  _globals['_COMPRESSOR_COMMONDIRECTIONCONFIG']._serialized_end=1131
  _globals['_COMPRESSOR_REQUESTDIRECTIONCONFIG']._serialized_start=1133
  _globals['_COMPRESSOR_REQUESTDIRECTIONCONFIG']._serialized_end=1259
  _globals['_COMPRESSOR_RESPONSEDIRECTIONCONFIG']._serialized_start=1262
  _globals['_COMPRESSOR_RESPONSEDIRECTIONCONFIG']._serialized_end=1550
  _globals['_RESPONSEDIRECTIONOVERRIDES']._serialized_start=1610
  _globals['_RESPONSEDIRECTIONOVERRIDES']._serialized_end=1705
  _globals['_COMPRESSOROVERRIDES']._serialized_start=1708
  _globals['_COMPRESSOROVERRIDES']._serialized_end=1909
  _globals['_COMPRESSORPERROUTE']._serialized_start=1912
  _globals['_COMPRESSORPERROUTE']._serialized_end=2065
# @@protoc_insertion_point(module_scope)
