# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/api_key_auth/v3/api_key_auth.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/api_key_auth/v3/api_key_auth.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import sensitive_pb2 as udpa_dot_annotations_dot_sensitive__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n@envoy/extensions/filters/http/api_key_auth/v3/api_key_auth.proto\x12-envoy.extensions.filters.http.api_key_auth.v3\x1a\x1fxds/annotations/v3/status.proto\x1a udpa/annotations/sensitive.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x82\x02\n\nApiKeyAuth\x12V\n\x0b\x63redentials\x18\x01 \x03(\x0b\x32\x39.envoy.extensions.filters.http.api_key_auth.v3.CredentialB\x06\xb8\xb7\x8b\xa4\x02\x01\x12M\n\x0bkey_sources\x18\x02 \x03(\x0b\x32\x38.envoy.extensions.filters.http.api_key_auth.v3.KeySource\x12M\n\nforwarding\x18\x03 \x01(\x0b\x32\x39.envoy.extensions.filters.http.api_key_auth.v3.Forwarding\"\xa3\x02\n\x12\x41piKeyAuthPerRoute\x12V\n\x0b\x63redentials\x18\x01 \x03(\x0b\x32\x39.envoy.extensions.filters.http.api_key_auth.v3.CredentialB\x06\xb8\xb7\x8b\xa4\x02\x01\x12M\n\x0bkey_sources\x18\x02 \x03(\x0b\x32\x38.envoy.extensions.filters.http.api_key_auth.v3.KeySource\x12\x17\n\x0f\x61llowed_clients\x18\x03 \x03(\t\x12M\n\nforwarding\x18\x04 \x01(\x0b\x32\x39.envoy.extensions.filters.http.api_key_auth.v3.Forwarding\";\n\nCredential\x12\x14\n\x03key\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x17\n\x06\x63lient\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\"j\n\tKeySource\x12!\n\x06header\x18\x01 \x01(\tB\x11\xfa\x42\x0er\x0c\x18\x80\x08\xc0\x01\x01\xc8\x01\x00\xd0\x01\x01\x12\x17\n\x05query\x18\x02 \x01(\tB\x08\xfa\x42\x05r\x03\x18\x80\x08\x12!\n\x06\x63ookie\x18\x03 \x01(\tB\x11\xfa\x42\x0er\x0c\x18\x80\x08\xc0\x01\x01\xc8\x01\x00\xd0\x01\x01\"F\n\nForwarding\x12\x1e\n\x06header\x18\x01 \x01(\tB\x0e\xfa\x42\x0br\t\x18\x80\x08\xc0\x01\x01\xc8\x01\x00\x12\x18\n\x10hide_credentials\x18\x02 \x01(\x08\x42\xc5\x01\n;io.envoyproxy.envoy.extensions.filters.http.api_key_auth.v3B\x0f\x41piKeyAuthProtoP\x01Zcgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/api_key_auth/v3;api_key_authv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.api_key_auth.v3.api_key_auth_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n;io.envoyproxy.envoy.extensions.filters.http.api_key_auth.v3B\017ApiKeyAuthProtoP\001Zcgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/api_key_auth/v3;api_key_authv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_APIKEYAUTH'].fields_by_name['credentials']._loaded_options = None
  _globals['_APIKEYAUTH'].fields_by_name['credentials']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_APIKEYAUTHPERROUTE'].fields_by_name['credentials']._loaded_options = None
  _globals['_APIKEYAUTHPERROUTE'].fields_by_name['credentials']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_CREDENTIAL'].fields_by_name['key']._loaded_options = None
  _globals['_CREDENTIAL'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CREDENTIAL'].fields_by_name['client']._loaded_options = None
  _globals['_CREDENTIAL'].fields_by_name['client']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_KEYSOURCE'].fields_by_name['header']._loaded_options = None
  _globals['_KEYSOURCE'].fields_by_name['header']._serialized_options = b'\372B\016r\014\030\200\010\300\001\001\310\001\000\320\001\001'
  _globals['_KEYSOURCE'].fields_by_name['query']._loaded_options = None
  _globals['_KEYSOURCE'].fields_by_name['query']._serialized_options = b'\372B\005r\003\030\200\010'
  _globals['_KEYSOURCE'].fields_by_name['cookie']._loaded_options = None
  _globals['_KEYSOURCE'].fields_by_name['cookie']._serialized_options = b'\372B\016r\014\030\200\010\300\001\001\310\001\000\320\001\001'
  _globals['_FORWARDING'].fields_by_name['header']._loaded_options = None
  _globals['_FORWARDING'].fields_by_name['header']._serialized_options = b'\372B\013r\t\030\200\010\300\001\001\310\001\000'
  _globals['_APIKEYAUTH']._serialized_start=239
  _globals['_APIKEYAUTH']._serialized_end=497
  _globals['_APIKEYAUTHPERROUTE']._serialized_start=500
  _globals['_APIKEYAUTHPERROUTE']._serialized_end=791
  _globals['_CREDENTIAL']._serialized_start=793
  _globals['_CREDENTIAL']._serialized_end=852
  _globals['_KEYSOURCE']._serialized_start=854
  _globals['_KEYSOURCE']._serialized_end=960
  _globals['_FORWARDING']._serialized_start=962
  _globals['_FORWARDING']._serialized_end=1032
# @@protoc_insertion_point(module_scope)
