# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/network/client_ssl_auth/v3/client_ssl_auth.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/network/client_ssl_auth/v3/client_ssl_auth.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import address_pb2 as envoy_dot_config_dot_core_dot_v3_dot_address__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nQcontrib/envoy/extensions/filters/network/client_ssl_auth/v3/client_ssl_auth.proto\x12\x33\x65nvoy.extensions.filters.network.client_ssl_auth.v3\x1a\"envoy/config/core/v3/address.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x9b\x02\n\rClientSSLAuth\x12\'\n\x10\x61uth_api_cluster\x18\x01 \x01(\tB\r\xfa\x42\nr\x08\x10\x01\xc0\x01\x02\xc8\x01\x00\x12\x1c\n\x0bstat_prefix\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x30\n\rrefresh_delay\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12L\n\rip_white_list\x18\x04 \x03(\x0b\x32\x1f.envoy.config.core.v3.CidrRangeB\x14\xf2\x98\xfe\x8f\x05\x0e\n\x0cip_allowlist:C\x9a\xc5\x88\x1e>\n<envoy.config.filter.network.client_ssl_auth.v2.ClientSSLAuthB\xd7\x01\nAio.envoyproxy.envoy.extensions.filters.network.client_ssl_auth.v3B\x12\x43lientSslAuthProtoP\x01Ztgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/client_ssl_auth/v3;client_ssl_authv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.client_ssl_auth.v3.client_ssl_auth_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nAio.envoyproxy.envoy.extensions.filters.network.client_ssl_auth.v3B\022ClientSslAuthProtoP\001Ztgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/client_ssl_auth/v3;client_ssl_authv3\272\200\310\321\006\002\020\002'
  _globals['_CLIENTSSLAUTH'].fields_by_name['auth_api_cluster']._loaded_options = None
  _globals['_CLIENTSSLAUTH'].fields_by_name['auth_api_cluster']._serialized_options = b'\372B\nr\010\020\001\300\001\002\310\001\000'
  _globals['_CLIENTSSLAUTH'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_CLIENTSSLAUTH'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CLIENTSSLAUTH'].fields_by_name['ip_white_list']._loaded_options = None
  _globals['_CLIENTSSLAUTH'].fields_by_name['ip_white_list']._serialized_options = b'\362\230\376\217\005\016\n\014ip_allowlist'
  _globals['_CLIENTSSLAUTH']._loaded_options = None
  _globals['_CLIENTSSLAUTH']._serialized_options = b'\232\305\210\036>\n<envoy.config.filter.network.client_ssl_auth.v2.ClientSSLAuth'
  _globals['_CLIENTSSLAUTH']._serialized_start=330
  _globals['_CLIENTSSLAUTH']._serialized_end=613
# @@protoc_insertion_point(module_scope)
