// Copyright 2025 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

#include "src/core/call/metadata_channelz.h"

namespace grpc_core {

channelz::PropertyList MetadataToChannelzProperties(
    const Arena::PoolPtr<grpc_metadata_batch>& md) {
  if (md == nullptr) {
    return channelz::PropertyList();
  }
  return MetadataToChannelzProperties(*md);
}

channelz::PropertyList MetadataToChannelzProperties(
    const grpc_metadata_batch& md) {
  channelz::PropertyList properties;
  md.Log([&properties](absl::string_view key, absl::string_view value) {
    if (!IsMetadataKeyAllowedInDebugOutput(key)) return;
    properties.Add(key, value);
  });
  return properties;
}

}  // namespace grpc_core
