// Copyright 2018 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package grpc.gcp;

option java_package = "io.grpc.alts.internal";

// The security level of the created channel. The list is sorted in increasing
// level of security. This order must always be maintained.
enum SecurityLevel {
  SECURITY_NONE = 0;
  INTEGRITY_ONLY = 1;
  INTEGRITY_AND_PRIVACY = 2;
}

// Max and min supported RPC protocol versions.
message RpcProtocolVersions {
  // RPC version contains a major version and a minor version.
  message Version {
    uint32 major = 1;
    uint32 minor = 2;
  }
  // Maximum supported RPC version.
  Version max_rpc_version = 1;
  // Minimum supported RPC version.
  Version min_rpc_version = 2;
}

// The ordered list of protocols that the client wishes to use, or the set
// that the server supports.
message TransportProtocolPreferences {
  repeated string transport_protocol = 1;
}

// The negotiated transport protocol.
message NegotiatedTransportProtocol {
  string transport_protocol = 1;
}
