# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/admission_control/v3/admission_control.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/admission_control/v3/admission_control.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.type.v3 import range_pb2 as envoy_dot_type_dot_v3_dot_range__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nJenvoy/extensions/filters/http/admission_control/v3/admission_control.proto\x12\x32\x65nvoy.extensions.filters.http.admission_control.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\x19\x65nvoy/type/v3/range.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x9a\x07\n\x10\x41\x64missionControl\x12\x39\n\x07\x65nabled\x18\x01 \x01(\x0b\x32(.envoy.config.core.v3.RuntimeFeatureFlag\x12p\n\x10success_criteria\x18\x02 \x01(\x0b\x32T.envoy.extensions.filters.http.admission_control.v3.AdmissionControl.SuccessCriteriaH\x00\x12\x32\n\x0fsampling_window\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x37\n\naggression\x18\x04 \x01(\x0b\x32#.envoy.config.core.v3.RuntimeDouble\x12:\n\x0csr_threshold\x18\x05 \x01(\x0b\x32$.envoy.config.core.v3.RuntimePercent\x12:\n\rrps_threshold\x18\x06 \x01(\x0b\x32#.envoy.config.core.v3.RuntimeUInt32\x12G\n\x19max_rejection_probability\x18\x07 \x01(\x0b\x32$.envoy.config.core.v3.RuntimePercent\x1a\x8e\x03\n\x0fSuccessCriteria\x12x\n\rhttp_criteria\x18\x01 \x01(\x0b\x32\x61.envoy.extensions.filters.http.admission_control.v3.AdmissionControl.SuccessCriteria.HttpCriteria\x12x\n\rgrpc_criteria\x18\x02 \x01(\x0b\x32\x61.envoy.extensions.filters.http.admission_control.v3.AdmissionControl.SuccessCriteria.GrpcCriteria\x1aP\n\x0cHttpCriteria\x12@\n\x13http_success_status\x18\x01 \x03(\x0b\x32\x19.envoy.type.v3.Int32RangeB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x1a\x35\n\x0cGrpcCriteria\x12%\n\x13grpc_success_status\x18\x01 \x03(\rB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x42\x1a\n\x13\x65valuation_criteria\x12\x03\xf8\x42\x01\x42\xd2\x01\n@io.envoyproxy.envoy.extensions.filters.http.admission_control.v3B\x15\x41\x64missionControlProtoP\x01Zmgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/admission_control/v3;admission_controlv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.admission_control.v3.admission_control_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n@io.envoyproxy.envoy.extensions.filters.http.admission_control.v3B\025AdmissionControlProtoP\001Zmgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/admission_control/v3;admission_controlv3\272\200\310\321\006\002\020\002'
  _globals['_ADMISSIONCONTROL_SUCCESSCRITERIA_HTTPCRITERIA'].fields_by_name['http_success_status']._loaded_options = None
  _globals['_ADMISSIONCONTROL_SUCCESSCRITERIA_HTTPCRITERIA'].fields_by_name['http_success_status']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_ADMISSIONCONTROL_SUCCESSCRITERIA_GRPCCRITERIA'].fields_by_name['grpc_success_status']._loaded_options = None
  _globals['_ADMISSIONCONTROL_SUCCESSCRITERIA_GRPCCRITERIA'].fields_by_name['grpc_success_status']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_ADMISSIONCONTROL'].oneofs_by_name['evaluation_criteria']._loaded_options = None
  _globals['_ADMISSIONCONTROL'].oneofs_by_name['evaluation_criteria']._serialized_options = b'\370B\001'
  _globals['_ADMISSIONCONTROL']._serialized_start=279
  _globals['_ADMISSIONCONTROL']._serialized_end=1201
  _globals['_ADMISSIONCONTROL_SUCCESSCRITERIA']._serialized_start=775
  _globals['_ADMISSIONCONTROL_SUCCESSCRITERIA']._serialized_end=1173
  _globals['_ADMISSIONCONTROL_SUCCESSCRITERIA_HTTPCRITERIA']._serialized_start=1038
  _globals['_ADMISSIONCONTROL_SUCCESSCRITERIA_HTTPCRITERIA']._serialized_end=1118
  _globals['_ADMISSIONCONTROL_SUCCESSCRITERIA_GRPCCRITERIA']._serialized_start=1120
  _globals['_ADMISSIONCONTROL_SUCCESSCRITERIA_GRPCCRITERIA']._serialized_end=1173
# @@protoc_insertion_point(module_scope)
