# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: contrib/envoy/extensions/filters/network/generic_proxy/matcher/v3/matcher.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.type.matcher.v3 import string_pb2 as envoy_dot_type_dot_matcher_dot_v3_dot_string__pb2
from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nOcontrib/envoy/extensions/filters/network/generic_proxy/matcher/v3/matcher.proto\x12\x39\x65nvoy.extensions.filters.network.generic_proxy.matcher.v3\x1a\"envoy/type/matcher/v3/string.proto\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x13\n\x11ServiceMatchInput\"\x10\n\x0eHostMatchInput\"\x10\n\x0ePathMatchInput\"\x12\n\x10MethodMatchInput\"4\n\x12PropertyMatchInput\x12\x1e\n\rproperty_name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\"\x13\n\x11RequestMatchInput\"q\n\x12KeyValueMatchEntry\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x44\n\x0cstring_match\x18\x02 \x01(\x0b\x32$.envoy.type.matcher.v3.StringMatcherB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\"\x91\x02\n\x0eRequestMatcher\x12\x32\n\x04host\x18\x01 \x01(\x0b\x32$.envoy.type.matcher.v3.StringMatcher\x12\x32\n\x04path\x18\x02 \x01(\x0b\x32$.envoy.type.matcher.v3.StringMatcher\x12\x34\n\x06method\x18\x03 \x01(\x0b\x32$.envoy.type.matcher.v3.StringMatcher\x12\x61\n\nproperties\x18\x04 \x03(\x0b\x32M.envoy.extensions.filters.network.generic_proxy.matcher.v3.KeyValueMatchEntryB\xdd\x01\nGio.envoyproxy.envoy.extensions.filters.network.generic_proxy.matcher.v3B\x0cMatcherProtoP\x01Zrgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/generic_proxy/matcher/v3;matcherv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.generic_proxy.matcher.v3.matcher_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\nGio.envoyproxy.envoy.extensions.filters.network.generic_proxy.matcher.v3B\014MatcherProtoP\001Zrgithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/generic_proxy/matcher/v3;matcherv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _PROPERTYMATCHINPUT.fields_by_name['property_name']._options = None
  _PROPERTYMATCHINPUT.fields_by_name['property_name']._serialized_options = b'\372B\004r\002\020\001'
  _KEYVALUEMATCHENTRY.fields_by_name['name']._options = None
  _KEYVALUEMATCHENTRY.fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _KEYVALUEMATCHENTRY.fields_by_name['string_match']._options = None
  _KEYVALUEMATCHENTRY.fields_by_name['string_match']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_SERVICEMATCHINPUT']._serialized_start=267
  _globals['_SERVICEMATCHINPUT']._serialized_end=286
  _globals['_HOSTMATCHINPUT']._serialized_start=288
  _globals['_HOSTMATCHINPUT']._serialized_end=304
  _globals['_PATHMATCHINPUT']._serialized_start=306
  _globals['_PATHMATCHINPUT']._serialized_end=322
  _globals['_METHODMATCHINPUT']._serialized_start=324
  _globals['_METHODMATCHINPUT']._serialized_end=342
  _globals['_PROPERTYMATCHINPUT']._serialized_start=344
  _globals['_PROPERTYMATCHINPUT']._serialized_end=396
  _globals['_REQUESTMATCHINPUT']._serialized_start=398
  _globals['_REQUESTMATCHINPUT']._serialized_end=417
  _globals['_KEYVALUEMATCHENTRY']._serialized_start=419
  _globals['_KEYVALUEMATCHENTRY']._serialized_end=532
  _globals['_REQUESTMATCHER']._serialized_start=535
  _globals['_REQUESTMATCHER']._serialized_end=808
# @@protoc_insertion_point(module_scope)
