/* This file was generated by upb_generator from the input file:
 *
 *     envoy/admin/v3/config_dump.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#include <stddef.h>
#include "upb/generated_code_support.h"
#include "envoy/admin/v3/config_dump.upb_minitable.h"
#include "envoy/admin/v3/config_dump_shared.upb_minitable.h"
#include "envoy/config/bootstrap/v3/bootstrap.upb_minitable.h"
#include "google/protobuf/any.upb_minitable.h"
#include "google/protobuf/timestamp.upb_minitable.h"
#include "udpa/annotations/status.upb_minitable.h"
#include "udpa/annotations/versioning.upb_minitable.h"

// Must be last.
#include "upb/port/def.inc"

extern const UPB_PRIVATE(upb_GeneratedExtensionListEntry)* UPB_PRIVATE(upb_generated_extension_list);
typedef struct {
  upb_MiniTableField fields[1];
  upb_MiniTableSubInternal subs[1];
} envoy__admin__v3__ConfigDump_msg_init_Fields;

static const envoy__admin__v3__ConfigDump_msg_init_Fields envoy_admin_v3_ConfigDump__fields = {{
  {1, 8, 0, UPB_SIZE(3, 4), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init},
}};

const upb_MiniTable envoy__admin__v3__ConfigDump_msg_init = {
  &envoy_admin_v3_ConfigDump__fields.fields[0],
  16, 1, kUpb_ExtMode_NonExtendable, 1, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.admin.v3.ConfigDump",
#endif
};

typedef struct {
  upb_MiniTableField fields[2];
  upb_MiniTableSubInternal subs[2];
} envoy__admin__v3__BootstrapConfigDump_msg_init_Fields;

static const envoy__admin__v3__BootstrapConfigDump_msg_init_Fields envoy_admin_v3_BootstrapConfigDump__fields = {{
  {1, UPB_SIZE(12, 16), 64, 6, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(16, 24), 65, UPB_SIZE(4, 5), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__config__bootstrap__v3__Bootstrap_msg_init},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init},
}};

const upb_MiniTable envoy__admin__v3__BootstrapConfigDump_msg_init = {
  &envoy_admin_v3_BootstrapConfigDump__fields.fields[0],
  UPB_SIZE(24, 32), 2, kUpb_ExtMode_NonExtendable, 2, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.admin.v3.BootstrapConfigDump",
#endif
};

typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[3];
} envoy__admin__v3__SecretsConfigDump_msg_init_Fields;

static const envoy__admin__v3__SecretsConfigDump_msg_init_Fields envoy_admin_v3_SecretsConfigDump__fields = {{
  {1, 8, 0, UPB_SIZE(9, 10), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 16), 0, UPB_SIZE(7, 9), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(16, 24), 0, UPB_SIZE(5, 8), 11, (int)kUpb_FieldMode_Array | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &envoy__admin__v3__SecretsConfigDump__StaticSecret_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__admin__v3__SecretsConfigDump__DynamicSecret_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__admin__v3__SecretsConfigDump__DynamicSecret_msg_init},
}};

const upb_MiniTable envoy__admin__v3__SecretsConfigDump_msg_init = {
  &envoy_admin_v3_SecretsConfigDump__fields.fields[0],
  UPB_SIZE(24, 32), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(255), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.admin.v3.SecretsConfigDump",
#endif
};

typedef struct {
  upb_MiniTableField fields[6];
  upb_MiniTableSubInternal subs[3];
} envoy__admin__v3__SecretsConfigDump__DynamicSecret_msg_init_Fields;

static const envoy__admin__v3__SecretsConfigDump__DynamicSecret_msg_init_Fields envoy_admin_v3_SecretsConfigDump_DynamicSecret__fields = {{
  {1, UPB_SIZE(28, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(36, 32), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(12, 48), 64, 12, 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {4, UPB_SIZE(16, 56), 65, UPB_SIZE(10, 11), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {5, UPB_SIZE(20, 64), 66, UPB_SIZE(8, 10), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {6, UPB_SIZE(24, 12), 0, kUpb_NoSub, 5, (int)kUpb_FieldMode_Scalar | (int)kUpb_LabelFlags_IsAlternate | ((int)kUpb_FieldRep_4Byte << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init},
  {.UPB_PRIVATE(submsg) = &envoy__admin__v3__UpdateFailureState_msg_init},
}};

const upb_MiniTable envoy__admin__v3__SecretsConfigDump__DynamicSecret_msg_init = {
  &envoy_admin_v3_SecretsConfigDump_DynamicSecret__fields.fields[0],
  UPB_SIZE(48, 72), 6, kUpb_ExtMode_NonExtendable, 6, UPB_FASTTABLE_MASK(56), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.admin.v3.SecretsConfigDump.DynamicSecret",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x002000003f000012, &upb_DecodeFast_String_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x000c00003f000030, &upb_DecodeFast_Varint32_Scalar_Tag1Byte},
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
  })
};

typedef struct {
  upb_MiniTableField fields[3];
  upb_MiniTableSubInternal subs[2];
} envoy__admin__v3__SecretsConfigDump__StaticSecret_msg_init_Fields;

static const envoy__admin__v3__SecretsConfigDump__StaticSecret_msg_init_Fields envoy_admin_v3_SecretsConfigDump_StaticSecret__fields = {{
  {1, UPB_SIZE(20, 16), 0, kUpb_NoSub, 9, (int)kUpb_FieldMode_Scalar | ((int)kUpb_FieldRep_StringView << kUpb_FieldRep_Shift)},
  {2, UPB_SIZE(12, 32), 64, UPB_SIZE(6, 7), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
  {3, UPB_SIZE(16, 40), 65, UPB_SIZE(4, 6), 11, (int)kUpb_FieldMode_Scalar | ((int)UPB_SIZE(kUpb_FieldRep_4Byte, kUpb_FieldRep_8Byte) << kUpb_FieldRep_Shift)},
 },
  {
  {.UPB_PRIVATE(submsg) = &google__protobuf__Timestamp_msg_init},
  {.UPB_PRIVATE(submsg) = &google__protobuf__Any_msg_init},
}};

const upb_MiniTable envoy__admin__v3__SecretsConfigDump__StaticSecret_msg_init = {
  &envoy_admin_v3_SecretsConfigDump_StaticSecret__fields.fields[0],
  UPB_SIZE(32, 48), 3, kUpb_ExtMode_NonExtendable, 3, UPB_FASTTABLE_MASK(8), 0,
#ifdef UPB_TRACING_ENABLED
  "envoy.admin.v3.SecretsConfigDump.StaticSecret",
#endif
  UPB_FASTTABLE_INIT({
    {0x0000000000000000, &_upb_FastDecoder_DecodeGeneric},
    {0x001000003f00000a, &upb_DecodeFast_String_Scalar_Tag1Byte},
  })
};

static const upb_MiniTable *messages_layout[5] = {
  &envoy__admin__v3__ConfigDump_msg_init,
  &envoy__admin__v3__BootstrapConfigDump_msg_init,
  &envoy__admin__v3__SecretsConfigDump_msg_init,
  &envoy__admin__v3__SecretsConfigDump__DynamicSecret_msg_init,
  &envoy__admin__v3__SecretsConfigDump__StaticSecret_msg_init,
};

const upb_MiniTableFile envoy_admin_v3_config_dump_proto_upb_file_layout = {
  messages_layout,
  NULL,
  NULL,
  5,
  0,
  0,
};

#include "upb/port/undef.inc"

