// Copyright 2017 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

import "src/proto/grpc/testing/control.proto";
import "src/proto/grpc/testing/stats.proto";

package grpc.testing;

message ProxyStat {
  double latency = 1;
}

service ProxyClientService {
  rpc GetConfig(Void) returns (ClientConfig);
  rpc ReportTime(stream ProxyStat) returns (Void);
  rpc ReportHist(stream HistogramData) returns (Void);
}

