/* This file was generated by upb_generator from the input file:
 *
 *     envoy/config/rbac/v3/rbac.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef ENVOY_CONFIG_RBAC_V3_RBAC_PROTO_UPB_H__UPB_MINITABLE_H_
#define ENVOY_CONFIG_RBAC_V3_RBAC_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable envoy__config__rbac__v3__RBAC_msg_init;
extern const upb_MiniTable envoy__config__rbac__v3__RBAC__AuditLoggingOptions_msg_init;
extern const upb_MiniTable envoy__config__rbac__v3__RBAC__AuditLoggingOptions__AuditLoggerConfig_msg_init;
extern const upb_MiniTable envoy__config__rbac__v3__RBAC__PoliciesEntry_msg_init;
extern const upb_MiniTable envoy__config__rbac__v3__Policy_msg_init;
extern const upb_MiniTable envoy__config__rbac__v3__SourcedMetadata_msg_init;
extern const upb_MiniTable envoy__config__rbac__v3__Permission_msg_init;
extern const upb_MiniTable envoy__config__rbac__v3__Permission__Set_msg_init;
extern const upb_MiniTable envoy__config__rbac__v3__Principal_msg_init;
extern const upb_MiniTable envoy__config__rbac__v3__Principal__Set_msg_init;
extern const upb_MiniTable envoy__config__rbac__v3__Principal__Authenticated_msg_init;
extern const upb_MiniTable envoy__config__rbac__v3__Action_msg_init;

extern const upb_MiniTableFile envoy_config_rbac_v3_rbac_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* ENVOY_CONFIG_RBAC_V3_RBAC_PROTO_UPB_H__UPB_MINITABLE_H_ */
