/* This file was generated by upb_generator from the input file:
 *
 *     src/proto/grpc/channelz/v2/promise.proto
 *
 * Do not edit -- your changes will be discarded when the file is
 * regenerated.
 * NO CHECKED-IN PROTOBUF GENCODE */

#ifndef SRC_PROTO_GRPC_CHANNELZ_V2_PROMISE_PROTO_UPB_H__UPB_MINITABLE_H_
#define SRC_PROTO_GRPC_CHANNELZ_V2_PROMISE_PROTO_UPB_H__UPB_MINITABLE_H_

#include "upb/generated_code_support.h"

// Must be last.
#include "upb/port/def.inc"

#ifdef __cplusplus
extern "C" {
#endif

extern const upb_MiniTable grpc__channelz__v2__Promise_msg_init;
extern const upb_MiniTable grpc__channelz__v2__Promise__Custom_msg_init;
extern const upb_MiniTable grpc__channelz__v2__Promise__If_msg_init;
extern const upb_MiniTable grpc__channelz__v2__Promise__Loop_msg_init;
extern const upb_MiniTable grpc__channelz__v2__Promise__ForEach_msg_init;
extern const upb_MiniTable grpc__channelz__v2__Promise__Race_msg_init;
extern const upb_MiniTable grpc__channelz__v2__Promise__SeqStep_msg_init;
extern const upb_MiniTable grpc__channelz__v2__Promise__Seq_msg_init;
extern const upb_MiniTable grpc__channelz__v2__Promise__JoinBranch_msg_init;
extern const upb_MiniTable grpc__channelz__v2__Promise__Join_msg_init;
extern const upb_MiniTable grpc__channelz__v2__Promise__Map_msg_init;

extern const upb_MiniTableFile src_proto_grpc_channelz_v2_promise_proto_upb_file_layout;

#ifdef __cplusplus
}  /* extern "C" */
#endif

#include "upb/port/undef.inc"

#endif  /* SRC_PROTO_GRPC_CHANNELZ_V2_PROMISE_PROTO_UPB_H__UPB_MINITABLE_H_ */
