# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/http/peak_ewma/v3alpha/peak_ewma.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/http/peak_ewma/v3alpha/peak_ewma.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nGcontrib/envoy/extensions/filters/http/peak_ewma/v3alpha/peak_ewma.proto\x12/envoy.extensions.filters.http.peak_ewma.v3alpha\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"\x1a\n\x0ePeakEwmaConfig:\x08\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x42\xc0\x01\n=io.envoyproxy.envoy.extensions.filters.http.peak_ewma.v3alphaB\rPeakEwmaProtoP\x01Z^github.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/http/peak_ewma/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.http.peak_ewma.v3alpha.peak_ewma_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n=io.envoyproxy.envoy.extensions.filters.http.peak_ewma.v3alphaB\rPeakEwmaProtoP\001Z^github.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/http/peak_ewma/v3alpha\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_PEAKEWMACONFIG']._loaded_options = None
  _globals['_PEAKEWMACONFIG']._serialized_options = b'\322\306\244\341\006\002\010\001'
  _globals['_PEAKEWMACONFIG']._serialized_start=188
  _globals['_PEAKEWMACONFIG']._serialized_end=214
# @@protoc_insertion_point(module_scope)
