// Protocol Buffers - Google's data interchange format
// Copyright 2023 Google LLC.  All rights reserved.
//
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file or at
// https://developers.google.com/open-source/licenses/bsd

syntax = "proto2";

package upb.test;

message TestMessage {
  optional int32 i32 = 1 [default = 5];
  repeated int32 r_i32 = 2;
  optional string str = 3 [default = "abc"];
  repeated string r_str = 4;
  optional TestMessage msg = 5;
  repeated TestMessage r_msg = 6;
}
