// Copyright 2023 gRPC authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

syntax = "proto3";

package grpc.testing;

message ResourceQuota {}

message FuzzingChannelArg {
    string key = 1;
    oneof value {
        string str = 2;
        int64 i = 3;
        ResourceQuota resource_quota = 4;
    }
}

message FuzzingChannelArgs {
    repeated FuzzingChannelArg args = 1;
}
